/***************************************************************************************************
 *
 *  CTT3. Original versions by VUC++. Version 3.0 by Zeropoint.
 *
 *  $CLASS    CTTScoreBoard
 *  $VERSION  1.00
 *  $AUTHOR   Daan 'Defrost' Scheerens  initial version
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/
class CTTScoreBoard extends UnrealCTFScoreBoard;

#exec texture import name=snowFlake1 file="Resources\SnowFlake1.pcx" mips=off flags=2
#exec texture import name=snowFlake2 file="Resources\SnowFlake2.pcx" mips=off flags=2
#exec texture import name=snowFlake3 file="Resources\SnowFlake3.pcx" mips=off flags=2
#exec texture import name=snowFlake4 file="Resources\SnowFlake4.pcx" mips=off flags=2
#exec texture import name=snowFlake5 file="Resources\SnowFlake5.pcx" mips=off flags=2
#exec texture import name=snowFlake6 file="Resources\SnowFlake6.pcx" mips=off flags=2
#exec texture import name=snowFlake7 file="Resources\SnowFlake7.pcx" mips=off flags=2
#exec texture import name=snowFlake8 file="Resources\SnowFlake8.pcx" mips=off flags=2
#exec texture import name=snowFlake9 file="Resources\SnowFlake9.pcx" mips=off flags=2
#exec texture import name=snowFlake10 file="Resources\SnowFlake10.pcx" mips=off flags=2
#exec texture import name=snowFlake11 file="Resources\SnowFlake11.pcx" mips=off flags=2
#exec texture import name=snowFlake12 file="Resources\SnowFlake12.pcx" mips=off flags=2
#exec texture import name=snowFlake13 file="Resources\SnowFlake13.pcx" mips=off flags=2
#exec texture import name=snowFlake14 file="Resources\SnowFlake14.pcx" mips=off flags=2
#exec texture import name=snowFlake15 file="Resources\SnowFlake15.pcx" mips=off flags=2
#exec texture import name=snowFlake16 file="Resources\SnowFlake16.pcx" mips=off flags=2

struct ParticleInfo {
	var int spriteNum;
	var float cx;
	var float cy;
	var float ct;
	var float waveFreq;
	var float waveAmplitude;
	var float dy;
	var float dx;
	var color col;
};

var color baseColor;
var bool bSnowInitialized;
var Texture sprites[16];
var ParticleInfo particles[100];
var float lastUpdateTime;

var float minDX;
var float maxDX;
var float minDY;
var float maxDY;
var float minWaveAmplitude;
var float maxWaveAmplitude;

// Non scaled constants.
const minWaveFreq = 0.25;
const maxWaveFreq = 1.0;
const minGlow = 0.40;
const maxGlow = 1.00;

// Scaled constants (set for a resolution of 1280x1024 px).
const scaleMinDX = -20.0;
const scaleMaxDX = 20.0;
const scaleMinDY = 100.0;
const scaleMaxDY = 300.0;
const scaleMinWaveAmplitude = 8;
const scaleMaxWaveAmplitude = 22;
const scaleWidth = 1280;
const scaleHeight = 1024;


function showScores(Canvas c) {
	super.showScores(c);
	renderSnow(c);
}

function showMiniScores(Canvas c) {
	super.showMiniScores(c);
	renderSnow(c);
}

simulated function renderSnow(Canvas c) {
	local int index;
	local Texture sprite;
	local float cx, cy;
	
	// Update position of each particle.
	updateSnow(c);
	
	// Draw each particle.
	c.style = ERenderStyle.STY_Translucent;
	for (index = 0; index < arrayCount(particles); index++) {
		// Set position.
		cx = particles[index].cx;
		cy = particles[index].cy;
		cx += sin(particles[index].ct * particles[index].waveFreq * 2 * pi) * particles[index].waveAmplitude;
		c.setPos(cx, cy);
		
		// Draw particle sprite.
		c.drawColor = particles[index].col;
		sprite = sprites[particles[index].spriteNum];
		c.drawTile(sprite, sprite.uSize, sprite.vSize, 0, 0, sprite.uSize, sprite.vSize);
	}
}

simulated function updateSnow(Canvas c) {
	local float deltaTime;
	local int index;
	
	// Prepare for update.
	setupScalars(c);
	if (!bSnowInitialized) {
		initializeSnow(c);
	}
	deltaTime = fMin(0.5, level.timeSeconds - lastUpdateTime);
	
	// Move each particle.
	for (index = 0; index < arrayCount(particles); index++) {
		particles[index].cx += particles[index].dx * deltaTime;
		particles[index].cy += particles[index].dy * deltaTime;
		particles[index].ct += deltaTime;
		
		// Check if particle has left the screen.
		if (particles[index].cy > c.clipY) {
			// It has, reset particle.
			initializeParticle(index, c, true);
		}
	}
	lastUpdateTime = level.timeSeconds;
}

simulated function initializeSnow(Canvas c) {
	local int index;
	bSnowInitialized = true;
	
	// Initialize each particle.
	for (index = 0; index < arrayCount(particles); index++) {
		initializeParticle(index, c);
	}
	lastUpdateTime = level.timeSeconds;
}

simulated function initializeParticle(int index, Canvas c, optional bool bReset) {
	particles[index].spriteNum = rand(arrayCount(sprites));
	particles[index].cx = fRand() * c.clipX;
	if (bReset) {
		particles[index].cy = -sprites[particles[index].spriteNum].vSize;
	} else {
		particles[index].cy = fRand() * c.clipY;
	}	
	particles[index].ct = 0.0;
	particles[index].dx = fRand() * (maxDX - minDX) + minDX;
	particles[index].dy = fRand() * (maxDY - minDY) + minDY;
	particles[index].waveFreq = fRand() * (maxWaveFreq - minWaveFreq) + minWaveFreq;
	particles[index].waveAmplitude = fRand() * (maxWaveAmplitude - minWaveAmplitude) + minWaveAmplitude;
	particles[index].waveFreq *= particles[index].dy / maxDY;
	particles[index].waveAmplitude *= particles[index].dy / maxDY;
	//if (31 Dec or 1 Jan) {
		particles[index].col = baseColor * (fRand() * (maxGlow - minGlow) + minGlow);
	//} else {
	//	particles[index].col.r = rand(256);
	//	particles[index].col.g = rand(256);
	//	particles[index].col.b = rand(256);
	//}
}

simulated function setupScalars(Canvas c) {
	minDX = scaleMinDX / scaleWidth * c.clipX;
	maxDX = scaleMaxDX / scaleWidth * c.clipX;
	minDY = scaleMinDY / scaleHeight * c.clipY;
	maxDY = scaleMaxDY / scaleHeight * c.clipY;
	minWaveAmplitude = scaleMinWaveAmplitude / scaleWidth * c.clipX;
	maxWaveAmplitude = scaleMaxWaveAmplitude / scaleWidth * c.clipX;
}

defaultproperties {
	baseColor=(r=255,g=255,b=255)
	sprites(0)=Texture'snowFlake1'
	sprites(1)=Texture'snowFlake2'
	sprites(2)=Texture'snowFlake3'
	sprites(3)=Texture'snowFlake4'
	sprites(4)=Texture'snowFlake5'
	sprites(5)=Texture'snowFlake6'
	sprites(6)=Texture'snowFlake7'
	sprites(7)=Texture'snowFlake8'
	sprites(8)=Texture'snowFlake9'
	sprites(9)=Texture'snowFlake10'
	sprites(10)=Texture'snowFlake11'
	sprites(11)=Texture'snowFlake12'
	sprites(12)=Texture'snowFlake13'
	sprites(13)=Texture'snowFlake14'
	sprites(14)=Texture'snowFlake15'
	sprites(15)=Texture'snowFlake16'
}