/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenControllerPawn
 *  $VERSION      1.00 (11-11-2007 19:33)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  The Nexgen Controller Pawn. This experimental class is used to detect events that
 *                can't be detected (in some cases) directly in the NexgenController class. An
 *                example of such an event is a leaving player when the game is paused. It is
 *                essential for the game to detect this event, otherwise the game might get stuck
 *                forever in a pause when there are no more match admins around to unpause the game.
 *
 **************************************************************************************************/
class NexgenControllerPawn extends Pawn;

var NexgenController control; // Server controller.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Pre-init process.
 *  $OVERRIDE
 *
 **************************************************************************************************/
event preBeginPlay() {
	// Check owner.
	if (owner == none || !owner.isA('NexgenController')) {
		destroy();
		return;
	}
	
	// Initialize controller pawn.
	control = NexgenController(owner);
    addPawn();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Post-init process.
 *  $OVERRIDE
 *
 **************************************************************************************************/
event postBeginPlay() {
	// Ignore.
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Post-net-init process.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated event postNetBeginPlay() {
	// Ignore.
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a pawn was killed.
 *  $PARAM        killer      The pawn that has killed the pawn.
 *  $PARAM        other       The pawn that was killed.
 *  $PARAM        damageType  Type of damage that killed the pawn.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function killed(pawn killer, pawn other, name damageType) {
	local NexgenClient client;
	
	// Check if player has left the game.
	if (killer == none && other != none && other.isA('PlayerPawn') && damageType == 'none' &&
	    other.inventory == none && other.weapon == none) {
		
		// He/she has, notify controller.
		client = control.getClient(other);
		if (client != none) {
			control.playerLeft(client);
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	fovAngle=0
	visibility=0
	sightRadius=-1000
	hearingThreshold=-1000
	health=0
	attitudeToPlayer=ATTITUDE_Ignore
	intelligence=BRAINS_NONE
	bIsPawn=false
	bIsPlayer=false
	remoteRole=ROLE_None
	bIsKillGoal=false
	bCollideActors=false
	bCollideWorld=false
	bBlockActors=false
	bBlockPlayers=false
	bProjTarget=false
	bRotateToDesired=false
	netPriority=0
	bHidden=true
	physics=PHYS_None
}