/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenAccountListBox
 *  $VERSION      1.00 (14-10-2007 22:45)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Offline user account listbox GUI component.
 *
 **************************************************************************************************/
class NexgenAccountListBox extends UWindowListBox;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Renders the specified listbox item.
 *  $PARAM        c     The canvas object on which the rendering will be performed.
 *  $PARAM        item  Item to render.
 *  $PARAM        x     Horizontal offset on the canvas.
 *  $PARAM        y     Vertical offset on the canvas.
 *  $PARAM        w     Width of the item that is to be rendered.
 *  $PARAM        h     Height of the item that is to be rendered.
 *  $REQUIRE      c != none && item != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function drawItem(Canvas c, UWindowList item, float x, float y, float w, float h) {
	local string text;
	
	if(NexgenAccountListItem(item).bSelected) {
		c.drawColor.r = 0;
		c.drawColor.g = 0;
		c.drawColor.b = 128;
		drawStretchedTexture(c, x, y, w, h - 1, Texture'WhiteTexture');
		c.drawColor.r = 255;
		c.drawColor.g = 255;
		c.drawColor.b = 255;
	} else {
		c.drawColor.r = 0;
		c.drawColor.g = 0;
		c.drawColor.b = 0;
	}
	
	c.font = root.fonts[F_Normal];
	
	text = "[" $ NexgenAccountListItem(item).title $ "] " $ NexgenAccountListItem(item).accountName;
	
	clipText(c, x + 2, y, text);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Retrieves the NexgenAccountListItem with the specified account number.
 *  $PARAM        accountNum  The account number of the NexgenAccountListItem that is to be found.
 *  $RETURN       The requested NexgenAccountListItem or none if no item with the specified account
 *                number was found.
 *
 **************************************************************************************************/
function NexgenAccountListItem getAccount(byte accountNum) {
	local NexgenAccountListItem item;

	// Search for player.
	for (item = NexgenAccountListItem(items); item != none; item = NexgenAccountListItem(item.next)) {
		if (item.accountNum == accountNum) {
			return item;
		}
	}
	
	// Player not found, return none.
	return none;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	listClass=class'NexgenAccountListItem'
	itemHeight=13
}
