/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenRCPServerSettings3
 *  $VERSION      1.02 (15-12-2007 15:40)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Nexgen about control panel page.
 *
 **************************************************************************************************/
class NexgenRCPServerSettings3 extends NexgenPanel;

var NexgenClientCore rpci;                        // Remote Procedure Call interface.

var NexgenSimpleListBox ignoredWeaponList;
var NexgenSimpleListBox hudReplacementList;
var UWindowSmallButton weapSaveButton;
var UWindowSmallButton weapRemButton;
var UWindowSmallButton hudReplSaveButton;
var UWindowSmallButton hudReplRemButton;
var UWindowCheckbox ignorePrimaryFireInp;
var UWindowCheckbox ignoreAltFireInp;
var NexgenEditControl weaponClassInp;
var NexgenEditControl originalHUDClassInp;
var NexgenEditControl replacementHUDClassInp;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Creates the contents of the panel.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function setContent() {	
	local NexgenContentPanel p;
	
	// Create layout & add components.
	createWindowRootRegion();
	
	// Ignored weapons.
	splitRegionH(88, defaultComponentDist);
	p = addContentPanel();
	p.splitRegionV(256, defaultComponentDist);
	ignoredWeaponList = NexgenSimpleListBox(p.addListBox(class'NexgenSimpleListBox'));
	p.divideRegionH(4);
	p.addLabel(client.lng.ignoredWeaponsTxt, true, TA_Center);
	p.splitRegionV(96);
	p.divideRegionV(2, 2 * defaultComponentDist);
	p.splitRegionV(192);
	p.addLabel(client.lng.weaponClassTxt);
	weaponClassInp = p.addEditBox();
	ignorePrimaryFireInp = p.addCheckBox(TA_Left, client.lng.ignorePrimaryFireTxt);
	ignoreAltFireInp = p.addCheckBox(TA_Left, client.lng.ignoreAltFireTxt);
	p.divideRegionV(2, defaultComponentDist);
	p.skipRegion();
	weapSaveButton = p.addButton(client.lng.saveTxt);
	weapRemButton = p.addButton(client.lng.removeTxt);
	
	// HUD replacement classes.
	splitRegionH(88, defaultComponentDist);
	p = addContentPanel();
	p.splitRegionV(256, defaultComponentDist);
	hudReplacementList = NexgenSimpleListBox(p.addListBox(class'NexgenSimpleListBox'));
	p.divideRegionH(4);
	p.addLabel(client.lng.hudReplaceClassesTxt, true, TA_Center);
	p.splitRegionV(128);
	p.splitRegionV(128);
	p.splitRegionV(192);
	p.addLabel(client.lng.originalHUDClassTxt);
	originalHUDClassInp = p.addEditBox();
	p.addLabel(client.lng.replacementHUDClassTxt);
	replacementHUDClassInp = p.addEditBox();
	p.divideRegionV(2, defaultComponentDist);
	p.skipRegion();
	hudReplSaveButton = p.addButton(client.lng.saveTxt);
	hudReplRemButton = p.addButton(client.lng.removeTxt);
	
	// Configure components.
	weaponClassInp.setMaxLength(64);
	originalHUDClassInp.setMaxLength(64);
	replacementHUDClassInp.setMaxLength(64);
	loadIgnoredWeaponList();
	loadHUDReplacementList();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads the list of weapons ignored by the spawn protector.
 *
 **************************************************************************************************/
function loadIgnoredWeaponList() {
	local int index;
	local NexgenSimpleListItem item;
	local string weaponClass;
	local string remaining;

	// Clear list.
	ignoredWeaponList.selectedItem = none;
	ignoredWeaponList.items.clear();
	
	// Add weapon classes.
	while (index < arrayCount(client.sConf.spawnProtectExcludeWeapons) &&
	       client.sConf.spawnProtectExcludeWeapons[index] != "") {
	       
		class'NexgenUtil'.static.split(client.sConf.spawnProtectExcludeWeapons[index], weaponClass, remaining);
		
		item = NexgenSimpleListItem(ignoredWeaponList.items.append(class'NexgenSimpleListItem'));
		item.displayText = weaponClass;
		item.itemID = index;
		
		index++;
	}
	
	// Add 'add new item' option if list isn't full.
	if (index < arrayCount(client.sConf.spawnProtectExcludeWeapons)) {
		item = NexgenSimpleListItem(ignoredWeaponList.items.insert(class'NexgenSimpleListItem'));
		item.displayText = client.lng.addNewItemTxt;
		item.itemID = -1;
	}
	
	// Signal item select event.
	weaponSelected();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads the HUD class replacement list.
 *
 **************************************************************************************************/
function loadHUDReplacementList() {
	local int index;
	local NexgenSimpleListItem item;
	
	// Clear list.
	hudReplacementList.selectedItem = none;
	hudReplacementList.items.clear();
	
	// Add weapon classes.
	while (index < arrayCount(client.sConf.replacementClass) &&
	       client.sConf.replacementClass[index] != "") {
		
		item = NexgenSimpleListItem(hudReplacementList.items.append(class'NexgenSimpleListItem'));
		item.displayText = client.sConf.replacementClass[index];
		item.itemID = index;
		
		index++;
	}
	
	// Add 'add new item' option if list isn't full.
	if (index < arrayCount(client.sConf.replacementClass)) {
		item = NexgenSimpleListItem(hudReplacementList.items.insert(class'NexgenSimpleListItem'));
		item.displayText = client.lng.addNewItemTxt;
		item.itemID = -1;
	}
	
	// Signal item select event.
	hudReplacementSelected();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when an item from the ignored weapon list was selected.
 *
 **************************************************************************************************/
function weaponSelected() {
	local NexgenSimpleListItem item;
	local string weaponClass;
	local string excludedModes;
	
	// Get selected item.
	item = NexgenSimpleListItem(ignoredWeaponList.selectedItem);
	
	// Update GUI.
	weapSaveButton.bDisabled = (item == none);
	weapRemButton.bDisabled = (item == none) || (item.itemID < 0);
	ignorePrimaryFireInp.bDisabled = (item == none);
	ignoreAltFireInp.bDisabled = (item == none);
	weaponClassInp.setDisabled((item == none));
	if (item == none) {
		ignorePrimaryFireInp.bChecked = false;
		ignoreAltFireInp.bChecked = false;
		weaponClassInp.setValue("");
	} else if (item.itemID < 0) {
		ignorePrimaryFireInp.bChecked = false;
		ignoreAltFireInp.bChecked = false;
		weaponClassInp.setValue("");
	} else {
		class'NexgenUtil'.static.split(client.sConf.spawnProtectExcludeWeapons[item.itemID],
		                               weaponClass, excludedModes);
		ignorePrimaryFireInp.bChecked = (instr(excludedModes, client.sConf.IW_Fire) >= 0);
		ignoreAltFireInp.bChecked = (instr(excludedModes, client.sConf.IW_AltFire) >= 0);
		weaponClassInp.setValue(weaponClass);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when an item from the hud replacement list was selected.
 *
 **************************************************************************************************/
function hudReplacementSelected() {
	local NexgenSimpleListItem item;
	local string str;

	// Get selected item.
	item = NexgenSimpleListItem(hudReplacementList.selectedItem);
	
	// Update GUI.
	hudReplSaveButton.bDisabled = (item == none);
	hudReplRemButton.bDisabled = (item == none) || (item.itemID < 0);
	originalHUDClassInp.setDisabled((item == none));
	replacementHUDClassInp.setDisabled((item == none));
	
	if (item == none) {
		originalHUDClassInp.setValue("");
		replacementHUDClassInp.setValue("");
	} else if (item.itemID < 0) {
		originalHUDClassInp.setValue("");
		replacementHUDClassInp.setValue("");
	} else {
		str = client.sConf.replacementClass[item.itemID];
		originalHUDClassInp.setValue(left(str, instr(str, "=")));
		replacementHUDClassInp.setValue(mid(str, instr(str, "=") + 1));
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Attemps to locate the RPC interface for this control panel.
 *  $REQUIRE      client != none
 *  $RETURN       True if the RPC interface has been set, false if not.
 *  $ENSURE       result == true ? rcpi != none : true
 *
 **************************************************************************************************/
function bool setRPCI() {
	
	// Check if RPC interface is already set.
	if (rpci == none) {
		// Attempt to get the RPC interface.
		rpci = NexgenClientCore(client.getController(class'NexgenClientCore'.default.ctrlID));
		return rpci != none;
		
	} else {
		// It is.
		return true;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies the dialog of an event (caused by user interaction with the interface).
 *  $PARAM        control    The control object where the event was triggered.
 *  $PARAM        eventType  Identifier for the type of event that has occurred.
 *  $REQUIRE      control != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function notify(UWindowDialogControl control, byte eventType) {	
	super.notify(control, eventType);
	
	// Weapon selected?
	if (control == ignoredWeaponList && eventType == DE_Click) {
		weaponSelected();
	}
	
	// HUD replacement selected?
	if (control == hudReplacementList && eventType == DE_Click) {
		hudReplacementSelected();
	}
	
	// Remove weapon selected?
	if (control == weapRemButton && eventType == DE_Click && !weapRemButton.bDisabled && setRPCI()) {
		rpci.delIgnoredWeapon(NexgenSimpleListItem(ignoredWeaponList.selectedItem).itemID);
	}

	// Save weapon selected?
	if (control == weapSaveButton && eventType == DE_Click && !weapSaveButton.bDisabled && setRPCI()) {
		rpci.saveIgnoredWeapon(NexgenSimpleListItem(ignoredWeaponList.selectedItem).itemID,
		                       class'NexgenUtil'.static.trim(weaponClassInp.getValue()),
		                       ignorePrimaryFireInp.bChecked, ignoreAltFireInp.bChecked);
	}
	
	// Remove HUD replacement selected?
	if (control == hudReplRemButton && eventType == DE_Click && !hudReplRemButton.bDisabled && setRPCI()) {
		rpci.delHUDReplacementClass(NexgenSimpleListItem(hudReplacementList.selectedItem).itemID);
	}
	
	// Save HUD replacement selected?
	if (control == hudReplSaveButton && eventType == DE_Click && !hudReplSaveButton.bDisabled && setRPCI()) {
		rpci.saveHUDReplacementClass(NexgenSimpleListItem(hudReplacementList.selectedItem).itemID,
		                             class'NexgenUtil'.static.trim(originalHUDClassInp.getValue()),
		                             class'NexgenUtil'.static.trim(replacementHUDClassInp.getValue()));
	}
	
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies this panel that the server configuration has been updated.
 *  $PARAM        configType  Type of settings that have been changed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function configChanged(byte configType) {
	
	// Relevant settings for this panel?
	if (configType == client.sConf.CT_ExclWeaponList) {
		loadIgnoredWeaponList();
	} else if (configType == client.sConf.CT_HUDReplacementList) {
		loadHUDReplacementList();
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	panelIdentifier="about"
}
