/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenClientLoginHandler
 *  $VERSION      1.00 (10-3-2008 17:59)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Default login handler for the Nexgen Server Controller.
 *
 **************************************************************************************************/
class NexgenClientLoginHandler extends Object abstract;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Retrieves the login parameters for the specified client.
 *  $PARAM        clientID      Unique client identifier GUID.
 *  $PARAM        loginOptions  Extra login parameters.
 *
 **************************************************************************************************/
static function getLoginParameters(NexgenClient client, out string clientID, out string loginOptions) {
	local string clientKey;
	local string password;
	
	// Load client key.
	clientKey = client.gc.get(client.SSTR_ClientKey);
	clientID = client.gc.get(client.SSTR_ClientID);
	if (clientKey == "") {
		// Client has no key. Create a new one.
		clientKey = class'NexgenUtil'.static.makeKey();
		clientID = class'MD5Hash'.static.MD5String(clientKey);
		client.gc.set(client.SSTR_ClientKey, clientKey);
		client.gc.set(client.SSTR_ClientID, clientID);
		client.gc.saveConfig();
		
	} else if (class'MD5Hash'.static.MD5String(clientKey) != clientID) {
		// KEY/ID mismatch. Reset key.
		clientKey = class'NexgenUtil'.static.makeKey();
		clientID = class'MD5Hash'.static.MD5String(clientKey);
		client.gc.set(client.SSTR_ClientKey, clientKey);
		client.gc.set(client.SSTR_ClientID, clientID);
		client.gc.saveConfig();
		
	}
	
	// Load login info.
	password = client.sc.get(client.serverID, client.SSTR_ServerPassword);
	
	// Set login options.
	if (password != "") class'NexgenUtil'.static.addProperty(loginOptions, client.SSTR_ServerPassword, password);
}

