/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenControllerPawn
 *  $VERSION      1.00 (11-11-2007 19:33)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  The Nexgen Controller Pawn. This experimental class is used to detect events that
 *                can't be detected (in some cases) directly in the NexgenController class. An
 *                example of such an event is a leaving player when the game is paused. It is
 *                essential for the game to detect this event, otherwise the game might get stuck
 *                forever in a pause when there are no more match admins around to unpause the game.
 *
 **************************************************************************************************/
class NexgenControllerPawn extends StationaryPawn;

var NexgenController control; // Server controller.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Pre-init process.
 *  $OVERRIDE
 *
 **************************************************************************************************/
event preBeginPlay() {
	// Check owner.
	if (owner == none || !owner.isA('NexgenController')) {
		destroy();
		return;
	}
	
	// Initialize controller pawn.
	control = NexgenController(owner);
    
    // Pawn stuff. Hopefully some of this will magically fix the 'AActor::TestCanSeeMe' crashes.
    addPawn();
    super(Actor).preBeginPlay();
    instigator = self;
    desiredRotation = rotation;
    sightCounter = 0.2 * fRand();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a pawn was killed.
 *  $PARAM        killer      The pawn that has killed the pawn.
 *  $PARAM        other       The pawn that was killed.
 *  $PARAM        damageType  Type of damage that killed the pawn.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function killed(pawn killer, pawn other, name damageType) {
	local NexgenClient client;
	
	// Check if player has left the game.
	if (killer == none && other != none && other.isA('PlayerPawn') && damageType == 'none' &&
	    other.inventory == none && other.weapon == none) {
		
		// He/she has, notify controller.
		client = control.getClient(other);
		if (client != none) {
			control.playerLeft(client);
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	health=100
    mesh=Flag1M
    skin=JFlag11
	remoteRole=ROLE_None
	bIsKillGoal=false
	bCollideActors=false
	bCollideWorld=false
	bBlockActors=false
	bBlockPlayers=false
	bProjTarget=false
	collisionHeight=64
	collisionRadius=32
	baseEyeHeight=48
	eyeHeight=48
	fatness=100
	sightRadius=128
}