/***************************************************************************************************
 *
 *  Nexgen statistics viewer by Zeropoint.
 *
 *  $CLASS        NSVMain
 *  $VERSION      1.03 (23-6-2008 22:50)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Nexgen Statistics Viewer plugin.
 *
 **************************************************************************************************/
class NSVMain extends NexgenPlugin;

var NSVLang lng;                        // Language localization support.
var NSVConfig conf;                     // Configuration.
var NSVReplicationInfo statsRI;         // Statistics replication info.

var int versionNum;                     // Plugin version number.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the plugin. Note that if this function returns false the plugin will
 *                be destroyed and is not to be used anywhere.
 *  $RETURN       True if the initialization succeeded, false if it failed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function bool initialize() {
	
	// Load localization support.
	lng = spawn(class'NSVLang', self);
	
	// Load settings.
	if (control.bUseExternalConfig) {
		conf = spawn(class'NSVConfigExt', self);
	} else {
		conf = spawn(class'NSVConfigSys', self);
	}
	conf.install();
	if (!conf.validate()) {
		control.nscLog(lng.invalidConfigMsg);
	}
	conf.initialize();
	
	// Load replication info.
	statsRI = spawn(class'NSVReplicationInfo', self);
	
	// Load UTStats client.
	if (conf.enableUTStatsClient && class'NexgenUtil'.static.trim(conf.utStatsHost) != "") {
		spawn(class'NSVUTStatsClient', self);
	}
	
	// Initialization complete.
	return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a new client has been created. Use this function to setup the new
 *                client with your own extensions (in order to support the plugin).
 *  $PARAM        client  The client that was just created.
 *  $REQUIRE      client != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function clientCreated(NexgenClient client) {
	local NSVClient xClient;
	
	xClient = NSVClient(client.addController(class'NSVClient', self));
	
	xClient.conf = conf;
	xClient.statsRI = statsRI;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies the clients that a certain part of the server configuration has changed.
 *  $PARAM        configType  Type of settings that have been changed.
 *  $ENSURE       new.conf.updateCounts[configType] = old.conf.updateCounts[configType] + 1
 *
 **************************************************************************************************/
function signalConfigUpdate(byte configType) {
	local NexgenClient client;
	local NSVClient xClient;
	local int index;
	
	// Set update counter.
	conf.updateCounts[configType]++;
	
	// Update checksum.
	conf.updateChecksum(configType);
	
	// Notify clients.
	for (client = control.clientList; client != none; client = client.nextClient) {
		xClient = NSVClient(client.getController(class'NSVClient'.default.ctrlID));
		if (xClient != none) {
			xClient.nexgenStatsViewerConfigChanged(configType, conf.updateCounts[configType], conf.dynamicChecksums[configType]);
		}
	}
	
	// Notify plugins.
	while (index < arrayCount(control.plugins) && control.plugins[index] != none) {
		control.plugins[index].notifyEvent(conf.EVENT_NexgenStatsViewerConfigChanged, string(configType));
		index++;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	pluginName="Nexgen stats viewer"
	pluginAuthor="Zeropoint"
	pluginVersion="1.00 build 1008"
	versionNum=100
}

