/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLFlag
 *  $VERSION  1.00
 *  $AUTHOR   Daan 'Defrost' Scheerens  original version
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/
 
// $CLASSIFIED

class SLFlag extends CTFFlag;



/***************************************************************************************************
 *
 *  $DESCRIPTION  The flag is currently dropped.
 *
 **************************************************************************************************/
state dropped {
	
	
	
	/***********************************************************************************************
	 *
	 *  $DESCRIPTION  This Actor is being touched by another one. Checks if the Actor is allowed to
	 *                touch this flag. If so super.touch() is called, if not, nothing will be done.
	 *  $PARAM        other  The Actor that is touching this actor.
	 *  $REQUIRE      other != none
	 *  $OVERRIDE
	 *
	 **********************************************************************************************/
	function touch(Actor other) {
		if (other.isA('Pawn') && mayTouchFlag(Pawn(other))) {
			super.touch(other);
		}
	}
	
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  The flag is home, at the base of it's team.
 *
 **************************************************************************************************/
auto state home {
	
	

	/***********************************************************************************************
	 *
	 *  $DESCRIPTION  This Actor is being touched by another one. Checks if the Actor is allowed to
	 *                touch this flag. If so super.touch() is called, if not, nothing will be done.
	 *  $PARAM        other  The Actor that is touching this actor.
	 *  $REQUIRE      other != none
	 *  $OVERRIDE
	 *
	 **********************************************************************************************/
	function touch(Actor other) {
		if (other.isA('Pawn') && mayTouchFlag(Pawn(other))) {
			super.touch(other);
		}
	}
	
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks if the given Pawn is allowed to touch this flag. It isn't if it is
 *                currently piloting a SLV rocket.
 *  $PARAM        other  The Pawn for which has to be checked if it allowed to touch the flag.
 *  $REQUIRE      other != none
 *  $RETURN       True if the Pawn is allowed to touch the flag, false if not.
 *
 **************************************************************************************************/
function bool mayTouchFlag(Pawn other) {
	local StrangeShell sl;
	
	// Check if currently piloting a rocket.
	foreach other.touchingActors(class'StrangeShell', sl) {
		if (sl.getPilot() == other) {
			// Gotcha! Can't touch it if you're piloting a rocket!
			return false;
		}
	}
	
	// Not piloting a rocket, so allowed to touch the flag.
	return true;
}

// $END_CLASSIFIED
