/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLFuelcore
 *  $VERSION  1.12
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLFuelcore extends SLAmmo;

var SLFuelpod pod;
var() float hoverDist;
var() float hoverRadius;
var() float hoverRate;
var() float hoverTimer;
var vector hoverLoc;
var bool bHover;
var bool bInAir;
var bool bRising;
var BlastMark bm;
var SLFuelglow glow;
var Class<SLFuelglow> glowClass;
var Sound casingSound;
var Sound riseSound;
var vector savedLoc;
var Rotator savedRot;
var float start;

function preBeginPlay() {
	savedLoc = location;
	savedRot = rotation;
	start = level.timeSeconds;
}

simulated function postBeginPlay() {
	local Actor a;
	local vector hl;
	local vector hn;
	local vector end;

	setRotation(rot(0, 0, 0));

	if (owner == none) {
		end = location;
		end.z -= hoverDist;
		a = trace(hl, hn, end, location, false);
		bInAir = a == none;
		if (!bInAir) {
			hoverLoc = hl + vect(0.0, 0.0, 1.0) * hoverDist;
			pod = spawn(class'SLFuelpod', self, , hl + hn, rotator(hn) + rot(-16384, 0, 0));
		}
	}
	
	configMaxAmmo = class'SLConfig'.default.maxFuelCores;
	maxAmmo = class'SLConfig'.default.maxFuelCores;
	
	super.postBeginPlay();
}

simulated function spawnGlow() {
	if (glow != none) {
		glow.destroy();
	}
	glow = spawn(glowClass, self);
}

auto state pickup {
	
	simulated function beginState() {
		if (owner == none) {
			
			if (bInAir) {
				loopAnim('hover');
				spawnGlow();
			} else {
				if (bHover) {
					stopHover();
				} else {
					landed(vect(0.0, 0.0, 1.0));
				}
				pod.touchCheck();
			}
		}
		super.beginState();
	}
  
	simulated function landed(vector hn) {
		drawScale = 0.001;
		setLocation(pod.location);
	}
  
	simulated function tick(float delta) {
		local float d;
  
		if (bRising) {
			d = hoverLoc.z - location.z;
			if (abs(d) < 5.0) {
				velocity = vect(0.0, 0.0, 0.0);
				setPhysics(PHYS_None);
				playSound(casingSound, SLOT_None, 0.85);
				bRising = false;
			} else {
				if (abs(d) < hoverDist * 0.5) {
					velocity = vect(0.0, 0.0, 1.0) * hoverRate * d / hoverDist * 0.5;
				}
			}
		}
		
		if (role < ROLE_Authority) {
			if (bHidden) {
				disable('tick');
				pickedUp();
        		gotoState('clientSleep');
			}
		} else {
			if (!bRising) {
				disable('tick');
			}
		}
	}
  
	simulated function animEnd() {
		if (animSequence == 'rise') {
			loopAnim('hover');
		}
	}

	simulated function startHover() {
		bHover = true;
		playAnim('rise');
		playSound(riseSound, SLOT_Misc, 1.0);
		velocity = vect(0.0, 0.0, 1.0) * hoverRate;
		setPhysics(PHYS_Projectile);
		bRising = true;
		enable('Tick');
		lightEffect = LE_FastWave;
		spawnGlow();
		drawScale = 1.0;
	}
  
	simulated function stopHover() {
		bHover = false;
		velocity = vect(0.0, 0.0, 0.0);
		disable('tick');
		
		if (mesh != none) {
			// ScriptWarning: (Function SLV202.SLFuelcore.Pickup.stopHover:0067) PlayAnim: No mesh
			playAnim('drop');
		}
		
		bCollideWorld = true;
		setPhysics(PHYS_Falling);
		lightEffect = LE_None;
		if (glow != none) {
			glow.fade();
		}
	}
  
	function takeDamage(int dam, Pawn inst, vector hl, vector mom, name dt) {
		if (bHover || bInAir) {
			instigator = inst;
			gotoState('explode');
		}
	}

begin:
	// Ignore.
  
}

state clientSleep {
	//ignores takeDamage;
	ignores touch, takeDamage;
  
	simulated function timer() {
		if (!bHidden) {
			gotoState('pickup');
		} else {
			setTimer(0.1, false);
		}
	}
  
	simulated function beginState() {
		setTimer(0.1, false);
	}

}

simulated function pickedUp() {	
	if (glow != none) {
		glow.destroy();
	}
	
	if (pod != none) {
		setLocation(pod.location);
		if (pod.bOpen) {
			pod.close();
		}
	}
}

simulated function startHover() {
	// Ignore.
}

simulated function stopHover() {
	// Ignore.
}

state sleeping {
	ignores touch, takeDamage;

	function beginState() {
		pickedUp();
		super.beginState();
	}
  
begin:
	sleep(respawnTime);
	playSound(respawnSound);
	sleep(0.3);
	gotoState('pickup');
}

state explode {
	//ignores takeDamage;
	ignores touch, takeDamage;

begin:
	bHidden = true;
	spawn(class'StrangeExpl', , , location);
	hurtRadius(165.0, 250.0, 'exploded', 140000.0, location);
	sleep(0.05);
	setRespawn();
}

function inventory spawnCopy(Pawn p) {
	local Inventory copy;
	local StrangeShell sl;

	copy = super.spawnCopy(p);
	foreach p.touchingActors(class'StrangeShell', sl) {
		if (sl.getPilot() == p) {
			sl.fc = SLFuelcore(copy);
		}
	}
	return copy;
}

simulated function destroyed() {
	if (glow != none) {
		glow.destroy();
	}

	if (pod != none) {
		pod.destroy();
		// Well this is a bit of a lame fix, but i'm unable to trace the source of the problem. At
		// least this works.
		// $FIX(SLV203)  Fuel pods sometimes disappear after a while.
		//if (role == ROLE_Authority) {
		// Oh a fix for a fix, how ugly! Should find another way in SLV204.
		if (role == ROLE_Authority && start < level.timeSeconds) {
			spawnSuccessor();
		}
	}
	
	super.destroyed();
}

function spawnSuccessor() {
	spawn(self.class, self.owner, self.tag, savedLoc, savedRot);
}
// $END_CLASSIFIED

defaultproperties {
	hoverDist=60.00
	hoverRadius=800.00
	hoverRate=180.00
	hoverTimer=1.00
	glowClass=Class'SLFuelglow'
	casingSound=Sound'SLV2Sounds.fuelcore.casing'
	riseSound=Sound'Botpack.ASMD.Vapour'
	configMaxAmmo=5
	PickupMessage="You picked up a fuel core."
	ItemName="SLFuelcore"
	PickupViewMesh=SkeletalMesh'SLV2Models.fcore'
	MaxDesireability=0.32
	PickupSound=Sound'SLV2Sounds.fuelcore.Pickup'
	Physics=PHYS_None
	Mesh=SkeletalMesh'SLV2Models.fcore'
	SoundRadius=64
	CollisionRadius=20.00
	CollisionHeight=6.00
	bProjTarget=True
	LightBrightness=255
	LightRadius=6
	RotationRate=(Pitch=0,Yaw=0,Roll=0),
}
