/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    StrangeMutator
 *  $VERSION  1.62
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *  $CHANGES
 *    2007.04.07  Def    -  Removed BETA expiration code.
 *
 **************************************************************************************************/
class StrangeMutator extends SLMutator;

var SLConfig conf;

var() string yjStr;
var() string yjAmmoStr;
var() int randSpawnDist;

function preBeginPlay() {
	conf = spawn(class'SLConfig');
	super.preBeginPlay();
}

function postBeginPlay() {
	if (conf.useKillMsgFix) {
		addMutator(spawn(class'SLKillMsgFix'));
	}
	if (conf.pilotCarryMode == CM_Normal) {
		spawn(class'SLFlagSN');
	}
	spawn(class'SLProjectileSN');
	setTimer(0.1, false);
	super.postBeginPlay();
}

function timer() {
	if (conf.randomSLVPlacement) {
		spawnRandSL(randSpawnDist);
	}
}

function spawnRandSL(int dist) {
	local FlagBase fb;
	local PathNode pn;
	local int Num;
	local int spawnindex;

	foreach allActors(class'FlagBase', fb) {
		num = 0;
		foreach radiusActors(class'PathNode', pn, dist, fb.location) {
			num++;
		}
		
		spawnindex = rand(num);
		num = 0;
		foreach radiusActors(class'PathNode', pn, dist, fb.location) {
			if (spawnindex == num) {
				spawn(class'SLKonglauncher', , , pn.location);
				break;
			}
			num++;
		}
	}
}

function modifyPlayer(Pawn o) {
	local SLFuelCore fc;
	
	if (level.game.isA('DeathMatchPlus') && !conf.noYJ) {
		deathMatchPlus(level.game).giveWeapon(o, string(class'SLUtil'.default.package) $ "." $ yjStr);
	}
	
	if (conf.initialFuelCores > 0) {
		fc = spawn(class'SLFuelCore', o);
		fc.giveTo(o);
		fc.ammoAmount = conf.initialFuelCores;
	}
	
	super.modifyPlayer(o);
}

function bool alwaysKeep(Actor o) {
	if (conf.giveMaxAmmo && o.isA('SLWeapon')) {
		SLWeapon(o).maxOut();
	}
	
	return super.alwaysKeep(o);
}

function bool checkReplacement(Actor o, out byte bSuperRelevant) {
	if (!conf.noYJ) {
		if (o.isA('enforcer')) {
			if (o.location != vect(0.0, 0.0, 0.0)) {
				replaceWith(o, string(class'SLUtil'.default.package) $ "." $ yjStr);
			}
			return false;
		}
	}

	if (o.isA('WarheadLauncher')) {
		replaceWith(o, string(class'SLUtil'.default.package) $ "." $ "SLKonglauncher");
		return false;
	}
	
	if (conf.replaceVials && o.isA('HealthVial') || conf.replacePacks && o.isA('MedBox')) {
		replaceWith(o, string(class'SLUtil'.default.package) $ "." $ "SLFuelcore");
		return false;
	}

	return true;
}

function bool replaceWith(Actor other, string aClassName) {
	local Actor a;
	local class<Actor> aClass;

	if (other.isA('Inventory') && (other.location == vect(0.0, 0.0, 0.0))) {
		return false;
	}
  
	aClass = class<Actor>(dynamicLoadObject(aClassName, class'Class'));
	if (aClass != none) {
		a = spawn(aClass, , other.tag, other.location, other.rotation);
	}
	
	if (other.isA('Inventory')) {
		if (Inventory(other).myMarker != none) {
			Inventory(other).myMarker.markedItem = Inventory(a);
			if (Inventory(a) != none) {
				Inventory(a).myMarker = Inventory(other).myMarker;
				a.setLocation(A.location + (a.collisionHeight - other.collisionHeight) * vect(0.0, 0.0, 1.0));
			}
			Inventory(other).myMarker = none;
		} else if (a.isA('Inventory')) {
			Inventory(a).bHeldItem = true;
			Inventory(a).respawnTime = 0.0;
		}
	}
  
	if (a != none) {
		a.event = other.event;
		a.tag = other.tag;
		a.rotationRate = other.rotationRate;
		return true;
	}
	
	return false;
}
// $END_CLASSIFIED

defaultproperties {
	yjStr="SLSidearm"
	yjAmmoStr="SLSidearmAmmo"
	randSpawnDist=2000
}
