/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLBotBrain
 *  $VERSION  1.13
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLBotBrain extends SLInfo;

var StrangeShell sl;
var Bot bot;
var bool bIsPilot;
var bool bGuiding;
var bool bEject;
var byte goal;
var SLSkyNode targetNode;
var SLSkyNode lastNode;
var SLSkyNode launchNode;
var bool bNoExpire;
var bool bReachEject;
var float targetAcq;
var() float targetExpireTime;
var() int ejectDist;
var() int ejectDistFlag;
var() float randomLinkAdjust;
var() float wallDeter;
var() float shotDeter;
var() float skill;
var bool bHunting;
var Actor nukeTarget;
var SLSkyNet skynet;
var float suggestedSpeed;
var bool bBotsObeyOrders;

function postBeginPlay() {
	goal = -1;
}

function spawnSkyNet() {
	foreach allActors(class'SLSkyNet', skynet) {
		break;
	}
	
	if (skynet == none) {
		skynet = spawn(class'SLSkyNet');
	}
}

function setBot(Bot b, StrangeShell s, optional SLSkyNet net, optional SLSkyNode ln) {
	bot = b;
	sl = s;
	launchNode = ln;
	targetNode = ln;
	skill = b.skill / 3.0 * 0.2 + 0.8;
	
	bIsPilot = b == sl.getPilot();
	if (bIsPilot) {
		bGuiding = true;
		skynet = net;
		spawnSkyNet();
	} else {
		gotoState('gunnerBoarding');
	}
	
	setTimer(fmax(1.0 * (1.0 - skill), 0.01), true);
	
	setOwner(bot);
}

function timer() {
	if (sl.fuel < 3 && (sl.fc == none ||  !(sl.fc.getAmmo() > 0))) {
		if (bIsPilot) {
			sl.throttle = 1.0;
		}
		
		bEject = true;
		
	} else {
		if (bIsPilot) {
			pilotBehavior();
		} else {
			gunnerBehavior();
		}
	}
	
	if (bEject) {
		eject();
	}
}

function pilotBehavior() {
	local CTFFlag flag;
	local SLSkyNode temp;
	
	if (setGoal()) {
		if (skynet.bCTF) {
			flag = myFlag();
			if (flag != none) {
				if (bot.moveTarget == myFlag()) {
					nukeTarget = bot.moveTarget;
				} else if (bot.enemy == myFlag().holder) {
					sl.arm();
					sl.throttle = 0.1;
					nukeTarget = bot.enemy;
				} else {
					nukeTarget = none;
				}
			}
		}

		if (nukeTarget != none) {
			targetActor(nukeTarget);
		} else {
			if (targetNode == none) {
				temp = closestNode(skynet.linkRadius);
				if (temp != lastNode) {
					setTargetNode(temp);
				} else {
					bEject = true;
				}
			}
			
			if (bReachEject && targetNode != none) {
				if (inrange(targetNode, 0.5)) {
					sl.disarm();
					bEject = true;
				}
			} else {
				// Ignore.
			}
			
			if (targetNode != none && !inrange(targetNode, 2.0) && linedup(targetNode, 0.01)) {
				if (suggestedSpeed == -1) {
					sl.throttle = 0.5 + 0.5 * skill;
				}
				
				if (bot.enemy != none) {
					if (!sl.bShootMode) {
						sl.goShootMode();
					}
				}

			} else {
				if (suggestedSpeed == -1) {
					sl.throttle = 0.1;
				}
	
				if (sl.bShootMode) {
					sl.goShootMode();
				}
			}

			if (suggestedSpeed >= 0) {
				if (suggestedSpeed < 1.0) {
					sl.throttle = suggestedSpeed;
				} else {
					if (abs(vsize(sl.velocity) - suggestedSpeed) < 50.0) {
						// Hold current speed.
					} else {
						if (vsize(sl.velocity) < suggestedSpeed) {
							sl.throttle += 0.1;
						} else {
							sl.throttle -= 0.1;
						}
						sl.throttle = fclamp(sl.throttle, 0.0, 1.0);
					}
				}
			}
				
			if (targetNode != none && !bReachEject) {
				temp = skynet.goalNodes[goal].randomLink();
				if (temp != none && los(temp)) {
					setTargetNode(temp);
					bReachEject = true;
					bNoExpire = true;
				}
			}
			
			if (!bNoExpire && targetNodeExpired()) {
				setTargetNode(nextNode(targetNode, true));
			}
			
			if (targetNode != none) {
				targetActor(targetNode);
			}
		}
	}
}

function CTFFlag myFlag() {
	if (skynet.bCTF) {
		return CTFReplicationInfo(level.game.gameReplicationInfo).flagList[bot.playerReplicationInfo.team];
	}
	
	return none;
}

function bool enemyFlag(CTFFlag flag) {
	return flag != myFlag();
}

function float distto(Actor a) {
	return vsize(a.location - bot.location);
}

function bool inrange(Actor a, float range) {
	if (range < 10.0) {
		return distto(a) < range * skynet.linkRadius;
	} else {
		return distto(a) < range;
	}
}

function bool los(Actor a) {
	local bool l;

	l = fastTrace(a.location, bot.location);
	
	return l;
}

function bool linedup(Actor a, optional float error) {
	return (1.0 - (normal(sl.velocity) dot normal(a.location - bot.location))) <= error;
}

function targetActor(Actor a) {
	desiredRotation = adjustRot(rotator(a.location - bot.location));
}

function targetLoc(vector l) {
	desiredRotation = adjustRot(rotator(l - bot.location));
}

function bool setGoal() {
	local byte oldg;

	oldg = goal;
	if (skynet.getGoal(bot, goal)) {
		return true;
	}
	
	bEject = true;
	return false;
}

function deathBy(name type) {
	local SLSkyNode n;
	local byte team;

	if (lastNode != none && distto(lastNode) > distto(targetNode)) {
		n = targetNode;
	} else {
    	n = lastNode;
	}

	team = bot.playerReplicationInfo.team;

	switch (name) {
    case 'shot':
		n.shotDeaths[team]++;
		break;
    
    case 'HitWall':
		n.wallDeaths++;
		break;
	}
}

function bool targetNodeExpired() {
	return level.timeSeconds - targetAcq > targetExpireTime;
}

function setTargetNode(SLSkyNode sn) {
	lastNode = targetNode;
	
	targetNode = sn;
	
	if (targetNode != none) {
		targetAcq = level.timeSeconds;
		suggestedSpeed = sn.suggestedSpeed;
		
		if (skynet.bShowLinks) {
			if (lastNode != none) {
				lastNode.vis(false);
			}
			targetNode.vis(true, goal);
		}
		
		targetNode.passes++;
		targetNode.teamPasses[bot.playerReplicationInfo.team]++;
	}
}

function SLSkyNode closestNode(float r) {
	return skynet.closestNode(bot.location, r);
}

function nodeReached(SLSkyNode sn) {
	if (sn == targetNode) {
		
		if (sn != launchNode) {
			launchNode.addLaunch(true);
		}
		
		setTargetNode(nextNode(sn, true));
	}
}

function SLSkyNode nextNode(SLSkyNode cur, optional bool random, optional bool bNextNext) {
	local float rla;
	local int i;
	local SLSkyNode next;
	local SLSkyNode nn;
	local float w;
	local float curw;

	if (randomLinkAdjust > 0) {
		rla = randomLinkAdjust;
	} else {
		rla = skynet.randomLinkAdjust;
	}
	
	for (i = 0; i < arrayCount(cur.links); i++) {
		if (bNextNext && !los(cur.links[i])) {
			continue;
		}
		
		if (cur.links[i] == lastNode) {
			continue;
		}
		
		curw = cur.links[i].weight[goal];
		
		if (random) {
			curw += frand() * rla;
		}

		if (curw > w || next == none) {
			next = cur.links[i];
			w = curw;
		}
	}
  
	if (next == none || w < cur.Weight[goal] || avoidNode(next, bot.playerReplicationInfo.team)) {
		if (!bNextNext) {
			bEject = true;
		}
	} else {
		// Ignore.
	}
	
	return next;
}

function bool avoidNode(SLSkyNode n, byte team) {
	local bool avoid;

	avoid = (n.wallDeaths / n.passes > wallDeter) || (n.shotDeaths[team] / n.teamPasses[team] > shotDeter);
	if (avoid) {
		sl.disarm();
	}
	return avoid;
}

function FlagBase getFlag() {
	return CTFReplicationInfo(level.game.gameReplicationInfo).flagList[bot.playerReplicationInfo.team].homeBase;
}

function tick(float delta) {
	if (sl != none) {
		setRotation(rotation + sturn(desiredRotation, rotation, int(delta * skill * rotationRate.yaw)));
	}
}

function rotator adjustRot(rotator r) {
	return r - sturn(rotator(sl.velocity), r, 0, 16384);
}

function rotator sturn(rotator newr, rotator oldr, optional int max, optional int limit) {
	local int dyaw, dpitch;
	local rotator turn;

	dyaw = (newr.yaw & 65535) - (oldr.yaw & 65535);
	dpitch = (newr.pitch & 65535) - (oldr.pitch & 65535);

	if (limit > 0) {
		if (abs(dyaw) > limit) {
			dyaw = 0;
		}
		if (abs(dpitch) > limit) {
			dpitch = 0;
		}
	}

	if (dyaw < -32768) {
		dyaw += 65536;
	} else if (dyaw > 32768) {
		dyaw -= 65536;
	}

	if (dpitch < -32768) {
		dpitch += 65536;
	} else if (dpitch > 32768) {
		dpitch -= 65536;
	}

	if (max > 0) {
		dyaw = fmin(dyaw, max);
		dpitch = fmin(dpitch, max);
	}

	turn.yaw = dyaw;
	turn.pitch = dpitch;
	turn.roll = 0;

	return turn;
}

function rotator getGuidedRotation() {
	return rotation;
}

function gunnerBehavior() {
	bot.bCanFly = true;
	bot.bCanWalk = false;
	bot.bAdvancedTactics = false;

	if (sl.getPilot() == none && !isInState('GunnerEject')) {
		gotoState('GunnerEject');
	}
	
	if (sl.fastTrace(sl.location + vect(0.0, 0.0, -1.0) * 360)) {
		// Ingore.
	} else if (skynet.bCTF && bot.playerReplicationInfo.hasFlag != none && inrange(getFlag(), ejectDistFlag)) {
		eject();
	} else if (bot.moveTarget != none && bot.moveTarget != sl.getPilot() && inrange(bot.moveTarget, ejectDist)) {
		if ((bBotsObeyOrders && !(bot.orders == 'Follow' && bot.orderObject == sl.getPilot())) || ((bot.moveTarget.isA('Weapon') || bot.moveTarget.isA('Ammo')) && bot.weapon != none && (bot.weapon.bMeleeWeapon || bot.weapon.ammoType.ammoAmount == 0))) {
			eject();
		}
	}
}

state gunnerBoarding {
	ignores gunnerBehavior;

begin:
	sleep(3.0);
	gotoState('');
}

state gunnerEject {
	ignores gunnerBehavior;

begin:
	sleep(1.5);
	if (sl.gunner != none) {
		eject();
	}
}

function eject() {
	sl.eject(bot);
	bot.whatToDoNext('', '');
}

function destroyed() {
	if (skynet != none) {
		if (skynet.bShowLinks) {
			if (targetNode != none) {
				targetNode.vis(false);
			}
		}
	}
	
	super.destroyed();
}
// $END_CLASSIFIED

defaultproperties {
	targetExpireTime=3.00
	ejectDist=1000
	ejectDistFlag=2000
	wallDeter=0.50
	shotDeter=0.50
	skill=0.50
	rotationRate=(Pitch=10000,Yaw=10000,Roll=0)
	bBotsObeyOrders=true
}
