/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLContrail
 *  $VERSION  1.61
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLContrail extends Effects;

var() float intro;
var() float length;
var bool bRoot;
var StrangeShell sl;
var SLContrail Last;
var float elapsed;
var() float life;
var() bool bDDStart;
var() bool bDDRestart;
var() bool bDDEnd;
var() bool bDDShort;
var() float rotRate;
var bool bDDShorted;
var bool bDDEnded;
var bool bEnd;
var bool bDead;
var() Texture endTex;
var PlayerPawn vp;

simulated function postBeginPlay() {
	if (owner != none) {
		if (owner.isA('StrangeShell')) {
			bRoot = true;
			bHidden = true;
			sl = StrangeShell(owner);
		}
	}
	
	life = class'SLConfig'.default.contrailLifeTime;
}

simulated function end() {
	if (last != none && !last.bEnd) {
		last.setEndTexture(true);
	}
	
	if (sl != none) {
		sl.rootct = none;
	}
	bDead = true;
}

simulated function roottick(float d) {
	local Pawn slowner;
	local Vector x;
	local Vector y;
	local Vector z;
	local Vector newloc;
	local Rotator newRot;
	local SLContrail prev;
	local PlayerPawn pp;

	if (last != none && last.bDead) {
		destroy();
		return;
	}
	
	if (bDDEnded) {
		if (!bDDRestart) {
			end();
			return;
		}
	}
	
	if (level.bDropDetail && bDDEnd) {
		bDDEnded = True;
		return;
	}
	
	if (class'SLConfig'.default.contrailViewAlign) {
		viewAlignChain(getViewPort());
	} else if (class'SLConfig'.default.contrailRotate) {
		rotateChain(d);
	}
	
	if (bDead) {
		return;
	}
	
	if (last == none || bDDEnded) {
		if (last == none && sl != none) {
			slowner = sl.getPilot();
		}
		
		if (bDDStart && !bDDRestart && level.bDropDetail) {
			end();
			return;
		}
		
		bDDEnded = false;
		
		if (last != none) {
			prev = last;
		}

		last = spawn(class'SLContrail', slowner, , , rotator(sl.velocity));
		getAxes(sl.rotation, x, y, z);
		last.setLocation(sl.location + last.length * x);
		last.setEndTexture(false);
		setTimes(last);
		if (prev != none) {
			last.last = prev;
		}
		
	} else {
		
		if (vsize(sl.location - last.location) > last.length * 2) {
			prev = last;
			getAxes(last.rotation, x, y, z);
			newloc = last.location + last.length * x;
			newrot = rotator(sl.location - newloc);
			
			if (!class'SLConfig'.default.contrailRotate && !class'SLConfig'.default.contrailViewAlign) {
				newRot.roll = sl.rotation.roll;
			}
			
			last.bOwnerNoSee = false;
			last = spawn(class'SLContrail', self, , newloc, newRot);
			last.last = prev;
			setTimes(last);
		}
	
	}
}

simulated function rotateChain (float d) {
	local SLContrail next;
	local rotator r;

	next = last;
	while (next != none) {
		r = next.rotation;
		r.roll += rotRate * d;
		next.setRotation(r);
		next = next.last;
	}
}

simulated function float arccos(float a) {
	return atan(sqrt(1.0 - a * a) / a);
}

simulated function viewAlignChain(PlayerPawn vp) {
	local SLContrail next;
	local rotator r;
	local vector v;
	local float uupr, a;

	if (vp == none) {
		return;
	}

	uupr = 65536.0 / (2.0 * 3.14);

	next = last;
	while (next != none) {
		r = next.rotation;
		a = r.yaw / uupr;
		v = vp.location - next.location;
		v.x = v.x * cos(a) - v.y * sin(a);
		v.y = v.y * cos(a) + v.x * sin(a);
		r.roll = atan(v.z / v.y) * (65536.0 / (2.0 * 3.14)) + 16384;
		next.setRotation(r);
		next = next.last;
	}
}

simulated function PlayerPawn getViewPort() {
	local PlayerPawn pp;

	if (vp == none || !vp.player.isA('Viewport')) {
		vp = none;
		foreach allActors(class'PlayerPawn', pp) {
			if (pp.player.isA('Viewport')) {
				vp = pp;
				break;
			}
		}
	}
	
	return vp;
}

simulated function setTimes(SLContrail c) {
	c.life = life;
	c.elapsed = life;
	c.Intro = 600.0 / vsize(sl.velocity) * default.intro;
	c.bDDShorted = bDDShorted;
}

simulated function short() {
	life /= 4.0;
	elapsed /= 4.0;
	intro /= 2.0;
	bDDShorted = true;
}

simulated function tick(float d) {
	if (bDDShort && !bDDShorted && level.bDropDetail) {
		short();
	}
	
	if (bRoot) {
		roottick(d);
		return;
	}
	
	elapsed -= d;
	if (elapsed <= 0.0) {
		destroy();
		return;
	}
	
	if (elapsed > life - intro) {
		scaleGlow = (life - elapsed) / intro;
	} else {
		scaleGlow = elapsed / (life - intro);
	}
}

simulated function setEndTexture(bool reverse) {
	local vector x;
	local vector y;
	local vector z;
	local rotator newRot;

	bEnd = true;
	skin = endTex;
	if (reverse) {
    	getAxes(rotation, x, y, z);
		setLocation(location + x * length);
		newRot = rotation;
		newRot.pitch = rotation.Pitch + 65535 * 0.5;
		setRotation(newrot);
	}
}

simulated function destroyed() {
	bDead = True;
}
// $END_CLASSIFIED

defaultproperties {
    intro=2.0
    length=100.0
    life=20.0
    bDDShort=True
    rotRate=5000.0
    endTex=Texture'SLV2Models.Skins.contrailEnd'
    bOwnerNoSee=True
    RemoteRole=ROLE_None
    drawType=DT_Mesh
    style=STY_Translucent
    mesh=SkeletalMesh'SLV2Models.contrailM'
    scaleGlow=0.00
    bUnlit=True
    lightBrightness=255
    lightRadius=1
}
