/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLEjectedBrass
 *  $VERSION  1.30
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLEjectedBrass extends UT_ShellCase;

var() bool bLevel;
var() Sound landSound;
var() float sinktime;
var() float sinkfudge;
var() int maxBounces;
var() float noBounceChance;
var() bool bRandHitSound;
var() Sound hitSounds[10];
var() int num;

simulated function postBeginPlay() {
	if (level.bDropDetail && level.netMode != NM_DedicatedServer && level.netMode != NM_ListenServer) {
		lifeSpan = 1.5;
	}
}

simulated function playHitSound() {
	local Sound s;

	if (bRandHitSound) {
		s = hitSounds[rand(num)];
	} else {
		S = landSound;
	}
	
	playSound(s, SLOT_Misc, 1.0);
}

simulated function hitWall (vector hn, Actor wall) {
	local vector realhn;

	if (level.bDropDetail) {
		destroy();
	} else {
		if (bHasBounced && (numBounces > maxBounces || frand() < noBounceChance)) {
			bBounce = false;
		}
		
		numBounces++;
		if (!region.zone.bWaterZone ) {
			playHitSound();
		}
		
		realhn = hn;
		hn = normal(hn + 0.41 * vrand());
		if ((hn dot realhn) < 0) {
			hn *= -0.5;
		}
		
		velocity = 0.5 * (velocity - 2 * hn * (velocity dot hn));
		randSpin(spinRate());
		bHasBounced = true;
	}
}

simulated function float spinRate() {
	return rand(100000) + 100000;
}

simulated function landed(vector hn) {
	local rotator r;

	if (level.bDropDetail || numBounces > 3) {
		destroy();
	} else {
		if (!region.zone.bWaterZone && (landSound != none)) {
			playHitSound();
		}
		
		setPhysics(PHYS_None);
		
		if (bLevel) {
			r = rotation;
			r.pitch = 0;
			r.roll = 0;
			SetRotation(r);
		}

		if (lifeSpan > sinktime) {
			setTimer(lifeSpan - sinktime, false);
		} else {
			gotoState('Sinkout');
		}
	}
}

simulated function timer() {
	gotoState('Sinkout');
}

state sinkout {

	simulated function beginState() {
		local float vish;
		local vector v;
  
		if (level.bDropDetail) {
			destroy();
			return;
		}
		
		rotationRate = rot(0,0,0);
		vish = (collisionHeight + sinkfudge) * drawScale;
		v = location;
		v.z += vish;
		setLocation(v);
		v = vect(0.0, 0.0, 0.0);
		v.z = -vish;
		prePivot = v;
		bCollideWorld = false;
		setPhysics(PHYS_Projectile);
		v = vect(0.0, 0.0, 0.0);
		v.z -= vish / lifeSpan;
		velocity = v;
	}
  
}
// $END_CLASSIFIED

defaultproperties {
	bLevel=True
	landSound=Sound'SLV2Sounds.Weapon.brasshit1'
	sinktime=2.00
	sinkfudge=5.00
	maxBounces=3
	noBounceChance=0.50
	bRandHitSound=True
	HitSounds(0)=Sound'SLV2Sounds.Weapon.brasshit1'
	HitSounds(1)=Sound'SLV2Sounds.Weapon.brasshit2'
	HitSounds(2)=Sound'SLV2Sounds.Weapon.brasshit3'
	Num=3
	LifeSpan=6.00
	Mesh=SkeletalMesh'SLV2Models.brass'
	LightType=LT_None
}
