/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLFontLib
 *  $VERSION  1.20
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLFontlib extends FontInfo;

enum FStyle {
	FS_Normal,
	FS_Slick,
	FS_Title
};

var() name fnames[6];
var() FStyle fstyles[6];
var int currindex;
var FStyle currstyle;

static function Font GetStaticHugeFont(float width) {
	return Font(dynamicLoadObject("SLV2Fonts.title18", class'Font'));
}

static function Font GetStaticBigFont(float width) {
	return Font(dynamicLoadObject("SLV2Fonts.leg16", class'Font'));
}

static function Font GetStaticMediumFont(float width) {
	return Font(dynamicLoadObject("SLV2Fonts.leg16", class'Font'));
}

static function Font GetStaticSmallFont(float width) {
	return Font(dynamicLoadObject("SLV2Fonts.leg8", class'Font'));
}

static function Font GetStaticSmallestFont(float width) {
	return Font(dynamicLoadObject("SLV2Fonts.leg8", class'Font'));
}

static function Font GetStaticAReallySmallFont(float width) {
	return Font(dynamicLoadObject("SLV2Fonts.leg8", class'Font'));
}

static function Font GetStaticACompletelyUnreadableFont(float width) {
	return Font(dynamicLoadObject("SLV2Fonts.leg8", class'Font'));
}

static function Font getFont(optional FStyle style, optional bool big) {
	local string name;

	switch (Style) {
		case FS_Slick:
			if (big) {
				name = "techno16";
			} else {
				name = "techno8";
			}
			break;
			
		case FS_Title:
			if (big) {
				name = "title18";
			} else {
				name = "title9";
			}
			break;

		default:
			if (big) {
				name = "leg16";
			} else {
				name = "leg8";
			}
	}
	return Font(dynamicLoadObject("SLV2Fonts." $ name, class'Font'));
}

function Font getNextFont(FStyle style, optional bool reset) {
	local int i;

	if (reset || style != currstyle) {
		currindex = 0;
		currstyle = style;
	}
	
	while (currindex < 6) {
		if (fstyles[i] == currstyle) {
			return Font(dynamicLoadObject("SLV2Fonts." $ string(fnames[i]), class'Font'));
		}
		currindex++;
	}
	
	currindex = 0;
	return none;
}

function Font fitFont(Canvas c, string s, out float w, out float h, FStyle style) {
	local Font font;
	local Font last;
	local float xl;
	local float yl;

	font = getNextFont(style);
	while (font != none) {
		c.font = font;
		c.textSize(s, xl, yl);
		if (xl <= w && yl <= h) {
			w = xl;
			h = yl;
			return font;
		}
		
		last = font;
		font = getNextFont(style);
	}
	return last;
}
// $END_CLASSIFIED

defaultproperties {
    fnames(0)=leg16
    fnames(1)=leg8
    fnames(2)=techno16
    fnames(3)=techno8
    fnames(4)=title18
    fnames(5)=title9
    fstyles(2)=FS_Slick
    fstyles(3)=FS_Slick
    fstyles(4)=FS_Title
    fstyles(5)=FS_Title
}
