/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLHUD
 *  $VERSION  1.23
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *  $CHANGES
 *    2006.11.18  Def    -  Shake magnitude will now be set to 0.0 when reset is called. This
 *                          prevents the hud from shaking when you launch a new rocket.
 *
 *    2007.04.07  Def    -  Removed BETA expiration code.
 *
 **************************************************************************************************/
class SLHUD extends SLMutator;

var StrangeShell sl;
var() Color col1;
var() Color col2;
var() Color col3;
var() Color col4;
var() float lucent;
var() float fade;
var() float warnBlink;
var() float warnLvl;
var Font font;
var Font lgFont;
var Font debugFont;
var() float u2m;
var bool bInit;
var float screenFade;
var float shakemag;
var int shakex;
var int shakey;
var() float shakeRate;
var int skewx;
var int skewy;
var bool bWarned;
var bool bAlarm;
var float keysFade;
var int keysSlide;
var() string keys[7];
var float keymax;
var int keysUp;
var SLHUDLevel throLvl;
var SLHUDLevel veloLvl;
var SLHUDLevel hullLvl;
var SLHUDLevel fuelLvl;
var SLHUDLevel boostLvl;
var SLHUDLevel coreLvl;
var int retw;
var int reth;
var float retScale;
var int indw;
var int indh;
var int Slide;
var float Last;
var Canvas C;
var SLStylus yy;
var SLUtil U;

function postBeginPlay() {
	local Font lg;
	local Font sm;

	super.postBeginPlay();
	u = spawn(class'SLUtil', self);
	lg = class'SLFontlib'.static.getFont(FS_Slick, true);
	sm = class'SLFontlib'.static.getFont(FS_Slick, false);
	font = sm;
	
	lgFont = class'SLFontlib'.static.getFont(FS_Title);
	debugFont = class'SLFontlib'.static.getFont(FS_Normal);
	
	throLvl = spawn(class'SLHUDLevel', self);
	throLvl.init(0.0, "THRO", "%", lg, sm);
	
	veloLvl = spawn(class'SLHUDLevel', self);
	veloLvl.init(0.0, "VELO", "m/s", lg, sm, throLvl);
	
	hullLvl = spawn(class'SLHUDLevel', self);
	hullLvl.init(0.2, "HULL", "%", lg, sm, veloLvl);
	
	fuelLvl = spawn(class'SLHUDLevel', self);
	fuelLvl.init(0.2, "FUEL", "", lg, sm, hullLvl);

	boostLvl = spawn(class'SLHUDLevel', self);
	boostLvl.init(0.0, "BOOST", "", lg, sm, fuelLvl);
	
	coreLvl = spawn(class'SLHUDLevel', self);
	coreLvl.init(0.0, "CORES", "", lg, sm, boostLvl);
	coreLvl.bNoFitLabels = true;
	
	yy = spawn(class'SLStylus', self);
	reset();
}

function reset(optional StrangeShell newsl) {
	if (newsl != none) {
		setOwner(newsl);
	}

	sl = StrangeShell(owner);
	sl.HUD = self;
	ho = PlayerPawn(sl.getPilot());
	
	if (newsl == none || !u.UTPureOn(ho)) {
		register();
	}
	
	// $FIX(SLV202)  Shake magnitude in SLHUD wasn't reset for new rockets.
	shakemag = 0.0;
	
	screenFade = 0.0;
	slide = -1;
	bInit = False;
	gotoState('Norm');
}

function init(Canvas c) {
	local int x;
	local int y;
	local int w;
	local int h;
	
	if (slide == -1) {
		slide = c.clipx;
		keysSlide = -1;
	}

	retw = max(128, c.clipx * 0.16);
	reth = retw * 0.25;

	indw = 70;
	indh = reth - 13;

	x = (c.clipx + retw) * 0.5;
	y = (c.clipy - reth) * 0.5 + 2;

	w = reth;
	h = reth - 4;

	throLvl.position(x, y, w, h);
	veloLvl.position(x + (w + 1) * 1, y, w, h);
	hullLvl.position(x + (w + 1) * 2, y, w, h);
	fuelLvl.position(x + (w + 1) * 3, y, w, h);
	boostLvl.position(x + (w + 1) * 4, y, w, h);
	if (class'SLConfig'.default.enableBoosters) {
		coreLvl.position(x + (w + 1) * 5, y, w / 4, h);
	} else {
		coreLvl.position(x + (w + 1) * 4, y, w / 4, h);
	}

	bInit = true;
}

auto state norm {
	
	function beginState() {
		// Ignore.
	}
	
}

state warning {

	function beginState() {
		alarm();
		setTimer(warnBlink, false);
	}
  
	function endState() {
		bAlarm = false;
		bWarned = false;
	}
  
	function timer() {
		alarm();
	}
  
	function alarm() {		
		if (fuelLvl.alarmOn() || hullLvl.alarmOn()) {
			bAlarm = !bAlarm;
			if (bAlarm) {
				if (!bWarned || hullLvl.alarmOn() || coreLvl.realLvl == 0) {
					ho.playSound(sl.warnSound, SLOT_None, 1.0);
					bWarned = true;
				}
			}
			setTimer(warnBlink, false);
		} else {
			gotoState('norm');
		}
		
	}
  
}

function bool noMore() {
	if (sl == none || sl.getPilot() == none || sl.bDestroyed) {		
		gotoState('disabled');
		return true;
	}
	return false;
}

function postRender(Canvas c) {
	if (nextHUDMutator != none) {
		nextHUDMutator.postRender(c);
	}
	
	renderHUD(c);
}

function renderHUD(Canvas c) {
	local int i;
	local vector x;
	local vector y;
	local vector z;
	local int pods;
	local string vstr;
	local float xl;
	local float yl;
	
	if (noMore() || ho.bBehindView) {
		return;
	}

	self.c = c;
	
	if (yy.setCanvas(c) || !bInit) {
			init(c);
	}
	
	if (sl.bMarchPlaying) {
		col1 = col3;
	} else {
		col1 = Default.col1;
	}
	c.Style = ERenderStyle.STY_Translucent;
	
	if ((sl.bShootMode || sl.getPilot() != ho) && Slide == yy.clipX) {
		return;
	}
	
	if (level.bHighDetailMode) {
		c.drawColor = col3 * 0.7;
		yy.setPos(yy.cx - retw / 2.0 - indw - 40, yy.cy - (reth * 2.0) / 2.0);
		c.style = ERenderStyle.STY_Modulated;
		c.drawRect(Texture'SLV2Textures.slhud.visor', yy.clipx - c.curx, reth * 2.0);
	}
	
	if (level.bHighDetailMode) {
		c.style = ERenderStyle.STY_Translucent;
	} else {
		c.style = ERenderStyle.STY_Normal;
	}
	
	if (class'SLUtil'.default.bPrivate) {
		c.drawColor = col3 * screenFade;
		vstr = class'SLUtil'.static.getVersionString() @ "- NOT FOR PUBLIC RELEASE. AUTHORIZED USE ONLY. ";
		c.font = debugFont;
		c.textSize(vstr, xl, yl);
		yy.setPos(yy.cx - retw / 2 - indw, yy.cy + reth / 2 + 4);
		yy.drawText(vstr);
	}
	
	if (class'SLConfig'.default.showKeys) {
		drawKeys();
	}
	
	drawIndicators(yy.cx - retw / 2 - indw, yy.cy + reth / 2.0 - indh);
	drawHRot(yy.cx - retw / 2 - indw, yy.cy - reth / 2.0);	
	
	c.DrawColor = col1;
	
	throLvl.render(yy);
	veloLvl.render(yy);
	hullLvl.render(yy);
	fuelLvl.render(yy);
	
	if (class'SLConfig'.default.enableBoosters) {
		if (sl.boosterFuel <= 0.0) {
			if (sl.fc != none && sl.fc.getAmmo() > 0) {
				c.drawColor = col2;
			} else {
				c.drawColor = col3;
			}
		}
		boostLvl.render(yy);
	}
	
	if (sl.fc != none && sl.fc.getAmmo() > 0) {
		c.drawColor = col2;
		coreLvl.Render(yy);
	}
	
	drawReticule(yy.cx - retw / 2.0, yy.cy - reth / 2.0);
	
	drawTargets();
	
	drawLogo(yy, yy.cx - retw / 2 - indw, yy.cy - reth / 2);
	
	if (nextHUDMutator != none) {
		nextHUDMutator.postRender(c);
	}
	
	if(rearViewOn(WindowConsole(c.viewport.console))) {
		drawRearView(yy);
	}
}

state disabled {
	ignores renderHUD;

	function beginState() {
		disable('tick');
		disable('timer');
	}
  
	function endState() {
		enable('tick');
		enable('timer');
	}
  
	function bool noMore() {
		return true;
	}
  
}

function bool rearViewOn(WindowConsole wc) {
	return class'SLConfig'.default.showRearView && (wc == none || !wc.bShowConsole) && (wc.root == none || !wc.Root.bWindowVisible) && !wc.isInState('Typing') && slide == 0;
}

function drawRearView(SLStylus y) {
	local int h, w;
	local Weapon weap;
	
	w = 128;
	h = 64;
	c.drawColor = col1 * screenFade * 0.5;
	y.drawFrame(0, y.cy - h / 2 - 1, w + 1, h + 2, 1, true, true);
	
	weap = sl.pilot.weapon;
	sl.pilot.weapon = none;
	c.drawPortal(0, y.cy - h / 2, w, h, sl, sl.location - 70 * vector(sl.rotation), rotator(-vector(sl.rotation)));
	sl.pilot.weapon = weap;
}

function drawLogo(SLStylus s, int x, int y) {
	local int style;

	c.drawColor = col1 * screenFade;
	s.setPos(x - 32, y);
	c.drawRect(Texture'SLV2Textures.slhud.logo', 32.0, 32.0);
	c.drawColor = col1 * screenFade * 0.20;
	if (reth > 32) {
		style = 3;
	}
	s.drawFrame(x - 21, y, 21, 32, style);
}

function drawKeys() {
	local int x, y, i;
	local float xl, yl;
	local bool fill;
	local int sep;
	local String rstr;

	if (keysFade > 0 || keysSlide == -1) {
		x = -1 - keysSlide;
		y = (c.clipy - reth) / 2.0 - keysUp;

		c.font = lgFont;
		c.textSize(keys[0], xl, yl);

		for (i = 0; i < arrayCount(keys); i++) {

			if (keysSlide >= 0) {
				c.setPos(x + 6, y + 2 + (yl - 1) * i);
				sep = instr(keys[i], "(");
				
				c.drawColor = col4 * fmin(1.0, keysFade);
				if (sep == -1) {
					yy.drawText(keys[i]);
				} else {
					c.drawColor = c.drawColor * 0.6;
					yy.drawText(left(keys[i], sep));

					c.drawColor = col4 * fmin(1.0, keysFade);
					rstr = right(keys[i], len(keys[i]) - sep);
					c.textSize(left(keys[i], sep), xl, yl);
					c.setPos(x + 6 + xl, c.cury);
					yy.drawText(rstr);
				}
			} else {
				c.textSize(keys[i], xl, yl);
				if (xl > keymax)
					keymax = xl;
			}

		}

		if (class'SLConfig'.default.showRearView || ((yy.clipx - retw) / 2.0) - indw - 30 < keymax) {
			keysUp = (yl - 1) * arrayCount(keys) + 4 + reth / 2.0;
		} else {
			keysUp = 0;
		}

		if (keysSlide >= 0) {
			fill = level.bHighDetailMode;
			if (fill) {
				c.drawColor = col4 * (fmin(1.0, keysFade) / 4.0);
			}
			yy.drawFrame(x, y, keymax + 8, (yl - 1) * arrayCount(keys) + 2, 1, fill, true);
		} else {
			keysSlide = keymax + x + 6;
			keysFade = 7.0;
		}
	}
}

function drawReticule(int x, int y) {
	local int sret;
	local int cx;
	local int cy;
	local Color col;
	local Color retcol;
	local int w;

	cx = x + retw / 2;
	cy = y + reth / 2;

	if (level.bHighDetailMode) {
		col = col1 * 0.2 * screenFade;
	} else {
		col = col1 * screenFade;
	}
	
	retcol = col1 * (1.0 - retScale) * screenFade;
	c.drawColor = col;
	
	yy.drawFrame(x, y, retw, reth, 4);
	
	c.drawColor = retcol;
	
	if (sl.bMarchPlaying) {
		yy.setPos(x + 4, y + 4);
		c.font = font;
		yy.drawText("TOTAL COMMITMENT");
		return;
	}
	
	w = retw - 2;
	yy.drawFrame(x + 1 + round(((1.0 - retScale) * w) * 0.5), y + round(((1.0 - retScale) * reth) * 0.5), round(w * retScale), round(reth * retScale), 1);
}

function drawHRot(int x, int y) {
	local float dpp;
	local float xl, yl;
	local float head, left;
	local int i;
	local float edgeFade;
	
	dpp = 1.0;
	c.font = font;
	head = toHeading(rotator(sl.velocity).yaw);
	left = head - indw * 0.5 * dpp;

	if (level.bHighDetailMode) {
		c.drawColor = col1 * lucent * screenFade;
		yy.drawFrame(x, y, indw, reth - indh - 1, 3, true);
	}
	
	for (i = (left / 15) * 15; i < ((head - left) + head); i += 15) {
		if (i >= left) {
			edgeFade = abs(head - i) / dpp;
			
			if (edgeFade > 20) {
				edgeFade = fmax(0.0, 1.0 - ((edgeFade - 20) / 10.0));
			} else {
				edgeFade = 1.0;
			}
			
			yy.setPos(x + (i - left) * dpp, y + 1);
			if (i % 45 == 0) {
				c.drawColor = col1 * 0.4 * screenFade * edgeFade;
				yy.pix(1, 3);
				c.textSize(i, xl, yl);
				yy.setPos(x + (i - left) * dpp - (xl * 0.5), y + 6);
				yy.drawText(i);
			} else {
				c.drawColor = col1 * lucent * screenFade * edgeFade;
				yy.pix(1, 2);
			}
		}
	}
	
	c.textSize(int(head), xl, yl);
	yy.setPos(x + indw / 2.0 - xl * 0.5, y + 6);
	c.drawColor = col1 * screenFade;
	yy.drawText(int(head));
}

function float toHeading (int yaw) {
	if (yaw < 0) {
		yaw = 65535 + yaw;
	}
	return yaw / 65535.0 * 360.0;
}

function drawVRot(int x, int y) {
	c.drawColor = col1 * lucent * screenFade;
	yy.drawFrame(x, y, 32, reth, 0, true);
	c.drawColor = col1 * screenFade;
}

function drawIndicators(int x, int y) {
	local float xl;
	local float yl;
	local string whs;
	local Color on, off;
	
	whs = "WARHEAD: ";
	c.font = font;
	c.textSize("h", xl, yl);
	c.textSize(whs $ "DISARMED", xl, yl);
	indw = xl + 20;

	if (level.bHighDetailMode) {
		on = col1 * screenFade;
		off = col1 * lucent * screenFade;
		c.drawColor = off;
		yy.drawFrame(x, y, indw, indh, 2, true);
	} else {
		on = col1 * screenFade;
		off = col1 * 0.5 * screenFade;
	}
	
	y += (indh - yl * 3) / 2;
	
	c.textSize(whs $ "DISARMED", xl, yl);
	c.drawColor = on;
	yy.setPos(x + 4, y + 1);
	
	switch (sl.warstat) {
	case WH_Auto:
		yy.drawText(whs $ "AUTO");
		break;
	case WH_Armed:
		yy.drawText(whs $ "ARMED");
		break;
	case WH_Disarmed:
		yy.drawText(whs $ "DISARMED");
		break;
	}
	
	yy.setPos(x + 4, y + yl);
	if (sl.gunner == none) {
		c.drawColor = off;
	} else {
		c.drawColor = on;
	}
	yy.drawText("GUNNER ONBOARD");

	yy.setPos(x + 4, y + yl * 2 - 1);
	if (sl.bAfterburn) {
		c.drawColor = on;
	} else {
		c.drawColor = off;
	}
	yy.drawText("AFTERBURNERS ON");
}

function drawTargets() {
	local Pawn p;
	local Inventory i;
	local vector x;
	local vector y;
	local vector z;

	getAxes(ho.viewRotation, x, y, z);
	c.font = font;
	
	foreach sl.visibleCollidingActors(class'Pawn', p, 3000.0, , true) {
		drawTarget(p, x, y, z);
	}
	
	foreach sl.visibleCollidingActors(class'Inventory', i, 3000.0, , true) {
		drawTarget(i, x, y, z);
	}
}

function drawTarget(Actor a, vector x, vector y, vector z) {
	local float da, dist;
	local int xpos, ypos;
	local vector dir;
	local Color color;

	dir = a.location - sl.location;
	dist = vsize(dir);
	dir = normal(dir);
	da = dir dot x;
	
	if (da > 0.94) {
		if (a.isA('Inventory')) {
			color = col2;
		} else {
			color = col4;
		}

		xpos = 0.5 * yy.clipx * (1 + 1.6 * (dir dot y));
		ypos = 0.5 * yy.clipy * (1 - 1.6 * (dir dot z));

		c.drawColor = color * screenFade * ((da - 0.94) / 0.06) * fclamp((3000.0 - dist) / 1000.0, 0.0, 1.0);

		yy.drawFrame(xpos - 8, ypos - 8, 16, 16, 1);
		yy.setPos(xpos - 8, ypos + 9);
		yy.drawText(int((dist * u2m)) $ "m");
	}
}

function gunnerDisplay(Canvas c, Bot b) {
	c.drawColor = col1;
	c.font = lgFont;
	c.setPos(20.0, yy.cy + 40);
	yy.drawText("bot gunner: health: " $ b.health $ " orders: " $ b.orders $ " order object: " $ b.orderObject $ " state: " $ b.getStateName() $ " move target: " $ b.moveTarget);
}

function setSkew() {
	local float pitch;
	local float dyaw, dpitch;
	local float maxd;
	local int maxskew;

	maxd = 16384.0;
	maxskew = yy.clipx / 8;

	dyaw = (rotator(sl.velocity).yaw & 65535) - (sl.rotation.yaw & 65535);

	pitch = sl.rotation.pitch & 65535;
	dpitch = pitch - (rotator(sl.velocity).pitch & 65535);

	u.centerRotComp(dyaw);
	u.centerRotComp(dpitch);
	u.centerRotComp(pitch);

	dyaw = dyaw * (1.0 - fmin(abs(pitch) / maxd, 1.0));

	skewx = clamp(int(((dyaw / maxd) * maxskew) + 0.5), -maxskew, maxskew);
	skewy = clamp(int(((dpitch / maxd) * maxskew) + 0.5), -maxskew, maxskew);
}

function tick(float delta) {
	if (!bInit || noMore()) {
		return;
	}
	
	if (relicFirst(ho)) {
		register();
	}

	if (sl.bAfterburn) {
		throLvl.update(delta, 1.5, "100");
	} else {
		throLvl.update(delta, sl.thlvl, string(int(sl.thlvl * 100)));
	}
	
	veloLvl.update(delta, vsize(sl.velocity) / sl.fullspeed, int(u2m * vsize(sl.velocity)));
	hullLvl.update(delta, sl.armor / class'SLConfig'.default.rocketArmor, int(100.0 * sl.armor / class'SLConfig'.default.rocketArmor));
	fuelLvl.update(delta, sl.fuel / class'SLConfig'.default.maxFuel, int(sl.fuel));
	
	if (class'SLConfig'.default.enableBoosters) {
		if (sl.boosterFuel > 0.0) {
			boostLvl.update(delta, sl.boosterFuel / class'SLConfig'.default.boosterFuel, int(sl.boosterFuel));
		} else {
			boostLvl.update(delta, 1.0, 0);
		}
	}
	
	if (sl.fc != none) {
		coreLvl.update(delta, float(sl.fc.getAmmo()) / sl.fc.getMaxAmmo(), sl.fc.getAmmo());
	}

	if (fuelLvl.alarmOn() || hullLvl.alarmOn()) {
		gotoState('warning');
	}

	if (sl.bShootMode || sl.getPilot() != ho) {
		if (slide < yy.clipx) {
			slide = fmin(yy.clipx, slide + yy.clipx * 3.0 * delta);
		} else {
			return;
		}
	} else {
		if (slide > 0) {
			slide = fmax(0, slide - yy.clipx * 2.0 * delta);
		}
	}

	if (class'SLConfig'.default.showKeys) {
		if (keysSlide > 0) {
			keysSlide = max(0, keysSlide - delta * 300.0);
		} else if (keysFade > 0) {
			keysFade = fmax(0.0, keysFade - delta * 0.5);
		}
	}


	if (slide == 0 && screenFade < 1.0) {
		screenFade = fmin(1.0, (screenFade + delta * 0.5));
	}

	retScale += delta * fade * (vsize(sl.velocity) / ((sl.maxSpeed + sl.minSpeed) / 2.0));
	if (retScale > 1.0) {
		retScale = 0.0;
	}
	
	setOffsets(delta);
}

function setOffsets(float delta) {
	local float localShakeMag;
	local float flicker;

	if (shakemag > 0.0) {
		shakemag = fmax(0.0, shakemag - delta * shakeRate);
		flicker = shakemag / 3.0;
		screenFade = fclamp(screenFade + flicker - frand() * flicker * 2.0, 0.0, 1.0);
	}

	localShakeMag = shakemag;
	if (sl.bAfterburn) localShakeMag += 3.0;
	if (sl.bBoost)     localShakeMag += 3.0;

	if (localShakeMag > 0) {
		shakex = (rand(localShakeMag) - localShakeMag / 2.0);
		shakey = (rand(localShakeMag) - localShakeMag / 2.0);
	} else {
		shakex = 0;
		shakey = 0;
	}

	setSkew();

	yy.setOffsets(shakex + slide + skewx, shakey + skewy);
}

function shake(int sev) {
	if (sl != none && !sl.bShootMode) {
		shakemag = min(shakemag + sev, 90);
	}
}

function int round(float f) {
	return int(0.5 + f);
}

function destroyed() {
	local Mutator m;

	m = u.getHUDMutator(ho, name, true);
	
	u.destroy();
	
	super.destroyed();
}

// $END_CLASSIFIED

defaultproperties
{
	col1=(R=75,G=200,B=100,A=0),
	col2=(R=50,G=150,B=150,A=0),
	col3=(R=150,G=150,B=150,A=0),
	col4=(R=255,G=20,B=20,A=0),
	lucent=0.20
	fade=1.60
	warnBlink=1.00
	warnLvl=0.25
	u2m=0.0245
	shakeRate=12.00
	keys(0)="SLV2 Controls"
	keys(1)="Jump/AltFire (Eject)"
	keys(2)="Forward/Back (Throttle)"
	keys(3)="Next Weapon (Warhead Status)"
	keys(4)="Fire (Afterburner)"
	keys(5)="Duck (Shoot Mode)"
	keys(6)="Walk (Booster)"
}
