/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLKongLauncher
 *  $VERSION  1.22
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLKonglauncher extends SLWeapon;

#EXEC TEXTURE IMPORT NAME=kl_xl FILE=Resources\kl_xl.pcx FLAGS=2 MIPS=OFF

var SLFuelcore FC;
var Vector launchv;
var float LastFired;
var SLSkyNet skynet;
var SLSkyNode launchsn;
var bool bClear;
var float clearanceStamp;
var string teamSLClass[4];
var int numFuelCoresOnPickup;

replication {
	reliable if (role == ROLE_Authority && bNetOwner)
		bClear;
}

// Damn bTossedOut appears to be always false in this case.
/*
function Inventory spawnCopy(Pawn p) {
	local Inventory copy;
	copy = super.spawnCopy(p);
	
	// Check if the pawn should be given some (extra) fuel cores.
	if (copy != none && !bTossedOut) {
		givePickupCores(p);
	}
	
	return copy;
}
*/

function bool handlePickupQuery(Inventory item) {
	local bool bAccept;
	
	bAccept = super.handlePickupQuery(item);
	
	// Check if the pawn should be given some (extra) fuel cores.
	if (bAccept && item.class == class && !item.bTossedOut) {
		// $ADD(SLV203)  Option to give players some fuel cores each time they pickup a launcher.
		givePickupCores(Pawn(owner));
	}
	
	return bAccept;
}

function givePickupCores(Pawn p) {
	local SLFuelcore fc;
	
	// Get fuel core.
	fc = SLFuelcore(p.findInventoryType(class'SLFuelcore'));
	
	// Pawn has no fuel cores yet in its inventory.
	if (fc == none) {
		fc = spawn(class'SLFuelCore', p);
		fc.giveTo(p);
		fc.ammoAmount = 0;
	}
	
	// Add the fuel cores.
	fc.addAmmo(numFuelCoresOnPickup);
}

function bringUp() {
	local int team;

	if (Pawn(owner) != none) {
		// $FIX(SLV203)  Accessed array out of bounds warning in SLKonglauncher.bringUp().
		team = clamp(Pawn(owner).playerReplicationInfo.team, 0, 3);
		projectileClass = class<Projectile>(dynamicLoadObject(string(class'SLUtil'.default.package) $ "." $ teamSLClass[team], class'Class'));
		altProjectileClass = projectileClass;
	}
	super.bringUp();
}

state idle {
	
	simulated function timer() {
		super.timer();
		if (owner.isA('Bot') && useAltFire(Bot(owner))) {

			if (skynet != none) {
				if (skynet.bBuilt) {
					launchsn = skynet.findLaunchNode(Bot(owner));
					if (launchsn != none && fastTrace(launchsn.location, owner.location)) {
						launchsn.addLaunch();
						owner.setRotation(rotator(launchsn.location - owner.location));
						altFire(0.0);
					}
				}
			} else {
				spawnSkyNet();
			}
		}
	}
  
}

function bool useAltFire(Bot b) {
	local BotReplicationInfo bri;
	local StrangeShell sl;

	if (!class'SLSkyNet'.default.bBotSupport) {
		return false;
	}

	bri = BotReplicationInfo(b.playerReplicationInfo);
	
	if (!level.game.isA('CTFGame') || level.game.isA('Assault') && (bri.team == Assault(level.game).attacker.teamIndex)) {
		return false;
	}

	if (!ammoLeft() || b.enemy != none) {
		return false;
	}
	
	if (bri.realOrders == 'Defend' || bri.realOrders == 'Follow') {
		return false;
	}
	
	foreach b.radiusActors(class'StrangeShell', sl, vsize(class<StrangeShell>(projectileClass).default.pilotOffset) * 2.0) {
		if (sl.getPilot() == b || sl.getGunner() == b) {
			return false;
		}
	}

	return true;
}

function spawnSkyNet() {
	foreach allActors(class'SLSkyNet', skynet) {
		break;
	}

	if (skynet == none) {
		skynet = spawn(class'SLSkyNet');
	}
}

simulated function setRetMsgs() {
	super.setRetMsgs();

	if (!bClear && (level.timeSeconds - clearanceStamp < 1.5)) {
		retmsg = "no clearance!";
	}

	if (fc == none) {
		fc = SLFuelcore(Pawn(owner).findInventoryType(class'SLFuelcore'));
	}
	
	if (fc != none && fc.ammoAmount > 0) {
		retmsg2 = "cores " $ string(fc.ammoAmount);
	}
}

function setWeaponStay() {
	bWeaponStay = false;
}

function float rateSelf(out int bUseAltMode) {
	local Bot me;

	bUseAltMode = 0;
	
	me = Bot(owner);
	if (me == none || !ammoLeft() || level.timeSeconds - lastFired < 2.0) {
		return -2.0;
	}
	
	if (useAltFire(me)) {
		return 1.0;
	}
	
	if (me.enemy == none) {
		return -2.0;
	}

	if (vsize(me.enemy.location - me.location) > 800 && fastTrace(me.enemy.location, me.location)) {
		return 2.0;
	}

	return -1.0;
}

function float suggestAttackStyle() {
	return -1.0;
}

function spawnPuff() {
	local vector x;
	local vector y;
	local vector z;

	if (level.bHighDetailMode && !level.bDropDetail && level.netMode != NM_DedicatedServer) {
		getAxes(owner.rotation, x, y, z);
		spawn(class'SLLaunchPuff', , , owner.location + x * 80, rotation);
	}
}

function fire(float v) {
	lastFired = level.timeSeconds;
	super.fire(v);
	if (lastProj != none) {
		spawnPuff();
	}
}

function altFire(float v) {
	local bool bAmmo;

	bAmmo = ammoLeft();

	if (bAmmo) {
	    spawnProjOffset = -class<StrangeShell>(altProjectileClass).default.pilotOffset;
	    bSetProjOwner = true;
	    lastProj = none;
	}
	
	super.altFire(v);

	if (bAmmo) {
		if (lastProj != none) {
			StrangeShell(lastProj).setPilot(Pawn(owner), skynet, launchsn);
		} else {
			clearWarning();
			clearanceStamp = level.timeSeconds;
		}
		bClear = lastProj != none;
		spawnProjOffset = vect(0.0, 0.0, 0.0);
		bSetProjOwner = false;
	}
}

state altFiring {

begin:
	sleep(1.0);
	finish();
}

function clearWarning() {
	if (owner.isA('PlayerPawn')) {
		PlayerPawn(owner).clientPlaySound(class'StrangeShell'.default.warnSound, false, true);
	}
}

simulated function bool clientFire(float v) {
	launchv = vect(0.0, 0.0, 0.0);
	return super.clientFire(v);
}

simulated function bool clientAltFire(float v) {
	if (super.clientAltFire(v)) {
		clearanceStamp = level.timeSeconds;
		launchv = owner.velocity;
		return true;
	} else {
		return false;
	}
}
// $END_CLASSIFIED

defaultproperties {
	teamSLClass(0)="SLRocket1"
	teamSLClass(1)="SLRocket2"
	teamSLClass(2)="SLRocket3"
	teamSLClass(3)="SLRocket4"
	leftMesh="SLV2Models.lslview"
	assFire=(aname=Fire,arate=0.20,tween=0.10,Sound=Sound'SLV2Sounds.kongl.Fire',Volume=3.00,Slot=SLOT_None,bSkipOwner=True),
	assAltFire=(aname=AltFire,arate=0.00,tween=0.00,Sound=Sound'SLV2Sounds.kongl.AltFire',Volume=4.00,Slot=SLOT_None,bSkipOwner=True),
	assSelect=(aname=Select,arate=0.40,tween=0.00,Sound=Sound'SLV2Sounds.kongl.Select',Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	assDown=(aname=Down,arate=0.40,tween=0.05,Sound=None,Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	assReload=(aname=Reload,arate=0.50,tween=0.00,Sound=None,Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	assIdle=(aname=Idle,arate=0.90,tween=0.00,Sound=None,Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	idleLapse=3.00
	idleRandom=0.02
	xhair=Texture'SLV2Textures.xhair.kl'
	xhair_xl=Texture'kl_xl'
	paSlide=0.75
	mimicPriorityOf=WarheadLauncher
	WeaponDescription="Giant, yet portable anti-everything rocket."
	InstFlash=-0.40
	InstFog=(X=950.00,Y=650.00,Z=290.00),
	AmmoName=Class'StrangeAmmo'
	bWarnTarget=True
	bAltWarnTarget=True
	bSplashDamage=True
	bSpecialIcon=True
	FireOffset=(X=12.00,Y=-1.00,Z=-10.00),
	ProjectileClass=Class'StrangeShell'
	AltProjectileClass=Class'StrangeShell'
	shakemag=1000.00
	shaketime=0.30
	shakevert=30.00
	AIRating=1.00
	RefireRate=0.25
	AltRefireRate=0.25
	SelectSound=Sound'SLV2Sounds.kongl.Select'
	DeathMessage="%o was nuked by %k's %w."
	AutoSwitchPriority=10
	InventoryGroup=10
	PickupMessage="You picked up a Strangelove."
	ItemName="Strangelove"
	RespawnTime=60.00
	PlayerViewOffset=(X=-0.50,Y=-2.50,Z=-6.00),
	PlayerViewMesh=SkeletalMesh'SLV2Models.slview'
	BobDamping=0.98
	PickupViewMesh=SkeletalMesh'SLV2Models.slpickup'
	ThirdPersonMesh=SkeletalMesh'SLV2Models.slhand'
	StatusIcon=Texture'SLV2Textures.SLHUD.kongicon'
	MaxDesireability=1.00
	PickupSound=Sound'SLV2Sounds.kongl.Pickup'
	Icon=Texture'SLV2Textures.SLHUD.kongicon'
	Rotation=(Pitch=0,Yaw=0,Roll=-500),
	Mesh=SkeletalMesh'SLV2Models.slview'
	CollisionHeight=4.00
	Mass=40.00
}
