/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLSkyNet
 *  $VERSION  1.11
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLSkyNet extends SLInfo;

var bool bBotSupport;
var float buildStamp;
var bool bCTF;
var bool bAss;
var() name autotag;
var NavigationPoint autonp;
var SLSkyNode firstsn;
var SLSkyNode autosn;
var int i;
var int type;
var byte goalnum;
var Actor goals[16];
var SLSkyNode goalNodes[16];
var float maxGoalDist[16];
var int nodeSum;
var() float weldRadius;
var() float linkRadius;
var() float goodAngle;
var bool bBuilt;
var() bool bShowNodes;
var() bool bShowLinks;
var() bool bNoAutoWeightNodes;
var() float randomLinkAdjust;
var() int laps;

function postBeginPlay() {
	if (!properGame()) {
		destroy();
	}
	
	if (bAss) {
		goalnum = Assault(level.game).numForts;
	} else {
		goalnum = 4;
	}
	
	setMapType();
}

function setMapType() {
	local SLSkyNode sn;
	local NavigationPoint np;

	foreach allActors(class'SLSkyNode', sn) {
		type = 3;
		return;
	}
	
	foreach allActors(Class'NavigationPoint', np, 'SKYNODE') {
		type = 2;
		return;
	}
	
	type = 1;
}

function bool properGame() {
	if (level.game.isA('CTFGame')) {
		bCTF = true;
	}
	
	if (level.game.isA('Assault')) {
		bAss = True;
	}
	return bCTF || bAss;
}

function byte randomGoal(optional bool bEx, optional byte exgoal) {
	local int i;
	local int pnum;
	local int poss[16];

	for (i = 0; i < goalnum; i++) {
		if (goals[i] != none && !(bEx && i == exgoal)) {
			poss[pnum] = i;
			pnum++;
		}
	}
  	
  	if (pnum > 0) {
		return poss[rand(pnum)];
	} else {
		return 0;
	}
}

function SLSkyNode findLaunchNode(Bot b) {
	local int team;
	local SLSkyNode sn;
	local SLSkyNode ln;
	local byte goal;
	local float cw;
	local float w;

	goal = -1;
	if (getGoal(b, goal)) {
		if (vsize(goalNodes[goal].location - b.location) < linkRadius * 2.0) {
			return none;
		}
	}
	
	foreach b.radiusActors(class'SLSkyNode', sn, linkRadius) {
		if (sn.launchWeight() > 0.5) {
			if (goodNode(sn, b)) {
				if (goal == -1) {
					ln = sn;
					break;
				} else {
					cw = sn.getWeight(goal);
					
					if (ln == none) {
						ln = sn;
						w = cw;
					} else {
						if (cw > w) {
							ln = sn;
							w = cw;
						}
					}
				}
			}
		}
	}
	

	return ln;
}

function bool getGoal(Bot bot, out byte goal) {
	local byte team;
	local FortStandard fort;
	local int i;

	team = bot.playerReplicationInfo.team;
	if (bCTF) {
		
		if (bot.playerReplicationInfo.hasFlag != none) {
			goal = team;
		} else {
			goal = findEnemyFlagGoal(team);
		}
	
	} else if (bAss) {
		fort = Assault(level.game).bestFort;
		if (fort != none) {
			if (goal >= goalnum || fortStandard(goals[goal]) != fort) {

				for (i = 0; i < goalnum; i++) {
					if (goals[i] == fort) {
						goal = i;
						break;
					}
				}
			
			}
		} else {
			return false;
		}
	}
	return true;
}

function byte findEnemyFlagGoal(byte team) {
	local int i;

	for (i = 0; i < goalnum; i++) {
		if (i != team) {
			return i;
		}
	}
}

function bool flagAtBase(byte team) {
	return CTFReplicationInfo(level.game.gameReplicationInfo).flagList[team].bHome;
}

function bool goodNode(SLSkyNode n, Bot b) {
	local vector ton;

	if (n != none) {
		ton = normal(n.location - b.location);
		
		for (i = 0; i < arrayCount(n.links); i++) {
			if (n.links[i] != none && fastTrace(n.links[i].location, n.location) && (ton dot normal(n.location - n.links[i].location)) > goodAngle) {
				return true;
			}
		}
	}
	
	return false;
}

function SLSkyNode closestNode(vector loc, float radius) {
	local SLSkyNode sn;
	local SLSkyNode closest;
	local float dist;

	foreach radiusActors(class'SLSkyNode', sn, radius, loc) {
		if (closest == none || vsize(sn.location - closest.location) < dist) {
			closest = sn;
		}
	}
	
	return closest;
}

auto state buildSkyNet {

begin:
	buildStamp = level.timeSeconds;
	if (type == 1 || type == 2) {
		
		autonp = level.navigationPointList;
		while (autonp != none) {
			if (bCTF && autonp.isA('FlagBase')) {
				goals[FlagBase(autonp).team] = autonp;
			}
			if (type == 1 || type == 2 && autosn.tag == 'SKYNODE') {
				autosn = autoSkyNode(autonp, autosn);
				nodeSum++;
			}
			autonp = autonp.nextNavigationPoint;
			sleep(0.0);
		}
		
		firstsn = autosn;
		
		autosn = firstsn;
		while (autosn != none) {
			autosn.setWeldLoc();
			riseSkyNode(autosn);
			autosn = autosn.nextSkyNode;
			sleep(0.0);
		}	
	}

	if (bAss) {
		for (i = 0; i < goalnum; i++) {
			goals[i] = Assault(level.game).fort[i];
		}
	}

linkNodes:
	autosn = firstsn;
	while (autosn != none) {
		if (!autosn.linkUp(linkRadius)) {
			if (autosn == firstsn) {
				firstsn = autosn.nextSkyNode;
			}
			autosn.destroy();
		}
		autosn = autosn.nextSkyNode;
		sleep(0.0);
	}

weightNodes:
	if (!bNoAutoWeightNodes) {

		for (i = 0; i < goalnum; i++) {
			if (goals[i] != none) {
				goalNodes[i] = closestNode(goals[i].location, linkRadius);
			}
		}

		autosn = firstsn;
		while (autosn != none) {
			for (i = 0; i < goalnum; i++) {
				if (goalNodes[i] != none) {
					autosn.weight[i] = vsize(goalNodes[i].location - autosn.location);
					if (autosn.weight[i] > maxGoalDist[i]) {
						maxGoalDist[i] = autosn.Weight[i];
					}
				}
			}
			sleep(0.0);
			autosn = autosn.nextSkyNode;
		}
		
		autosn = firstsn;
		while (autosn != none) {
			for (i = 0; i < goalnum; i++) {
				if (goalNodes[i] != none) {
					autosn.weight[i] = 1.0 - autosn.weight[i] / maxGoalDist[i];
				}
			}
			sleep(0.0);
			autosn = autosn.nextSkyNode;
		}
	}
	
	bBuilt = true;
}

function SLSkyNode autoSkyNode(NavigationPoint np, SLSkyNode lastsn) {
	local SLSkyNode n;

	if (!isGoalNavPoint(np)) {
		foreach np.radiusActors(class'SLSkyNode', n, weldRadius) {			
			n.weld(np.location);
			return lastsn;
		}
	}
  
	n = spawn(class'SLSkyNode', , 'autoSkyNode', np.location);
	
	if (n == none) {
		return lastsn;
	}
	
	if (bShowNodes) {
		n.bHidden = false;
	}
	
	n.navPoint = np;
	n.nextSkyNode = lastsn;
	if (lastsn != none) {
		lastsn.prevSkyNode = n;
	}
	
	return n;
}

function bool isGoalNavPoint(NavigationPoint np) {
	return bAss && np.isA('FortStandard') || bCTF && np.isA('FlagBase');
}

function linkNodes() {
	local SLSkyNode last;
	local SLSkyNode n;

	foreach allActors(class'SLSkyNode', n) {
		n.nextSkyNode = last;
		last = n;
	}	
}

function riseSkyNode(SLSkyNode n) {
	local vector down;
	local vector up;

	if (!isGoalNavPoint(n.navPoint)) {
		down = roof(n.navPoint, -500.0);
		up = roof(n.navPoint, 500.0);
		n.setLocation(down + (up - down) * 0.5);
	}
}

function vector roof(Actor a, float dist) {
	local Actor hit;
	local vector end;
	local vector hl;
	local vector hn;

	end = a.location + vect(0.0, 0.0, 1.0) * dist;
	hit = a.trace(hl, hn, end, a.location, false);
	if (hit != none) {
		return hl;
	} else {
		return end;
	}
}
// $END_CLASSIFIED

defaultproperties {
	bBotSupport=True
	autotag=autoSkyNode
	weldRadius=300.00
	linkRadius=1800.00
	goodAngle=0.20
	randomLinkAdjust=0.05
}
