/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLSkyNode
 *  $VERSION  1.80
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLSkyNode extends Actor;

var() float linkRadius;
var() float randomLinkAdjust;
var() float suggestedSpeed;
var() bool bOffensiveNode;
var() bool bGateNode;
var() bool bFinalNode;
var SLSkyNode links[5];
var float ldists[5];
var SLSkyNodeBeam beams[5];
var float weight[16];
var bool bWeighed;
var int weighDepth;
var int launches;
var int launchSxs;
var int passes;
var int teamPasses[4];
var int wallDeaths;
var int shotDeaths[4];
var SLSkyNode nextSkyNode;
var SLSkyNode prevSkyNode;
var NavigationPoint navPoint;
var Vector weldLoc;
var int weldSum;

function float getWeight(byte team) {
	return weight[team];
}

function weld(vector l) {
	weldSum++;
	weldLoc += l;
}

function setWeldLoc() {
	if (weldSum > 0) {
		setLocation((location + weldLoc) / (weldSum + 1.0));
	}
}

function addLaunch(optional bool bSuccess) {
	if (bSuccess) {
		launchSxs++;
	} else {
		launches++;
	}
}

function float launchWeight() {
	return (launchSxs + 1) / float(launches + 1);
}

function SLSkyNode randomLink() {
	local int i;

	i = rand(arrayCount(links));
	while (links[i] == none && i > 0) {
		i--;
	}
	
	return links[i];
}

function bool linkUp(float r) {
	local SLSkyNode sn;
	local int i;
	local float dist;
	local bool bNone;

	bNone = true;
	if (linkRadius > 0) {
		r = linkRadius;
	}

	foreach radiusActors(class'SLSkyNode', sn, r) {
		
		if (sn != self && fastTrace(sn.location, location)) {
			dist = vsize(location - sn.location);
			bNone = false;

			for (i = 0; i < arrayCount(links); i++) {
				if (dist < ldists[i] || ldists[i] == 0) {
					insertLink(i, sn, dist);
					break;
        		}
      		}
		}
		
	}
	
	return !bNone;
}

function showBeams(byte goal) {
	local int i;
	local SLSkyNodeBeam beam;
	local float w;
	local float hi;
	local float lo;

	for (i = 0; i < arrayCount(links); i++) {
		if (beams[i] == none) {
			w = links[i].weight[goal];
			if (i == 0) {
				hi = w;
				lo = w;
			} else if (w < lo) {
				lo = w;
			} else if (w > hi) {
				hi = w;
			}
			
			beams[i] = spawn(class'SLSkyNodeBeam', , , location, rotator(links[i].location - location));
			
			if (beams[i] != none) {
				beams[i].drawScale = ldists[i] / 100.0;
			}
		}
	}

	for (i = 0; i < arrayCount(links); i++) {
		if (beams[i] != none) {
			beams[i].ambientGlow = 255 * ((links[i].weight[goal] - lo) / (hi - lo));
			beams[i].scaleGlow = beams[i].ambientGlow;
			beams[i].lightBrightness = beams[i].ambientGlow;
		}
	}
}

function hideBeams() {
	local int i;

	for (i = 0; i < arrayCount(links); i++) {
		if (beams[i] != none) {
			beams[i].destroy();
		}
	}
}

function insertLink(int index, SLSkyNode n, float dist) {
	local int i;
	
	for (i = arrayCount(links) - 2; i >= index; i--) {
		if (links[i] != none) {
			links[i + 1] = links[i];
			ldists[i + 1] = ldists[i];
		}
	}
	
	links[index] = n;
	ldists[index] = dist;
}

function vis(bool on, optional byte goal) {
	if (!on) {
		hideBeams();
	} else {
		showBeams(goal);
	}
}

function destroyed() {
	if (prevSkyNode != none) {
		prevSkyNode.nextSkyNode = nextSkyNode;
	}
}
// $END_CLASSIFIED

defaultproperties {
    suggestedSpeed=-1.00
    bHidden=True
    Style=STY_Translucent
    Texture=Texture'SLV2Textures.muzf.Spark'
    bCollideActors=True
}
