/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    StrangeShell
 *  $VERSION  1.44
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION  Ridable Strangelove rocket.
 *
 **************************************************************************************************/

class StrangeShell extends Projectile;

#EXEC AUDIO IMPORT FILE=Resources\BoosterStart.wav NAME=booster_kick

// From Warshell.
var float cannonTimer;

// From GuidedWarshell.
var rotator guidedRotation;
var float currentTimeStamp, lastUpdateTime, clientBuffer, serverUpdate;
var bool bUpdatePosition;
var bool bDestroyed;
var SavedMove savedMoves;
var SavedMove freeMoves;

// From StrangeShell.
var SLController slc;
var bool dumb;                 // Rocket was spawned from a dumb fire, i.e. not from alt fire.
var() float armor;             // Amount of damage the hull can withstand.
var() float fuel;              // Current fuel level of the rocket.
var() float teamDamage;        // Multiply damage with this amount for friendly fire.
var() vector pilotOffset;      // Pilot location relative to the rocket location.
var() float minspeed;
var() float fullspeed;
var() float accelmax;
var() float decelRatio;
var bool bAfterburn;           // Whether or not the afterburner is on.
var() float afterburnRate;     // Extra fuel ratio you burn when afterburners are enabled.
var() float baseFuelRate;      // Fuel per second you burn at cruising speed.
var bool bNoFuel;              // Rocket has run out of fuel.
var() float thrate;
var() Class<SLHUD> HUDClass;
var() Class<SLTwinkle> twinkleClasses[3];
var() Class<SLDamagePuff> damagePuffClass;
var() Class<SLContrail> contrailClass;
var SLContrail rootct;
var() float puffRand;          // Amount of randomness in the start location of puffs.
var() float puffOffset;        // Distance the puffs starts behind the rocket.
var() float crushAngle;        // The impact angle the hull can survive at max velocity. This is a
                               // percentage of 90, so 45 degrees would be 0.5.
var() Sound gunnerSound;       // Sound played when a player is picked up by a StrangeShell.
var() Sound crashSound;        // Sound played when the rocket crashes.
var() Sound warnSound;         // Sound played when the player should be warned of a critical event.
var() Sound abFadeSound;       // Sound played when the afterburner is fading out.
var() Sound abLoopSound;       // Sound played when the afterburner is active.
var() Sound abKickSound;       // Sound played when the afterburner kicks in.
var() Sound refuelSound;       // Sound played when the rocket refuels.
var() Sound hullHits[4];       // Sound played when the hull gets hit a projectile.
var() Sound bounces[5];        // Sound played when the rocket bounces off (e.g. a wall).
var() Sound sndPilotEj;
var() Sound sndGunnerEj;
var() Sound sndDrop;
var() Sound boostSound;        //

var vector realLocation;
var vector realVelocity;
var vector realAccel;

var() enum EWarStat {
	WH_Auto,
	WH_Armed,
	WH_Disarmed
} warstat;

var bool bCanOverride;
var GameReplicationInfo GRI;
var bool bDetonate;
var float errorSum;
var bool bClientNoFuel;
var bool bBlasted;
var Pawn ejected[2];
var float ejectTime[2];
var() float touchLapse;
var float tstamp;
var float smokeTimer;
var float closeTimer;
var bool bClosed;
var name dtype;
var() localized string normDamStr;
var() localized string normSelfDamStr;
var() localized string ramDamStr;
var() localized string ramSelfDamStr;
var() localized string ramSelf2DamStr;
var() localized string shotDamStr;
var() localized string shotSelfDamStr;
var() localized string hitWallDamStr;
var() localized string hitWall2DamStr;
var() localized string hitSLDamStr[5];
var() localized string hitSLDamStr2;
var bool bSputter;
var float sputtime;
var float launched;
var() float shootCR;
var() float lvlimit;
var StrangeShell rammed;
var Teleporter lastTP;
var rotator lastTPRot;
var Pawn pilot;
var Pawn gunner;
var bool bShootMode;
var bool addedHUD;
var SLHUD HUD;
var float throttle;
var float thrprev;
var float thlvl;
var bool bThrotSent;
var int throtDir;

struct RiderInfo {
	var Pawn Pawn;
	var bool bOldJumpStatus;
	var bool bDelayedEject;
	var SLBotBrain brain;
	var Actor trailers[5];
	var Vector lastLoc;
};

var RiderInfo pInfo;
var RiderInfo gInfo;
var vector fogReset;
var bool bFog;
var SLNullWeapon nw;
var Weapon lastWeapon;
var float teleportDist;
var SLFuelcore fc;
var float marchWait;
var bool bMarchUnder;
var bool bMarchPlaying;
var() Sound sndMarchLoop;
var() Sound sndTotalComm;
var float savedBob;
var bool bWahooPlayed;
var Sound sndWahoo;
var() localized string gunnerMountStr[3];
var() localized string gunnerMountStr2[3];
var float painTimer;
var SLUtil u;

var Weapon savedWeapon;        // Used to restore the weapon of the instigator when spawning a new
                               // StrangeWave, such that there won't be accessed none warnings when
                               // a SpecialDamage death string is parsed.

var bool bPilotEjected;        //

var Class<SLConfig> conf;      //

var bool bDisableArmedWH;      //

var float boosterFuel;         //
var bool bBoost;               //
const boosterBurnRate = 20.0;  //
const boostPuffRand = 8.0;     //
const twinklePuffRand = 16.0;  //

var bool bTeamDetClass;        //

var bool bNoPilotTeamDam;      //

var bool bPilotCantCarryFlag;  //

var byte teamNum;              //

var bool awaitingDestroy;      //

var bool semiDestroyed;        //

const destroyDelay = 5;        //

replication {
	// Server to client:
	unreliable if (role == ROLE_Authority)
		clientAdjustPosition, dumb, armor, bAfterburn, bNoFuel, warstat, pilot, gunner,
		bShootMode, throttle, fc, bBoost, clientDamageEffects, semiDestroyed;
		
	unreliable if (role == ROLE_Authority && bNetOwner)
		fuel, boosterFuel, bPilotEjected;
		
	unreliable if (role == ROLE_Authority && bNetOwner && bNetInitial)
		guidedRotation;

	unreliable if (role == ROLE_Authority && (!bNetOwner || remoteRole == ROLE_SimulatedProxy))
		realLocation, realVelocity, realAccel;
    
    // Client to server:
	unreliable if (role == ROLE_AutonomousProxy || role == ROLE_SimulatedProxy)
		serverMove;
        
	reliable if (role < ROLE_Authority && bNetOwner)
		clientSendThrot, goShootMode, startBooster;
}

simulated function postBeginPlay() {
	u = spawn(class'SLUtil', self);
	launched = level.timeSeconds;

	foreach allActors(class'GameReplicationInfo', gri) {
		break;
	}

	startContrail();
	setTimer(0.000000001, true);
	tstamp = level.timeseconds;

	bShootMode = false;
	thlvl = 0.0;
	throttle = 0.0;
	bThrotSent = false;
	addedHUD = false;
	bFog = false;
	fogReset.x = 0;
	fogReset.y = 100;
	fogReset.z = 0;
	throtDir = 0;
	boosterFuel = 0.0;
	
	if (instigator != none) {
		savedWeapon = instigator.weapon;
	}
	
	minSpeed = conf.default.minSpeed;
	fullSpeed = conf.default.fullSpeed;
	maxSpeed = conf.default.maxSpeed;
	armor = conf.default.rocketArmor;
	fuel = conf.default.maxFuel;
	switch (conf.default.warheadStatus) {
		case WH_Armed:    warStat = WH_Armed; break;
		case WH_Disarmed: warStat = WH_Disarmed; break;
		default:          warStat = WH_Auto;
	}
	bWahooPlayed = conf.default.disableLaunchScream;
	
	applyCustomSkin();
}

simulated function applyCustomSkin() {
	local Texture newSkin;
	local int index;
	
	// $ADD(SLV203)  Support for customized rocket skins.
	if (0 <= teamNum && teamNum <= 3) {
		for (index = teamNum * 2; index <= teamNum * 2 + 1; index++) {
			if (conf.default.customSLVSkins[index] != "") {
				
				// Load texture.
				newSkin = Texture(DynamicLoadObject(conf.default.customSLVSkins[index], class'Texture'));
				
				// Continue only if the texture was loaded.
				if (newSkin != none) {
					// Apply texture.
					multiSkins[index % 2] = newSkin;
				}
			}
		}
	}
}

simulated function startContrail() {
	if (level.netMode != NM_DedicatedServer && level.bHighDetailMode && !conf.default.disableContrails) {
		rootct = spawn(contrailClass, self);
	}
}

simulated function timer() {
	local float now;
	local float delta;

	if (awaitingDestroy) {
		destroy();
		awaitingDestroy = false;
	}

	if (bDestroyed) {
		return;
	}

	now = level.timeSeconds;

	if (tstamp != 0) {
		delta = now - tstamp;
		tstamp = now;
		
		if (slc == none && !dumb) {
			ctick(delta);
		}
		
		smoketick(delta);
	}
	
	tstamp = now;
}

simulated function smoketick(float delta) {
	local float rate;
	local vector x, y, z;

	if (level.bHighDetailMode) {
		rate = 0.01;
		if (level.bDropDetail) {
			rate = 0.1;
		}
	} else {
		rate = 0.15;
	}

	smokeTimer += delta;
	if (smokeTimer > rate) {
		smokeTimer -= rate;
	} else {
		return;
	}
	
	// $FIX(SLV202)  Rocket behaving weird and self destructing a few seconds after its pilot has
	//               disconnected from the server.
	if (!dumb && role == ROLE_Authority && remoteRole == ROLE_AutonomousProxy && owner == none) {
		remoteRole = ROLE_SimulatedProxy;
		pilot = none;
		autoArm(); // In case the player was piloting the rocket.
	}

	if (!dumb && pilot != none && !gameOver() && !instigator.isA('Bot') && role == ROLE_Authority && (level.timeSeconds - serverUpdate > 4.0)) {
		dtype = 'NoUpdate';
		explode(location, vect(0.0, 0.0, 1.0));
		return;
	}

	cannonTimer += rate;
	if (cannonTimer > 0.6) {
		//warnCannons();
		cannonTimer -= 0.6;
		getTrailers();
	}

	if (level.netMode != NM_DedicatedServer) {
		spawnPuffs(rate);
		
		if (!bClosed) {
			closeTimer += rate;

			if (closeTimer > 1.0) {
				
				if (dumb) {
					getAxes(rotation, x, y, z);
					if (!fastTrace(location + x * vsize(velocity) * 5.0)) {
						bClosed = true;
					}
				} else if (pilot == none) {
					bClosed = true;
				}

				closeTimer -= 1.0;

				if (bClosed) {
          			playAnim('close', 0.4, 0.2);
				}

			}
		}
	}
}

/*
function warnCannons() {
	local Pawn p;

	for (p = level.pawnlist; p != none; p = p.nextPawn) {
		if (p.isA('TeamCannon') && !p.isInState('TrackWarhead') && p.lineOfSightTo(self)) {
			p.target = self;
			p.gotoState('TrackWarhead');
		}
	}
}
*/

simulated function spawnPuffs(float rate) {
	local SLExhaustPuff puff;
	local vector x, y, z;
	local vector loc, baseLoc;
	local float pDamagePuff;
	local byte numTwinkles, count;
	
	local SLBoostSmoke sPuff;

	if (region.zone.bWaterZone || bNoFuel) {
		if (bSputter) {
			sputtime += rate;
		} else {
			bSputter = true;
			sputtime = 0.0;
		}
	} else {
		bSputter = False;
	}
	
	getAxes(rotation, x, y, z);
	
	if (!bSputter || bSputter && sputtime < 1.0 && frand() * 1.0 > sputtime) {
		baseLoc = location - x * puffOffset;
		
		if (level.bDropDetail) {
			numTwinkles = rand(2);
		} else if (level.bHighDetailMode) {
			numTwinkles = rand(3) + 1;
		} else {
			numTwinkles = rand(3);
		}

		for (count = 0; count < numTwinkles; count++) {
			loc = baseLoc;
			loc += y * (frand() * twinklePuffRand * 2.0 - twinklePuffRand);
			loc += z * (frand() * twinklePuffRand * 2.0 - twinklePuffRand);
	
			spawn(twinkleClasses[rand(arrayCount(twinkleClasses))], , , loc);
		}
	}
	
	if (bBoost && !region.zone.bWaterZone) {
		loc = location - x * puffOffset;
		loc += y * (frand() * boostPuffRand * 2.0 - boostPuffRand);
		loc += z * (frand() * boostPuffRand * 2.0 - boostPuffRand);
		sPuff = spawn(class'SLBoostSmoke', , , loc);
	}

	pDamagePuff = 1.0 - armor / conf.default.rocketArmor;
	pDamagePuff *= pDamagePuff;
	if (frand() <= pDamagePuff) {
		spawnDamPuffs(location - x * 20, y, z, 1.0 - armor / conf.default.rocketArmor);
	}
}

simulated function spawnDamPuffs(vector loc, vector y, vector z, float damf) {
	local SLDamagePuff puff;
	local float rand;

	if (puffRand > 0.0) {
		rand = puffRand * 2.0;
	} else {
		rand = 4.0;
	}

	loc += y * (frand() * rand * 2.0 - rand);
	loc += z * (frand() * rand * 2.0 - rand);

	puff = spawn(damagePuffClass, , , loc);
	puff.drawScale = damf * 2.0;
}

function StrangeShell findSL(Pawn p) {
	local StrangeShell s;

	foreach allActors(class'StrangeShell', s) {
		if (s.pilot != none && s.pilot == p || s.gunner != none && s.gunner == p) {
			return s;
		}
	}

	return none;
}

function switchWarhead() {
	if (bCanOverride) {
		switch (warstat) {
			case WH_Auto:
				if (bDisableArmedWH) {
					warstat = WH_Disarmed;
				} else {
				 	warstat = WH_Armed;
				}
				break;
			case WH_Armed: warstat = WH_Disarmed; break;
			case WH_Disarmed: warstat = WH_Auto; break;
		}
	}
}

function disarm() {
	if (bCanOverride) {
		warstat = WH_Disarmed;
	}
}

function arm() {
	if (bCanOverride) {
		warstat = WH_Armed;
	}
}

simulated function bool armed() {
	return warstat == WH_Armed;
}

function bool autoArm() {
	if (warstat == WH_Auto) {
		warstat = WH_Armed;
		return true;
	}

	return false;
}

static function setDefaultWarhead(int stat) {
	if (stat == 1) {
		default.warstat = WH_Armed;
	} else if (stat == 2) {
		default.warstat = WH_Disarmed;
	} else {
		default.warstat = WH_Auto;
	}
}

function setRemoteRole(Actor a) {
	if (level.netMode != NM_Standalone) {
		if (role == ROLE_Authority && !dumb && a != none && !a.isA('Bot') && (level.netmode != NM_ListenServer || !u.hasViewPort(a)) && !bNoFuel) {
			remoteRole = ROLE_AutonomousProxy;
		} else {
			remoteRole = ROLE_SimulatedProxy;
		}
	}
}

auto state flying {

	simulated function beginState() {
		local Pawn p;
		local float lv;

		playAnim('open', 0.5);

		if (owner != none) {
			serverUpdate = level.timeSeconds;
			guidedRotation = rotation;

			if (level.netMode == NM_Standalone || level.netMode == NM_ListenServer && u.hasViewPort(owner)) {
				slc = spawn(class'SLController', self);
			}

		} else {
			
			if (role == ROLE_Authority) {
				dumb = true;
				warstat = WH_Armed;
				acceleration = (accelmax * 0.20) * vector(rotation);
				velocity = minspeed * vector(rotation);
				setRemoteRole(owner);
				return;
			}
		}

		setLaunchVel(speed);
		setRemoteRole(owner);
	}
  
	simulated function setLaunchVel(int startSpeed) {
		local Pawn p;
		local vector lv;
		local vector zero;
		local rotator lr;
		local float z;

		if (role == ROLE_Authority) {
			
			if (instigator != none) {
				lv = instigator.velocity;
			}
			lr = rotation;
			
		} else {

			foreach radiusActors(class'Pawn', p, vsize(pilotOffset) * 2.0) {
				if (p.weapon != none && p.weapon.isA('SLKonglauncher') && SLKonglauncher(p.weapon).launchv != zero) {
					if (lv == zero) {
						lv = SLKonglauncher(p.weapon).launchv;
						lr = p.rotation;
					} else {
						lv = zero;
					}

					SLKonglauncher(p.weapon).launchv = zero;
				}
			}
		}

		if (vsize(lv) > lvlimit) {
			z = lv.z;
			lv = normal(lv) * lvlimit;
			lv.z = fmin(z, lvlimit * 1.5);
		}

		if (lr != rot(0, 0, 0)) {
			velocity = vector(lr) * startSpeed + lv;
		}
	}
	
	simulated function zoneChange(Zoneinfo newZone) {
		local WaterRing w;
		
		if (newZone.bWaterZone != region.zone.bWaterZone) {
			w = spawn(class'WaterRing', , , , rot(16384, 0, 0));
			w.drawScale = 0.2;
			w.remoteRole = ROLE_None;
		}
    }
  
	simulated function touch(Actor other) {
		if (other.isA('SLSkyNode') && pInfo.brain != none) {
			pInfo.brain.nodeReached(SLSkyNode(other));
		} else if (other.isA('Teleporter') && lastTP == none) {
			lastTP = Teleporter(other);
			lastTPRot = rotation;
			if (rootct != none) {
				rootct.end();
			}
		} else {
			super.touch(other);
		}
	}
	
	function processTouch(actor a, vector hl) {
		local vector x, y, z;

		if (role < ROLE_Authority || a.isA('SLFuelcore') || a == pilot || a == gunner || a.isA('Carcass') || (a.isA('Pawn') && wasEjected(Pawn(a))) || (a == instigator && ((level.timeseconds - launched) < 2.0))) {
			return;
		}

		if (a.isA('Pawn') && Pawn(a).bIsPlayer && findSL(Pawn(a)) == none) {
			if (!dumb && sameTeam(Pawn(a), instigator) && gunner == none) {
				setGunner(Pawn(a));
				return;
			}

		}

		if (armed() && fromFront(a)) {
			
			detonate(hl, normal(hl - a.location));
			
		} else {
			
			if (a.isA('StrangeShell')) {

				if ((a.instigator == instigator || a.instigator == gunner) && ((level.timeseconds - StrangeShell(a).launched) < 2.0 || (level.timeseconds - launched) < 2.0)) {
					return;
				}

				rammed = StrangeShell(a);


				getAxes(a.rotation, x, y, z);
				reflect(y, a);

			} else {

				if (a == instigator) {
					u.setDamageString(ramSelf2DamStr, a, a);
				} else {
					u.setDamageString(ramDamStr, instigator, a);

				}
				a.takeDamage(vsize(velocity - a.velocity) * 0.15, instigator, hl, normal(velocity) * momentumTransfer, myDamageType);
				playSound(bounces[rand(ArrayCount(bounces))], SLOT_None, 4.0);

				takeDamage(vsize(velocity) * 0.05, instigator, hl, normal(velocity) * momentumTransfer, 'Ram');
			}
		}
	}
  
	function detonate(vector hl, vector n) {
		bDetonate = true;
		explode(hl, n);
	}
  
	function explode(vector hl, vector n) {
		local Actor detonationActor;
		local Class<StrangeWave> detClass;

		if (role < ROLE_Authority) {
			return;
		}

		if (bDetonate) {
			if (savedWeapon == none && instigator != none) {
				savedWeapon = instigator.weapon;
			}
			
			// Determine detonation effect class.
			// $ADD(SLV203)  Option to use team colored shock waves.
			detClass = Class'StrangeWave';
			if (bTeamDetClass && instigator != none && instigator.playerReplicationInfo != none) {
				if (instigator.playerReplicationInfo.team == 0) {
					detClass = Class'StrangeWaveRed';
				} else if (instigator.playerReplicationInfo.team == 1) {
					detClass = Class'StrangeWaveBlue';
				} else if (instigator.playerReplicationInfo.team == 2) {
					detClass = Class'StrangeWaveGreen';
				} else if (instigator.playerReplicationInfo.team == 3) {
					detClass = Class'StrangeWaveYellow';
				} else {
					detClass = Class'StrangeWaveWhite';
				}
			}
			
			detonationActor = spawn(detClass, , , hl + n * 16, rot(0,0,0));
			
			if (detonationActor != none && detonationActor.isA('StrangeWave')) {
				StrangeWave(detonationActor).savedWeapon = savedWeapon;
			}
			
		} else {
			if (!level.bDropDetail && level.netMode != NM_DedicatedServer) {
				spawn(class'StrangeExpl', , , location);
				spawn(class'SLBlackCloud', , , location);
			}

			specialHurtRadius(damage, 50.0, dtype, momentumTransfer, hl);
		}
		
		if (dumb) {
			destroy();
		} else {
			semiDestroy();
		}
	}
  
}

function detonate(vector hl, vector n) {
	// Ignore.
}

function specialHurtRadius(float dam, float rad, name dtype, float mom, vector hl) {
	local Actor v;
	local float dscale, dist;
	local vector dir;
	local string sds;

	foreach visibleCollidingActors(class'Actor', v, rad, hl) {
		if (v != self) {

			if (v.isA('Pawn')) {

				sds = normDamStr;
				if (dtype == 'NoUpdate' && v == instigator) {
					
					sds = normSelfDamStr;
					
				} else if (dtype == 'Ram' && v == instigator) {
					
					sds = ramSelfDamStr;
					
				} else if (dtype == 'HitWall' && v == instigator) {
					
					if (frand() < 0.5) {
						sds = hitWallDamStr;
					} else {
						sds = hitWall2DamStr;
					}
					
				} else if (dtype == 'HitSL') {

					if (rammed == none || rammed.pilot == none) {
						if (v == instigator) {
							sds = hitSLDamStr[rand(arrayCount(hitSLDamStr))];
						}
					} else {
						if (v == rammed.pilot) {
							sds = hitSLDamStr2;
						} else if (v == instigator) {
							sds = " ";
						}
					}
					
				} else if (dtype == 'Shot') {
					
					if (v == instigator) {
						sds = shotSelfDamStr;
					} else if (v == pilot || v == gunner) {
						sds = shotDamStr;
					}
					
				}
	
				u.setDamageString(sds, instigator, v);
			}

			dir = v.location - hl;
			dist = fmax(1, vsize(dir));
			dir = dir / dist; 
			dscale = 1 - fmax(0, (dist - v.collisionRadius) / rad);

			v.takeDamage(dscale * dam, instigator, v.location - 0.5 * (v.collisionHeight + v.collisionRadius) * dir, dscale * mom * dir, myDamageType);
			
		} 
	}
}

function storeEject(Pawn p) {
	local float time;

	time = level.timeSeconds;
	if (ejected[1] == none || (time - ejectTime[1] > touchLapse)) {
		ejected[1] = p;
		ejectTime[1] = time;
	} else {
		ejected[0] = p;
		ejectTime[0] = time;
	}
}

function bool wasEjected(Pawn p) {
	if (ejected[0] == p && level.timeSeconds - ejectTime[0] <= touchLapse) {
		return true;
	} else if (ejected[1] == p && level.timeSeconds - ejectTime[1] <= touchLapse) {
		return true;
	}

	return false;
}

function bool fromFront(Actor a) {
	local vector dir;
	local vector x, y, z;

	if (a != none) {
		dir = normal(a.location - location);
		getAxes(rotator(velocity), x, y, z);
		return (dir dot x) > 0.7;
	} else {
		return false;
	}
}

simulated function sparks(int dam) {
	local int isparks;

	if (level.NetMode != NM_DedicatedServer) {
		for (isparks = min(15, dam / 2); isparks > 0; isparks--) {
			spawn(class'SLSparker');
		}
	}
}

function clientDamageEffects(int dam) {
	// Create a damage effect to the hud by shaking it.
	if (pilot != none && hud != none) {
		hud.shake(dam * 0.4);
	}
	
	// Spawn damage sparks, unless we are below the desired framerate.
	if (conf.default.showDamageSparks && !level.bDropDetail) {
		sparks(dam);
	}
}

function takeDamage(int dam, Pawn i, vector hl, vector m, name dtype) {
	
	// $FIX(SLV202)  A rocket with no pilot could still cause the SLHUD to shake.
	// $FIX(SLV202)  SLHUD not shaking in netgames.
	// $ADD(SLV203)  Option to show sparks when a rocket takes damage.
	clientDamageEffects(dam);

	if (armor < 0) {
		return;
	}
	
	sparks(dam);

	if (role < ROLE_Authority) {
		return;
	}
	
	// $ADD(SLV203)  Option to make unpiloted rockets either bounce off or explode on explosions of
	//               the same team.
	//if ((pilot != none || !bNoPilotTeamDam) && i != instigator && sameTeam(i, instigator)) {
	// Causes problems with dumb fires. Maybe something to fix in SLV204, for now sorry SuB.
	if ((pilot != none || !bNoPilotTeamDam && !dumb) && i != instigator && sameTeam(i, instigator)) {
		dam *= teamDamage;
	}
	
	armor -= dam;
	
	if (armor <= 0 || armed() && fromFront(i) && frand() < 0.40) {
		if (i != none && i != pilot) {
			instigator = i;
		}

		if (armed()) {
			
			detonate(hl, vect(0.0, 0.0, 1.0));
			
		} else {
			if (dtype != 'HitWall' && dtype != 'HitSL' && dtype != 'Ram') {
				self.dtype = 'shot';
			}

			playSound(crashSound, SLOT_None, 8.0);
			explode(hl, vect(0.0, 0.0, 1.0));
		}
		
	} else {
		
		playSound(hullHits[rand(arrayCount(hullHits))], SLOT_None, 6.0);
		
	}
}

simulated function hitWall(vector hn, Actor wall) {
	local float Dam;
	local DirectionalBlast db;

	if (armed()) {
		spawnBlast(true, location, hn);
		
		if (role == ROLE_Authority) {
			if (Mover(wall) != none && Mover(wall).bDamageTriggered) {
				wall.takeDamage(damage, instigator, location, normal(velocity) * momentumTransfer, 'RocketCrash');
			}

			makeNoise(1.0);
			detonate(location + explowallout * hn, hn);
		}

		setPhysics(PHYS_None);


	} else {
		dam = reflect(hn, wall);

		if (level.netmode != NM_DedicatedServer) {
			db = spawn(Class'SLDScar', self);
			if (db != none) {
				db.drawScale *= (dam * 0.03);
				db.directionalAttach(velocity, hn);
			}
		}
	}
}

simulated function float reflect(vector hn, Actor a, optional float soften) {
	local float theta;
	local vector nv, x, y, z;
	local float relv;
	local float dam;

	if (soften == 0.0) {
		soften = 1.0;
	}

	if (a.isA('StrangeShell')) {
		dtype = 'HitSL';
	} else {
		dtype = 'HitWall';
	}


	theta = abs(normal(velocity) dot hn);
	relv = vsize(velocity - a.velocity);


	dam = ((2.0 * (theta / crushAngle) + (relv / 1000.0)) / 3.0) * 100.0 * soften;

	nv = mirrorVectorByNormal(velocity, hn) * (1.0 - 0.40 * theta);

	if (soften < 1.0) {
		velocity = (velocity * (1.0 - soften) + nv * soften);
	} else {
		velocity = nv;
	}

	if (role == ROLE_Authority) {
		takeDamage(dam + (rand(10) - 5), instigator, location, normal(velocity) * momentumTransfer, dtype);

		if (pilot != none) {
			pilot.clientSetRotation(rotator(velocity));
		}
	}

	guidedRotation = rotator(velocity);

	if (bNoFuel) {
		spin();
	}

	playSound(bounces[rand(ArrayCount(bounces))], SLOT_None, 4.0);

	return dam;
}

simulated function float arccos(float x) {
	return atan((sqrt(1 - x ^ 2)) / x);
}

function afterburn() {
	if (bNoFuel) {
		return;
	}

	if (bAfterburn) {
		bAfterburn = false;
		throttle = thrprev;
	} else {
		bAfterburn = true;
		thrprev = throttle;
		throttle = 1.5;
	}

	updateSound(0.0);
}

simulated function bool gameOver() {
	return GRI != none && GRI.gameEndedComments != "";
}

simulated function burnFuel(float delta) {
	local float fr;

	if (gameOver()) {
		return;
	}

	if (role == ROLE_Authority || u.hasViewPort(pilot)) {
		
		if (fuel > 0.0) {
			fr = baseFuelRate;
			if (bAfterburn) {
				fr *= afterburnRate;
			}
			fuel = fmax(0.0, fuel - fr * delta);
		}

		if (fuel == 0.0) {
			if (!refuel() && role == ROLE_Authority) {
				bNoFuel = true;
			}
		}
		
		if (boosterfuel > 0.0) {
			fr = fmin(1.0, vsize(velocity) / (maxSpeed - conf.default.boosterSpeed));
			boosterFuel = fmax(0.0, boosterFuel - boosterBurnRate * fr * fr * delta);
			
			if (boosterfuel == 0.0 && role == ROLE_Authority) {
				bBoost = false;
			}
		}
		
	}

	if (bNoFuel) {
		if (!bClientNoFuel) {
			updateSound(0.0);
			setRemoteRole(pilot);
			setPhysics(PHYS_Falling);
			noFuel();
			spin();
			bClientNoFuel = true;
		}
	}
}

function noFuel() {
	eject(pilot, 0, true);
	eject(gunner, 0, true);
}

simulated function spin() {
	randSpin(rand(10000) + 10000);
}

simulated function bool refuel() {
	if (pilot != none) {
		if (fc != none && fc.ammoAmount > 0) {
			if (role == ROLE_Authority) {
				fc.useAmmo(1);
			}

			playSound(refuelSound, SLOT_None, 1.0);
			fuel = conf.default.maxFuel;
			return true;
		}
	}

	return false;
}

simulated function tick(float delta) {
	local Weapon weap;
	
	if (semiDestroyed && !bDestroyed) {
		clientSemiDestroy();
	}
	if (bDestroyed) return;
	
	if (bNoFuel) {
		return;
	}

	if (dumb) {
		burnFuel(delta);
		return;
	}

	if (GRI != none && GRI.gameEndedComments != "") {
		setPhysics(PHYS_None);
		velocity = vect(0.0, 0.0, 0.0);
		return;
	}
	
	
	//if (level.netMode == NM_Client && (instigator.isA('Bot') || !u.hasViewPort(instigator))) {
	
	// Original comment by 'yrns':
	// > The role is still auto on non-viewport clients. Duh. Took me
	// > this long to figure that out. Can't test for sim proxy on the client.
	// But the condition doesn't seem right, why shouldn't a client with a viewport update its
	// proxy? And besides this condition gives accessed none warnings.
	
	// $FIX(SLV202)  Null pointer access violation in StrangeShell.tick().	
	// $FIX(SLV202)  Rocket position at client not being synchronized with server.
	
	if (level.netMode == NM_Client && (instigator == none ||
	    role == ROLE_SimulatedProxy && conf.default.syncMode == SN_ClientSync ||
	    instigator.isA('Bot') || !u.hasViewPort(instigator))) {
		updateSimProxy();
		return;

	} else if (level.netMode != NM_Standalone && remoteRole == ROLE_AutonomousProxy) {
		return;
	}

	if (bUpdatePosition) {
		clientUpdatePosition();
	}

	if (guided()) {
		if (pInfo.brain != none && pInfo.brain.bGuiding) {
			guidedRotation = pInfo.brain.getGuidedRotation();
		} else {
			guidedRotation = pilot.viewRotation;
		}
	}

	if (role == ROLE_AutonomousProxy) {
		autoMove(delta);
	} else {
		moveRocket(delta, velocity, guidedRotation);
	}
}

simulated function updateSimProxy() {
	if (updateReal(realLocation, location, 20.0)) {
		setLocation(realLocation);
		realLocation = vect(0.0, 0.0, 0.0);
	}

	if (updateReal(realVelocity, velocity, 0.0)) {
		velocity = realVelocity;
		setRotation(rotator(velocity));
		realVelocity = vect(0.0, 0.0, 0.0);
	}

	if (updateReal(realAccel, acceleration, 0.0)) {
		acceleration = realAccel;
		realAccel = vect(0.0, 0.0, 0.0);
	}
}

simulated function bool updateReal(vector v1, vector v2, float mindelta) {
	return v1 != vect(0.0, 0.0, 0.0) && vsize(v1 - v2) > mindelta;
}

function autoMove(float delta) {
	local SavedMove nm;

	if (clientBuffer < 0) {
		clientBuffer += delta;
		moveRocket(delta, velocity, guidedRotation);
		return;
	} else {
		clientBuffer = clientBuffer + delta - 80.0 / PlayerPawn(instigator).player.currentNetSpeed;
	}

	if (savedMoves == none) {
		savedMoves = getFreeMove();
		nm = savedMoves;
	} else {
		nm = savedMoves;
		while (nm.NextMove != none) {
			nm = nm.nextMove;
		}
		nm.nextMove = getFreeMove();
		nm = nm.nextMove;
	}

	nm.timeStamp = level.timeSeconds;
	nm.delta = delta;
	nm.velocity = velocity;
	nm.setRotation(guidedRotation);
	
	moveRocket(delta, velocity, guidedRotation);
	
	// $FIX(SLV202)  No more 'Received unwanted function ServerMove' errors.
	//serverMove(level.timeSeconds, location, nm.rotation.pitch, nm.rotation.yaw);
	// $CHANGE(SLV203)  No longer extends GuidedWarshell, so this method has become obsolete.
	//xServerMove(level.timeSeconds, location, nm.rotation.pitch, nm.rotation.yaw);
	serverMove(level.timeSeconds, location, nm.rotation.pitch, nm.rotation.yaw);
}

simulated function bool guided() {
	return pilot != none && !bShootMode && pilot.health > 0;
}

simulated function clientAdjustPosition (float timeStamp, float newLocX, float newLocY, float
                                         newLocZ, float newVelX, float newVelY, float newVelZ) {
    local vector newLocation;

    if (CurrentTimeStamp > timeStamp) {
        return;
    }
    
    currentTimeStamp = timeStamp;
    newLocation.X = newLocX;
    newLocation.Y = newLocY;
    newLocation.Z = newLocZ;
    velocity.X = newVelX;
    velocity.Y = newVelY;
    velocity.Z = newVelZ;

    setLocation(newLocation);

    bUpdatePosition = true;
}

simulated function clientUpdatePosition() {
	local SavedMove curm;

	bUpdatePosition = false;
	curm = savedMoves;

	while (curm != none) {
		if (curm.timeStamp <= currentTimeStamp) {
			savedMoves = curm.nextMove;
			curm.nextMove = freeMoves;
			freeMoves = curm;
			freeMoves.clear();
			curm = savedMoves;
		} else {
			moveRocket(curm.delta, velocity, curm.rotation);
			curm = curm.nextMove;
		}
	}
}

function serverMove(float ts, vector clientLoc, int pitch, int yaw) {
	local float clientErr;
	local float Delta;
	local vector dloc;
	
	if (bDestroyed) return;

	if (currentTimeStamp >= ts) {
		return;
	}

	if (currentTimeStamp > 0) {
		delta = ts - currentTimeStamp;
	}

	currentTimeStamp = ts;
	guidedRotation.pitch = pitch;
	guidedRotation.yaw = yaw;

	if (delta > 0) {
		moveRocket(delta, velocity, guidedRotation);
	}

	if (level.timeSeconds - lastUpdateTime > 0.4) {
		clientErr = 10000.0;
	} else {
		if (level.timeSeconds - lastUpdateTime > 0.07) {
			dloc = location - clientLoc;
			clientErr = dloc dot dloc;
			errorSum += clientErr;
		}
	}

	if (clientErr > 3) {
		lastUpdateTime = level.timeSeconds;

		clientAdjustPosition(ts, location.x, location.y, location.z, velocity.x, velocity.y, velocity.z);
	}
}

simulated function rotator adjustGuideRot(rotator gr, rotator rr) {
	local float dyaw;
	local float pitch;
	local rotator newr;

	if (!guided() || pilot.isA('Bot')) {
		return gr;
	}

	dyaw = (pilot.viewRotation.yaw & 65535) - (rr.yaw & 65535);
	pitch = gr.pitch;
	u.centerRotComp(dyaw);
	u.centerRotComp(pitch);
	newr = gr;
	newr.yaw = rr.yaw + clamp(dyaw, -16384, 16384);
	newr.pitch = clamp(pitch, -16384, 16384);

	return newr;
}

simulated function moveRocket(float delta, vector cv, rotator gr) {
	local vector deltav;
	local int maxroll, oldroll, rollmag;
	local float smoothRoll;
	local vector oldv, x, y, z;

	if (lastTP != none) {
		if (lastTPRot != rotation) {
			cv = vsize(velocity) * vector(rotation);
			velocity = cv;
			gr = rotation;
		}
		lastTP = none;

		startContrail();
	}

	serverUpdate = level.timeseconds;
	oldroll = (rotation.roll & 65535);
	oldv = cv;

	deltav = vector(adjustGuideRot(gr, rotator(cv))) * ((maxspeed + 700) - vsize(cv)) * delta;

	velocity = normal(cv + deltav) * vsize(cv);

	maxroll = 65536 * 0.25;
	getAxes(gr, x, y, z);
	rollmag = int(10 * (y dot (velocity - oldv)) / delta);
	if (rollmag > 0) {
		gr.roll = min(maxroll, rollmag);
	} else {
		gr.roll = max(65536 - maxroll, 65536 + rollmag);
	}

	if (gr.roll > 32768) {
		if (oldroll < 32768) {
			oldroll += 65536;
		}
	} else if (oldroll > 32768) {
		oldroll -= 65536;
	}

	smoothroll = fmin(1.0, 2.0 * delta);
	gr.roll = gr.roll * smoothroll + oldroll * (1 - smoothroll);

	setRotation(gr);

	// $FIX(SLV202)  Null pointer access violation in StrangeShell.moveRocket().
	if ((level.netmode != NM_Standalone) &&
		(level.netmode != NM_ListenServer || !u.hasViewPort(instigator)) &&
		(instigator != none && !instigator.isA('Bot')) &&
		(pilot != none || conf.default.syncMode == SN_ServerSync && bPilotEjected)) {

		autonomousPhysics(delta);
	}

	if (role == ROLE_Authority) {
		realLocation = location;
		realVelocity = velocity;
		realAccel = acceleration;
	}
}

simulated function SavedMove getFreeMove() {
	local SavedMove s;

	if (freeMoves == none) {
		return spawn(class'SavedMove');
	} else {
		s = freeMoves;
		freeMoves = freeMoves.nextMove;
		s.nextMove = none;
		return s;
	}
}

//==============================================================================
// $SLV204_BEGIN_MODIFICATIONS.

simulated function clientSemiDestroy() {
	if (!bDestroyed) {
		destroyedX();
		destroyedXX();
	}
}



function semiDestroy() {
	awaitingDestroy = true;
	setTimer(destroyDelay, false);
	destroyedX();
	destroyedXX();
	semiDestroyed = true;
}



simulated function destroyedXX() {
	setPhysics(PHYS_None);
	bHidden = true;
	setCollision(false, false, false);
	ambientSound = none;
}



simulated function destroyedX() {
	if (level.netMode != NM_DedicatedServer && !level.bDropDetail) {
		spawnWreckage();
	}

	if (role == ROLE_Authority) {
		clearRider(pilot, pInfo, dtype);
		clearRider(gunner, gInfo);
	}

	if (rootct != none) {
		rootct.end();
	}

	spawnBlast();

	if (nw != none) {
		nw.destroy();
	}

	if (u != none) {
		u.destroy();
	}

	bDestroyed = true;
	
    while (freeMoves != none) {
        freeMoves.destroy();
        freeMoves = freeMoves.nextMove;
    }

    while (savedMoves != none) {
        savedMoves.destroy();
        savedMoves = savedMoves.nextMove;
    }
}



simulated function destroyed() {
	
	if (!bDestroyed) {
		destroyedX();
	}
    
    super.destroyed();
}

// $SLV204_END_MODIFICATIONS.
//==============================================================================

simulated function spawnBlast(optional bool havev, optional vector hitl, optional vector hitn) {
	local vector x, y, z;
	local vector hl, hn;
	local Actor wall;

	if (bBlasted || level.netMode == NM_DedicatedServer) {
		return;
	}

	if (havev) {
		hl = hitl;
		hn = hitn;
	} else if (role < ROLE_Authority && level.netMode != NM_Standalone) {
		getAxes(rotation, x, y, z);
		wall = trace(hl, hn, location + x * 200, location, false);
		if (wall == none) {
			return;
		}
	}

	spawn(explosionDecal, self, , hl, rotator(hn));
	bBlasted = true;
}

simulated function spawnWreckage() {
	local int i;
	local SLWreckage w;
	local int num;

	num = 5 + rand(10);
	for (i = 0; i < num; i++) {
		w = spawn(class'SLWreckage', , , location, rotator(vrand()));
		w.eject(velocity * 1.0 + vrand() * vsize(velocity) * 1.0);
	}
}

function setGunner(Pawn g) {
	local string s;

	gunner = g;
	g.playSound(gunnerSound, SLOT_None, 4.0 * g.soundDampening);
	setupRider(g, gInfo);
	if (pilot != none) {
		s = gunnerMountStr[Rand(arrayCount(gunnerMountStr))];
	} else {
		s = gunnerMountStr2[Rand(arrayCount(gunnerMountStr2))];
	}
	broadcastMessage(u.parseKillString(s, g, pilot, true));
}

function setPilot(Pawn p, optional SLSkyNet skynet, optional SLSkyNode launchsn) {
	pilot = p;
	setupRider(p, pInfo, skynet, launchsn);
	
	if (p.isA('PlayerPawn')) {
		savedBob = PlayerPawn(p).bob;
		PlayerPawn(p).bob = 0.0;
	} else if (p.isA('Bot')) {
		throttle = 0.1;
	}

	fc = SLFuelcore(p.findInventoryType(class'SLFuelcore'));
	bShootMode = true;
	goShootMode();
}

function setupRider(Pawn p, out RiderInfo info, optional SLSkyNet skynet, optional SLSkyNode launchsn) {
	local StrangeShell sl;

	info.pawn = p;

	foreach radiusActors(class'StrangeShell', sl, 250.0) {
		if (sl != self) {
			if (sl.pilot == p) {
				sl.clearRider(p, sl.pInfo);
				// $FIX(SLV202)  Bug where the player would be in gunner mode riding an slv, as the
                //               player gets on the new slv (with alt fire) the old slv doesn't
                //               auto arm itself when the warhead mode was set to auto.
				sl.autoArm();
			}

			if (sl.gunner == p) {
				sl.clearRider(p, sl.gInfo);
			}
		}
	}

	if (p.isA('PlayerPawn')) {
		info.bOldJumpStatus = playerPawn(p).bJumpStatus;
	}

	info.bDelayedEject = false;

	if (class'SLSkyNet'.default.bBotSupport && p.isA('Bot')) {
		info.brain = spawn(class'SLBotBrain');
		info.brain.setBot(bot(p), self, skynet, launchsn);
	} else {
		info.brain = none;
	}

	info.lastLoc = vect(0.0, 0.0, 0.0);
	p.bWarping = true;
	giveJD(p);
}

simulated function Pawn getPilot() {
	return pilot;
}

simulated function Pawn getGunner() {
	return gunner;
}

function adjustGlow() {
	if (pilot != none && pilot.isA('PlayerPawn')) {
		if (bFog) {
			bFog = false;
			PlayerPawn(pilot).clientAdjustGlow(0.2, fogReset * -1);
		} else {
			bFog = true;
      		PlayerPawn(pilot).clientAdjustGlow(-0.2, fogReset);
		}
	}
}

function goShootMode() {
	if (role < ROLE_Authority) {
		return;
	}

	bShootMode = !bShootMode;

	if (pilot.isA('PlayerPawn')) {
		adjustGlow();
		pilot.clientSetRotation(rotation);
	}

	if (bShootMode) {
		setCollisionSize(shootCR, default.collisionHeight);
	} else {
		setCollisionSize(default.collisionRadius, default.collisionHeight);
	}

	if (pilot != none) {
		handleInv(false);
	}
}

function startBooster() {
	if (fc != none && fc.ammoAmount > 0) {
		fc.useAmmo(1);
		boosterFuel += conf.default.boosterFuel;
		bBoost = true;
		if (!bAfterburn) {
			afterburn();
		}
		playSound(boostSound, SLOT_None, 4.0);
	}
}

function handleInv(bool ejected) {
	if (bShootMode || ejected) {
		if (pilot.weapon != none) {
			if (pilot.weapon.isA('SLNullWeapon')) {
				pilot.weapon = none;
				pilot.deleteInventory(nw);
				pilot.pendingWeapon = lastWeapon;
				pilot.changedWeapon();
				lastWeapon = none;
			}
		}
	
	} else {
		lastWeapon = pilot.weapon;
		
		// Explicitly set lastWeapon to downWeapon state to prevent the shootmode switch bug.
		if (lastWeapon != none) {
			lastWeapon.gotoState('downWeapon');
		}
		
		if (nw == none) {
			nw = createNullWeapon();
		}

		nw.giveTo(pilot);
		pilot.pendingWeapon = nw;
		pilot.changedWeapon();
		
		if (pilot.weapon.isA('SLNullWeapon')) {
			SLNullWeapon(pilot.weapon).fakeAmmo();
		}
		
	}
}

function SLNullWeapon createNullWeapon() {
	local SLNullWeapon nw;

	nw = spawn(class'SLNullWeapon', self);
	nw.sl = self;
	return nw;
}

simulated function addHUD(PlayerPawn p) {
	local SLHUD m;

	if (u.UTPureOn(p)) {
		foreach allActors(class'SLHUD', m) {
			break;
		}
	} else {
		m = SLHUD(u.getHUDMutator(p, HUDClass.name));
	}

	if (m != none) {
		m.reset(self);
	} else {
		spawn(HUDClass, self);
	}

	addedHUD = true;
}

simulated function ctick(float delta) {
	local Rotator newRot;
	
	if (gunner != none) {
		if (slc == none && level.netMode == NM_Client && u.hasViewPort(gunner)) {
			slc = spawn(class'SLController', self);
		}
	}

	if (pilot != none) {
		if (!bWahooPlayed && vsize(velocity) >= minspeed - 10) {
			playSound(sndWahoo, SLOT_None, 3.0);
			bWahooPlayed = true;
		}

		if (!addedHUD && u.hasViewPort(pilot)) {
			addHUD(PlayerPawn(pilot));
		}
	}

	if (role == ROLE_Authority) {
		handleEjects(pilot, pInfo);
		handleEjects(gunner, gInfo);

		if (u.legacy() && region.zone.bPainZone) {
			painTimer += delta;
			if (painTimer > 1.0) {
				takeDamage(region.zone.damagePerSec, instigator, location, vect(0.0, 0.0, 0.0), 'ZonePain');
        		painTimer -= 1.0;
			}
		}
		
		// $ADD(SLV203)  Option to prevent SLV rocket pilots from carrying / picking up a flag.
		if (bPilotCantCarryFlag && pilot != none && pilot.playerReplicationInfo != none &&
		    pilot.playerReplicationInfo.hasFlag != none) {
			pilot.playerReplicationInfo.hasFlag.drop(vect(0,0,0));
		}
	}

	if (level.netMode != NM_DedicatedServer && pilot != none && pilot.isA('PlayerPawn')) {
		
		if (playerPawn(pilot).bDuck != 0) {
			PlayerPawn(pilot).bDuck = 0;
			goShootMode();
			// $FIX(SLV203)  Rocket slightly deviates from path when switching back from gunner to
			//               pilot mode.
			pilot.viewRotation = rotation;
			newRot.yaw = rotation.yaw;
			pilot.setRotation(newRot);
		}
		
		if (playerPawn(pilot).bRun != 0) {
			PlayerPawn(pilot).bRun = 0;
			
			if (conf.default.enableBoosters && fc != none && fc.ammoAmount > 0) {
				startBooster();
			}
		}

	}

	if (pilot != none) {
		adjustThrot(delta);
	}

	updateSound(delta);

	if (bAfterburn && gunner != none && PlayerPawn(gunner) != none) {
		PlayerPawn(gunner).shakeView(delta, 400.0, 7.5);
	}

	if (role >= ROLE_AutonomousProxy) {
		accelerate(delta);
	}
	
	burnFuel(delta);
	updateRiders();
}

function handleEjects(Pawn p, RiderInfo info) {
	if (p != none) {
		if (p.health <= 0) {
			clearRider(p, info, 'shot');
		} else if (teleported(info)) {
			clearRider(p, info);
		} else if (info.bDelayedEject) {
			eject(p);
		} else if (level.netMode != NM_Standalone && jumped(p, info)) {
			eject(p);
		}
	}
}

function bool teleported(RiderInfo info) {
	if (vsize(info.lastLoc) != 0 && vsize(info.pawn.location - info.lastLoc) > teleportDist && vsize(info.pawn.location - location) > teleportDist) {
		return true;
	}
	return false;
}

function bool jumped(Pawn p, RiderInfo info) {
	return p != none && p.isA('PlayerPawn') && PlayerPawn(p).bJumpStatus != info.bOldJumpStatus;
}

simulated function updateSound(float d) {
	local float pitchLvl;
	local float speed;
	
	if (ambientSound == sndDrop) {
		soundRadius = 64.0;
		soundVolume = 255;
		soundPitch = 64.0;
		return;
	}

	if (updateMarch(d)) {
		return;
	}
	
	speed = vsize(velocity);
	if (conf.default.enableBoosters) {
		if (speed > maxspeed - conf.default.boosterSpeed) {
			pitchLvl = lerp((speed - maxspeed + conf.default.boosterSpeed) / conf.default.boosterSpeed, 1.0, 1.6);
		} else {
			pitchLvl = speed / (maxspeed - conf.default.boosterSpeed);
		}
	} else {
		pitchLvl = speed / maxspeed;
	}
	soundPitch = 24 + 40 * pitchLvl;
	
	/*
	if (bBoost) {
		soundPitch += 20;
	}
	*/

	if (bNoFuel) {
		if (d == 0) {
			playSound(abFadeSound, SLOT_None, 6.0);
		}

		ambientSound = none;

	} else if (bAfterburn) {
		if (d == 0) {
			playSound(abKickSound, SLOT_None, 2.0);
		}

		ambientSound = abLoopSound;
		soundRadius = 128;
		soundVolume = 255;
			
	} else {
		if (d == 0) {
			playSound(abFadeSound, SLOT_None, 2.0);
		}

		soundRadius = 32;
		soundVolume = min(255, 150 + 55 * (vsize(velocity) / maxspeed));
		ambientSound = default.ambientSound;
	}
}

simulated function bool updateMarch(float delta) {
	local int shields;
	local Inventory i;

	if (pilot == none) {
		return false;
	}

	shields = 0;
	for (i = pilot.inventory; i != none; i = i.inventory) {
		if (i.bIsAnArmor) {
			shields += i.charge;
		}
	}
  
	if (pilot.health >= 199 && shields >= 150 && armor >= conf.default.rocketArmor) {
		if (bMarchUnder) {
			bMarchPlaying = true;
			if (marchWait != -1.0) {
				if (marchWait > 4.5) {
					playSound(sndTotalComm, SLOT_None, 1.0);
					marchWait = -1.0;
				} else {
					marchWait += delta;
				}
			}

			ambientSound = sndMarchLoop;
			soundVolume = 255;
			soundPitch = 64;
			return True;
		}
	} else {
		bMarchUnder = true;
		marchWait = 0.0;
		bMarchPlaying = false;
	}
	
	return false;
}

simulated function getTrailers() {
	if (role == ROLE_AutonomousProxy || level.netMode == NM_StandAlone) {
		findTrailers(pilot, pInfo);
		findTrailers(gunner, gInfo);
	}
}

simulated function findTrailers(Pawn p, out RiderInfo info) {
	local Actor t;
	local int i;

	i = 0;
	info.trailers[i] = none;
	
	if (p != none) {
		foreach p.childActors(class'Actor', t) {
			if (t.physics == PHYS_Trailer) {
				info.trailers[i] = t;
				i++;
			}

			if (i == arrayCount(info.trailers)) {
				break;
			}
		}
	}
}

simulated function accelerate(float delta) {
	local float max;
	local float amax;
	local float desired;
	local float speed;

	if (bAfterburn) {
		if (!conf.default.enableBoosters || bBoost) {
			max = maxSpeed;
		} else {
			max = maxSpeed - conf.default.boosterSpeed;
		}
	} else {
		max = fullspeed;
	}

	if (pilot != none) {
		max -= 100;
	}

	if (gunner != none) {
		max -= 100;
	}

	speed = vsize(velocity);
	desired = lerp(fmin(throttle, 1.0), minspeed, max);
	if (!bAfterburn && bBoost) {
		desired = fMin(maxSpeed, desired + conf.default.boosterSpeed);
	}
	amax = accelmax;

	if (desired < speed) {
		if (conf.default.enableBoosters && speed > maxSpeed - conf.default.boosterSpeed) {
			amax *= -1.0;
		} else {
			amax *= -0.5;
		}
	} else if (bBoost) {
		//amax *= lerp(boosterFuel / (conf.default.maxFuelCores * conf.default.boosterFuel), 1.5, 3.0);
		amax *= 2.0;
	} else if (bAfterburn) {
		amax *= 1.5;
	}
	
	if (abs(speed - desired) < 0.5) {
		acceleration = vect(0.0, 0.0, 0.0);
	} else {
		acceleration = vector(rotation) * fmin(abs(speed - desired) / 50.0, 1.0) * amax;
	}
}

simulated function adjustThrot(float delta) {
	if (pilot.isA('PlayerPawn') && Viewport(PlayerPawn(pilot).player) != none && !bAfterburn) {
		
		if ((PlayerPawn(pilot).bWasForward || throtDir == 1) && thlvl < 1.0) {
			thlvl = fmin(thlvl + Delta * thrate, 1.0);
			bThrotSent = false;
		
		} else if ((PlayerPawn(pilot).bWasBack || throtDir == -1) && thlvl > 0.0) {
			thlvl = fmax(thlvl - Delta * thrate, 0.0);
			bThrotSent = false;

		} else {
			if (!bThrotSent) {
				clientSendThrot(thlvl);
				bThrotSent = true;
			}
		
		}
		
		if (thlvl == 0.0 || thlvl == 1.0) {
			throtDir = 0;
		}
	
	}
}

function clientSendThrot(float thr) {
	throttle = fclamp(thr, 0.0, 1.0);
}

simulated function updateRiders() {
	local vector x, y, z, loc;
	local float d, mx, my, th;

	getAxes(rotation, x, y, z);
	loc = location;

	if (pilot != none) {
		if (pilot.isA('PlayerPawn') && PlayerPawn(pilot).isInState('feigningDeath')) {
      		playerPawn(pilot).gotoState('playerWalking');
			playerPawn(pilot).weapon = nw;
		}

		loc += x * pilotOffset.x;
		loc += y * pilotOffset.y;
		loc += z * pilotOffset.z;
		loc -= z * 10 * vsize(velocity) / maxSpeed;

		if (pilot.isA('Bot') && !bShootMode && role == ROLE_Authority) {
			pilot.clientSetRotation(rotation);
		}

		updateRider(pilot, pInfo, loc);
	}

	if (gunner != none) {
		if (pilot != none) {
			loc = pilot.location;
			mx = pilot.collisionRadius + gunner.collisionRadius;
      		my = pilot.collisionHeight + gunner.CollisionHeight;
			th = rotation.pitch / 10430.2192;
			
			if (mx * tan(th) < my) {
				d = mx / cos(th);
			} else {
				d = my / sin(th);
			}

			d += 1;
			loc += x * -d;

		} else {
			loc = location;
			loc += x * (pilotOffset.x - gunner.collisionRadius);
			loc += y * pilotOffset.y;
			loc += z * pilotOffset.z;
		}

		updateRider(gunner, gInfo, loc);
	}
}

simulated function updateRider(Pawn p, RiderInfo info, Vector loc) {
	local EPhysics phys;

	if (level.netMode == NM_Standalone || level.netMode == NM_ListenServer && u.hasViewPort(p)) {
		phys = PHYS_Walking;
	} else {
		phys = PHYS_None;
	}

	info.lastLoc = p.location;
	p.setLocation(loc);
	p.velocity = velocity;
	updateTrailers(p, info);
	p.setPhysics(phys);
}

simulated function updateTrailers(Pawn p, RiderInfo info) {
	local int i;

	for (i = 0; i < arrayCount(info.trailers) && info.trailers[i] != none; i++) {
		if (info.trailers[i] != none) {
			info.trailers[i].autonomousPhysics(0.0);
		}
	}
}

function eject(Actor a, optional int delay, optional bool bForced) {
	local vector x, y, z, loc;
	
	if (a != none) {
		if (delay > 0) {

			if ( A == pilot ) {
				pInfo.bDelayedEject = true;
			} else {
				gInfo.bDelayedEject = true;
			}
			
		} else {

			if (level.netMode == NM_Standalone) {
				resetPhysics(Pawn(a));
				a.velocity = velocity;
			}

			getAxes(a.rotation, x, y, z);
			a.velocity += y * 100;
			
			if (a == pilot) {
								
				if (!bForced) {
					ambientSound = sndDrop;
					a.playSound(sndPilotEj, SLOT_None, 4.0 * Pawn(a).soundDampening);
					
					autoArm();
				}
				
				clearRider(pilot,pInfo);
			} else {
				if (!bForced) {
					a.playSound(sndGunnerEj, SLOT_None, 4.0 * Pawn(a).soundDampening);
				}
				
				clearRider(gunner, gInfo);
			}
		}
	}
}

function clearRider(Pawn p, RiderInfo info, optional name dtype) {
	if (p == none) {
		return;
	}

	if (p == pilot) {
		handleInv(true);

		if (pilot.isA('PlayerPawn')) {
			PlayerPawn(pilot).bob = savedBob;
		}
		
		if (!bShootMode) {
			pilot.clientSetRotation(rotation);
		}

		if (bFog) {
			adjustGlow();
		}
		
		// Original comment by 'yrns':
		// > Hmm, iffy. Need to unset the owner so we can get the "real" data.
		// He means the realLocation, etc. Thats true but this causes a lot of unwanted function
		// call errors at the server. Since StrangeShell no longer extends GuidedWarshell (SLV203),
		// the replication condition for these variables have been changed to fix this issue.
		//setOwner(none);
		
		// Change to a simulated proxy, unless the server synchronizes with the client.
		if (conf.default.syncMode != SN_ServerSync) {
			remoteRole = ROLE_SimulatedProxy;
		}
		pilot = none;
		bPilotEjected = true;

	} else if (p == gunner) {
		gunner = none;
		
	} else {
		return;
	}

	if (p.isA('Bot')) {
		if (info.brain != none) {
			if (dtype != '' ) {
				info.brain.deathBy(dtype);
			}
			info.brain.destroy();
		}

		p.bCanFly = false;
		p.bCanWalk = true;
	}
	
	p.bWarping = false;
	resetPhysics(p);
	clearJD(p);
	storeEject(p);
}

function resetPhysics(Pawn a) {
	local EPhysics phys;

	if (a.region.zone.bWaterZone) {
		phys = PHYS_Swimming;
	} else {
		phys = PHYS_Falling;
	}
	
	a.setPhysics(phys);
}

function giveJD(Pawn p) {
	local SLJumpDetector jd;

	if (p.isA('PlayerPawn') && (level.netMode == NM_Standalone || (level.netMode == NM_ListenServer && u.hasViewPort(p)))) {
		jd = spawn(class'SLJumpDetector', self);
		jd.giveTo(p);
		jd.sl = self;
	}
}

function clearJD(Pawn p) {
	local Inventory i;

	if (level.netMode == NM_Standalone) {
		i = p.inventory;

		while (i != none) {
			if (i.isA('SLJumpDetector')) {
				i.destroy();
				return;
			}
			i = i.inventory;
		}
	}
}

simulated function bool sameTeam(Pawn p, Pawn p2) {
	return p != none && p2 != none && level.game.isA('TeamGamePlus') && p.bIsPlayer && p2.bIsPlayer && p.playerReplicationInfo.team == p2.playerReplicationInfo.team;
}

// $END_CLASSIFIED

defaultproperties {
	damage=1000.0
	momentumTransfer=100000
	bNetTemporary=false
	ambientGlow=78
	bUnlit=true
	bProjTarget=true
	remoteRole=ROLE_DumbProxy
	netPriority=3
	armor=100.0
	fuel=60.0
	pilotOffset=(X=0.00,Y=0.00,Z=15.00),
	minSpeed=500.0
	fullSpeed=900.0
	maxSpeed=1350.0
	accelmax=150.0
	decelRatio=0.25
	afterburnRate=8.0
	baseFuelRate=1.0
	thrate=1.2
	HUDClass=Class'SLHUD'
	twinkleClasses(0)=Class'SLTwinkle'
	twinkleClasses(1)=Class'SLTwinkle'
	twinkleClasses(2)=Class'SLTwinkle'
	damagePuffClass=Class'SLDamagePuff'
	contrailClass=Class'SLContrail'
	puffOffset=70.0
	crushAngle=0.7
	gunnerSound=Sound'SLV2Sounds.Rocket.gunnermount'
	crashSound=Sound'SLV2Sounds.Rocket.crash'
	warnSound=Sound'SLV2Sounds.Rocket.Warning'
	abFadeSound=Sound'SLV2Sounds.Rocket.afterburner_fade'
	abLoopSound=Sound'SLV2Sounds.Rocket.afterburner_loop'
	abKickSound=Sound'SLV2Sounds.Rocket.afterburner_kick'
	refuelSound=Sound'SLV2Sounds.Rocket.refuel'
	boostSound=Sound'booster_kick'
	hullHits(0)=Sound'SLV2Sounds.Rocket.hullhit'
	hullHits(1)=Sound'SLV2Sounds.Rocket.hullhit2'
	hullHits(2)=Sound'SLV2Sounds.Rocket.hullhit3'
	hullHits(3)=Sound'SLV2Sounds.Rocket.hullhit4'
	bounces(0)=Sound'SLV2Sounds.Rocket.Bounce'
	bounces(1)=Sound'SLV2Sounds.Rocket.bounce2'
	bounces(2)=Sound'SLV2Sounds.Rocket.bounce3'
	bounces(3)=Sound'SLV2Sounds.Rocket.bounce4'
	bounces(4)=Sound'SLV2Sounds.Rocket.bounce5'
	bCanOverride=True
	touchLapse=2.0
	normDamStr="%os got more holes in %oop than a horse trader's mule."
	normSelfDamStr="%k shouldn'ta bought %kpa rocket from the Rooskies."
	ramDamStr="%o lost the nuclear combat, toe to toe with %k."
	ramSelfDamStr="%k should have bought American!"
	ramSelf2DamStr="That's what %k gets for horsing around on %kpa rocket!"
	shotDamStr="%k takes out the Strangelove under %os crotch."
	shotSelfDamStr="%k learns that good aim is pretty doggone important."
	hitWallDamStr="%k needs some driving lessons."
	hitWall2DamStr="%k needs to lay off the sauce."
	hitSLDamStr(0)="%k smacked into an unmanned rocket."
	hitSLDamStr(1)="%k needs to watch out for unmanned rockets."
	hitSLDamStr(2)="%k knocked %kpa fool self into an unmanned rocket."
	hitSLDamStr(3)="%k smacked into a rogue rocket."
	hitSLDamStr(4)="%k collided with a rogue rocket."
	hitSLDamStr2="In the game of chicken, %k and %o are BOTH losers."
	shootCR=8.0
	lvlimit=600.0
	sndPilotEj=Sound'SLV2Sounds.Rocket.piloteject'
	sndGunnerEj=Sound'SLV2Sounds.Rocket.GunnerEject'
	sndDrop=Sound'SLV2Sounds.Rocket.dropalarm'
	teleportDist=250.0
	sndMarchLoop=Sound'SLV2Sounds.Rocket.march'
	sndTotalComm=Sound'SLV2Sounds.Rocket.totalcomm'
	sndWahoo=Sound'SLV2Sounds.kongl.wahoo'
	gunnerMountStr(0)="%k rides shotgun on %os Strangelove!"
	gunnerMountStr(1)="%k hops aboard %os Strangelove!"
	gunnerMountStr(2)="%k gets taken for a ride on %os Strangelove!"
	gunnerMountStr2(0)="%k leaps onto a rogue Strangelove!"
	gunnerMountStr2(1)="%k commandeers a rogue Strangelove."
	gunnerMountStr2(2)="%k tames a maverick Strangelove."
	speed=300.0
	myDamageType=SpecialDamage
	explosionDecal=Class'SLBlastmark'
	bCanTeleport=True
	lifeSpan=0.0
	ambientSound=Sound'SLV2Sounds.Rocket.engines'
	mesh=SkeletalMesh'SLV2Models.Rocket'
	soundRadius=32
	soundVolume=215
	collisionRadius=45.0
	collisionHeight=12.0
	bBounce=True
	bFixedRotationDir=True
	conf=Class'SLConfig'
	teamNum=255
}
