/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLHUDLevel
 *  $VERSION  1.50
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLHUDLevel extends SLInfo;

var SLHUDLevel next;
var int x;
var int y;
var int w;
var int h;
var float alarm;
var float realLvl;
var float dispLvl;
var string label;
var string readout;
var string qualifier;
var Font lFont;
var Font sFont;
var float bubx;
var float buby;
var() float bubbleRate;
var() float lvlRate;
var int labely;
var Font labelFont;
var int roy;
var Font roFont;
var int qx;
var int qy;
var Font qualFont;
var bool bBorder;
var bool bNoFitLabels;

function init(float alarm, string Label, string qualifier, Font lFont, Font sFont, optional SLHUDLevel nextlvl) {
	self.alarm = alarm;
	self.label = label;
	self.qualifier = qualifier;
	self.lFont = lFont;
	self.sFont = sFont;
	bubx = rand(w - 1);
	buby = rand(h);
	next = nextlvl;
}

function position(int x, int y, int w, int h) {
	self.w = w;
	self.h = h;
	self.x = x;
	self.y = y;
	labely = -1;
}

function render(SLStylus s) {
	local Canvas c;
	local float lvl;
	local int x, y;
	local int mx;
	local int i;
	local int slide;
	local int top;
	local float partial;
	local float xl, yl;
	local color color, origColor;
	local float fade;

	c = s.c;
	lvl = dispLvl;
	fade = SLHUD(Owner).screenFade;
	x = self.x;
	y = self.y + h - 1;
	origColor = c.drawColor;
	color = c.drawColor;

	if (alarmOn())	{
		color = alarmColor(color, SLHUD(owner).bAlarm);
	} else {
		if (lvl > 1.0) {
			color = s.blendc(color, alarmColor(color, true), lvl - 0.5);
			lvl = 1.0;
		}
	}

	if (lvl > 0.0) {
		top = Min(h, lvl * h);
		c.drawColor = color * fade;

		if (top >= 1) {
			s.setPos(x + 1, y);
			s.pix(w, 1);
		}

		if (top == h) {
			s.setPos(x + 1, y - (h - 1));
			s.pix(w, 1);
		}

		if (top > 1) {
			s.setPos(x, y - (min(h - 1, top) - 1));
			s.pix(w, (min(h - 1, top) - 1));
		}

		if (level.bHighDetailMode) {
			partial = (lvl * h) % 1.0;
			if (partial > 0.0) {
				c.drawColor = color * partial * fade;
				mx = x;
				if (top == 0 || top == (h - 1)) {
					mx++;
				}
				s.setPos(mx, y - top);
				s.pix(w, 1);
			}
		}
	}

	if (level.bHighDetailMode && buby < top) {
		if (buby == top) {
			c.drawColor = color * partial * fade;
		} else {
			c.drawColor = color * fade;
		}
		s.setPos(bubx + x, y - buby);
		s.pix(1, 2);
	}
	
	if (bBorder && level.bHighDetailMode) {
		c.drawColor = color * 0.25 * (1.0 - lvl) * fade;
		s.drawFrame(x, y - h + 1, w + 1, h);
	}

	c.drawColor = color * fade;
	drawLabels(s, x, y);

	c.drawColor = origColor;
}

function color alarmColor(color c1, bool on) {
	local color newc;

	newc = c1;
	
	if (!on) {
		newc.r /= 3;
		newc.g /= 3;
		newc.b /= 3;
	} else {
		newc.r *= 3;
		newc.r = min(255, newc.r);
		newc.g /= 3;
		newc.b /= 3;
	}
	
	return newc;
}

function drawLabels(SLStylus s, int x, int y) {
	local Canvas c;
	local float xl, yl;

	c = s.c;

	if (label != "" && readout != "") {
		
		if (labely == -1) {
			if (label != "") {
				c.font = lFont;
				c.textSize(label, xl, yl);
				if (xl >= w) {
					c.font = sFont;
					c.textSize(label, xl, yl);

					if (xl >= w && !bNoFitLabels) {
						label = left(label, w / xl - 2);
						labely = -1;
						return;
					}
				}

				labely = y - yl + 1;
				labelFont = c.font;

				if (readout != "") {
					if (w < 32 && !bNoFitLabels) {
						c.font = sFont;
					} else {
						c.font = lFont;
					}
	
					c.textSize(readout, xl, yl);
					roy = labely - yl + 1;
					qx = x + 1 + xl;
					roFont = c.font;
	
					if (qualifier != "") {
						c.font = lFont;
						c.textSize(readout, xl, yl);
						if (xl + qx >= w) {
							c.font = sFont;
							c.textSize(readout, xl, yl);
						}
						qy = labely - yl;
						qualFont = c.font;
					}
				}
			}
		}

		s.setPos(x + 2, labely);
		c.font = labelFont;
		s.drawText(label);

		if (readout != "") {
			s.setPos(x + 1, roy);
			c.font = roFont;
			c.textSize(readout, xl, yl);
			s.drawText(readout);

			if (qualifier != "") {
				s.setPos(x + 1 + xl, qy);
				c.font = qualFont;
				s.drawText(qualifier);			
			}
		}
	}
}

function bool alarmOn() {
	return realLvl < alarm;
}

function update(float delta, float l, coerce string s) {
	realLvl = l;
	readout = s;
	
	if (level.bHighDetailMode) {
		buby += bubbleRate * delta;
		if (buby >= h) {
			buby = 1.0;
			bubx = rand(w - 1);
		}
	}
	
	if (dispLvl == realLvl) {
		// Ignore.
	} else if (dispLvl < realLvl) {
		dispLvl = fmin(realLvl, dispLvl + lvlRate * delta);
	} else {
		dispLvl = fmax(realLvl, dispLvl - lvlRate * delta);
	}
}

function destroyed() {
	if (next != none) {
		next.destroy();
	}
}
// $END_CLASSIFIED

defaultproperties {
	bubbleRate=50.00
	lvlRate=3.00
}
