/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLAmmo
 *  $VERSION  1.61
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLAmmo extends TournamentAmmo abstract;

#exec OBJ LOAD FILE=..\System\SLV2Models.u PACKAGE=SLV2Models
#exec OBJ LOAD FILE=..\Sounds\SLV2Sounds.uax PACKAGE=SLV2Sounds
#exec OBJ LOAD FILE=..\Textures\SLV2Textures.utx PACKAGE=SLV2Textures
#exec OBJ LOAD FILE=..\Textures\SLV2Fonts.utx PACKAGE=SLV2Fonts

var() bool bClipped;
var() Class<SLClip> clipClass;
var SLClip clips[32];
var int clipnum;
var() int maxClips;
var() Class<SLSpentClip> spentClip;
var bool bStored;
var int storeAmmo;
var int storeMax;
var int configMaxAmmo;

replication {
	reliable if (bNetOwner && Role == ROLE_Authority)
		clips, clipnum;
}

function preBeginPlay() {
	super.preBeginPlay();
	
	if (configMaxAmmo > 0) {
		maxAmmo = configMaxAmmo;
	}
}

simulated function int getMaxAmmo() {
	return maxAmmo;
}

simulated function int getAmmo() {
	return ammoAmount;
}

simulated function int getClips() {
	return clipnum;
}

function SLClip reload() {
	local int i;
	local SLClip fresh;

	if (bClipped) {
		if (clipnum > 0) {
			i = clipnum - 1;
			fresh = clips[i];
			clips[i] = none;
			clipnum--;
		}
		return fresh;
	}
	return none;
}

function bool useAmmo(int num) {
	if (!bClipped) {
		return Super.UseAmmo(num);
	}
	return false;
}

function bool addAmmo(int count) {
	if (count > 0) {
		if (bClipped) {
			addNewClips(clipClass, count);
		} else {
			return super.addAmmo(count);
		}
	}
	return false;
}

function weaponSpawned() {
	ammoAmount = 0;
}

function SLClip defaultClip() {
	return newClip(clipClass);
}

function bool handlePickupQuery(Inventory i) {
	if (bClipped) {
		if (class == i.class) {
			if (clipnum < maxClips) {
				pickupMessages(i, Pawn(owner));
				i.playSound(i.pickupSound);
				addNewClips(SLAmmo(i).clipClass, SLAmmo(i).ammoAmount);
				i.setRespawn();
			}
			return true;
		}
	}
	
	if (inventory == none) {
		return false;
	}

	return super.handlePickupQuery(i);
}

function bool addClip(SLClip clip) {
	local int where;

	if (clipnum < maxClips && !clip.empty()) {
		if (clip.full()) {
			where = clipnum;
		} else {
			where = 0;
		}
	
		insertClip(clip, where);
		return true;
	}
	
	return false;
}

function insertClip(SLClip clip, int index) {
	local int i;

	for (i = clipnum - 2; i >= index; i--) {
		clips[i + 1] = clips[i];
	}
	
	clips[index] = clip;
	clipnum++;
}

function addNewClips(Class<SLClip> cc, int no) {
	local int i;

	while (i < no && clipnum < maxClips) {
		addClip(newClip(cc));
		i++;
	}
}

function SLClip newClip(Class<SLClip> cc) {
	local SLClip clip;

	clip = spawn(cc, owner);
	clip.rounds = clip.getMaxRounds();
	ammoAmount += clip.getRounds();
	clip.ammo = self;
	return clip;
}

function pickupMessages(Inventory item, Pawn p) {
	if (level.game.localLog != none) {
		level.game.localLog.logPickup(item, p);
	}
	
	if (level.game.worldLog != none ){
		level.game.worldLog.logPickup(item, p);
	}
  
	if (item.pickupMessageClass == none) {
		p.clientMessage(item.pickupMessage, 'Pickup');
	} else {
		p.receiveLocalizedMessage(item.PickupMessageClass, 0, none, none, item.Class);
	}
}

function destroyed() {
	local int i;

	if (bClipped) {
		for (i = 0; i < maxClips; i++) {
			if (clips[i] != none) {
				clips[i].destroy();
				clips[i] = none;
			}
		}
	}
	
	super.destroyed();
}
// $END_CLASSIFIED

defaultproperties {
    AmmoAmount=1
    Physics=PHYS_Falling
}