/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLNullWeapon
 *  $VERSION  1.14
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *  $CHANGES
 *
 *    2006.11.23  Def    -  Added support for UTPureRC7G by giving the null weapon a playerViewMesh.
 *                          Pure doesn't seem to allow weapons with no mesh and kicks the player if
 *                          once detected. Giving this weapon a mesh doesn't cause problems since
 *                          bHidden is set to true.
 *
 **************************************************************************************************/
class SLNullWeapon extends SLWeapon;

var StrangeShell sl;

simulated function postBeginPlay() {
	autoSwitchPriority = 10;
	inventoryGroup = 10;
	statusIcon = class'SLKonglauncher'.default.statusIcon;
	projectileClass = class'SLKonglauncher'.default.projectileClass;
	altProjectileClass = class'SLKonglauncher'.default.altProjectileClass;
	super.postBeginPlay();
}

function fakeAmmo() {
	local Weapon kong;
	
	kong = Weapon(Pawn(Owner).findInventoryType(class'SLKonglauncher'));
	if (kong != none) {
		ammoType = kong.ammoType;
	}
}

simulated function postRender(Canvas c) {
	bOwnsCrosshair = True;
}

function fire(float v) {
	if (!owner.isA('Bot') && sl != none) {
		sl.afterburn();
	}
}

function altFire(float v) {
	if (!owner.isA('Bot') && sl != none) {
		sl.eject(Pawn(owner));
	}
}

simulated function bool clientFire(float v) {
	return false;
}

simulated function renderOverlays(Canvas canvas) {
	// Ignore.
}

state clientActive {
  
	simulated function beginState() {
		// Ignore.
	}

}

state idle {
  
	function bool putDown() {
		return global.putDown();
	}

}

auto state active {

	function bool putDown() {
		return global.putDown();
	}

	function fire(float v) {
		global.fire(v);
	}

	function altFire(float v) {
		global.altFire(v);
	}
  
	function beginState() {
		// Ignore.
	}
  
	function endState() {
	}
	
begin:
	// Ignore.
  
}

state downWeapon {
  
	function bool putDown() {
		return global.putDown();
	}
  
}

simulated function setHand(float hand) {
	// Ignore.
}

function bool putDown() {
	if (sl != none) {
		sl.switchWarhead();
	}
	bChangeWeapon = false;
	Pawn(Owner).pendingWeapon = None;
	return true;
}

simulated function playSelect() {
	// Ignore.
}

simulated function tweenToStill() {
	// Ignore.
}

function bringUp() {
	// Ignore.
}

// $FIX(SLV203)  Accessed null class context in SLNullWeapon.maxOut.
function maxOut() {
	// Ignore.
}
// $END_CLASSIFIED

defaultproperties {
	bCanThrow=False
	ItemName="'"
	playerViewMesh=SkeletalMesh'SLV2Models.slview'
	bHidden=True
	bCollideActors=False
}
