/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLSidearm
 *  $VERSION  1.12
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLSidearm extends SLWeapon;

#EXEC TEXTURE IMPORT NAME=Sidearm_xl FILE=Resources\Sidearm_xl.pcx FLAGS=2 MIPS=OFF

var() string deathStr[5];

simulated function postBeginPlay() {
	// $ADD(SLV203)  Configurable default fire mode of the yellow jacket.
	switch (class'SLConfig'.default.defSidearmMode) {
		case FM_Burst:    fireMode = FM_Burst; break;
		case FM_Auto:     fireMode = FM_Auto; break;
		default:          fireMode = FM_Single;
	}
	
	super.postBeginPlay();
}

simulated function tweenToStill() {
	tweenAnim('Still', 0.30);
	if (slave != none) {
		slave.tweenToStill();
	}
}

function processTraceHit(Actor a, Vector hl, Vector hn, Vector x, Vector y, Vector z) {
	default.deathMessage = u.parseKillString(deathStr[rand(5)], owner, a, false);
	super.processTraceHit(a, hl, hn, x, y, z);
}
// $END_CLASSIFIED

defaultproperties {
    deathStr(0)="%o was taken out by %ks %w."
    deathStr(1)="%o got capped by %ks %w."
    deathStr(2)="%o had a big taste of %ks %w."
    deathStr(3)="%o was reduced to a stain on the ground by %ks %w."
    deathStr(4)="%o was splattered against the floor by %ks %w."
    bFirearm=True
    hitdam=7
    bAltSwitchMode=True
    rpm=900
    bAkimbo=True
    slaveName="dual Yellow Jackets"
    leftMesh="SLV2Models.sarmview"
    assFire=(aname=Fire,arate=3.00,tween=0.10,Sound=Sound'SLV2Sounds.Sidearm.Fire',Volume=1.00,Slot=SLOT_None,bSkipOwner=True),
    assAltFire=(aname=AltFire,arate=4.00,tween=0.10,Sound=Sound'SLV2Sounds.Sidearm.Fire',Volume=1.00,Slot=SLOT_None,bSkipOwner=True),
    assSelect=(aname=Select,arate=1.00,tween=0.00,Sound=Sound'SLV2Sounds.Weapon.Reload',Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
    assReload=(aname=Reload,arate=1.00,tween=0.00,Sound=Sound'SLV2Sounds.Weapon.Reload',Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
    xhair=Texture'SLV2Textures.xhair.Sidearm'
    xhair_xl=Texture'Sidearm_xl'
    paSlide=1.75
    paFlashY=0.045
    paFlashO=0.006
    mimicPriorityOf=enforcer
    WeaponDescription="Hmm."
    AmmoName=Class'SLSidearmAmmo'
    bInstantHit=True
    FiringSpeed=2.00
    FireOffset=(X=4.00,Y=-5.00,Z=4.00),
    MyDamageType=shot
    RefireRate=0.60
    DeathMessage="%o was taken out by %ks %w."
    bDrawMuzzleFlash=True
    MuzzleScale=1.00
    FlashY=0.275
    FlashO=0.04
    FlashLength=0.04
    FlashS=256
    MFTexture=Texture'SLV2Textures.muzf.mf1'
    InventoryGroup=2
    PickupMessage="You picked up an RS1 Yellow Jacket."
    ItemName="Yellow Jacket"
    PlayerViewOffset=(X=9.00,Y=-8.00,Z=-6.50),
    PlayerViewMesh=SkeletalMesh'SLV2Models.rsarmview'
    PickupViewMesh=SkeletalMesh'SLV2Models.sarmpickup'
    ThirdPersonMesh=SkeletalMesh'SLV2Models.sarmhand'
    StatusIcon=Texture'SLV2Textures.SLHUD.sidearmicon'
    Icon=Texture'SLV2Textures.SLHUD.sidearmicon'
    Rotation=(Pitch=0,Yaw=0,Roll=-4600),
    Mesh=SkeletalMesh'SLV2Models.sarmpickup'
    CollisionRadius=16.00
    CollisionHeight=4.00
    Mass=15.00
}
