/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    StrangeArena
 *  $VERSION  1.64
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *  $CHANGES
 *    2007.04.07  Def    -  Removed BETA expiration code.
 *
 *    2007.03.22  Def    -  Added code so that certain actors wont be replaced by the
 *                          checkReplacement() function. These actors can be specified in the
 *                          configuration file. See SLConfig.excludedActors.
 *
 **************************************************************************************************/
class StrangeArena extends Arena;

var SLConfig conf;            // SLV Configuration.



function preBeginPlay() {
	conf = spawn(class'SLConfig');
	if (!conf.noYJ) {
		defaultWeapon = Class'SLSidearm';
	}
	// $CHANGE(SLV203)  The SLKillMsgFix mutator is automatically loaded if useKillMsgFix is set to true.
	if (conf.useKillMsgFix) {
		addMutator(spawn(class'SLKillMsgFix'));
	}
	if (conf.pilotCarryMode == CM_Normal) {
		spawn(class'SLFlagSN');
	}
	spawn(class'SLProjectileSN');
	super.preBeginPlay();
}



function bool alwaysKeep(Actor other) {
	if (other.isA(weaponName) || !conf.noYJ && other.isA('SLSidearm')) {
		if (other.isA('SLWeapon')) {
			SLWeapon(other).maxOut();
		}
		return true;
	}

	if (Other.isA('StrangeAmmo')) {
		return true;
	}

	if (!conf.noYJ && other.isA('SLSidearmAmmo')) {
		Ammo(other).ammoAmount = Ammo(other).maxAmmo;
		return true;
	}
	
	if (other.isA(AmmoName)) {
		return true;
	}

	if (nextMutator != none) {
		return nextMutator.alwaysKeep(other);
	}
	return false;
}



function bool checkReplacement(Actor other, out byte bSuperRelevant) {
	// $ADD(SLV203)  Weapons can be excluded from being replaced by the StrangeArena mutator.
	if ((other.isA('Weapon') || other.isA('Ammo')) && dontReplaceActor(other)) {
		return true;
	}
		
	if (other.isA('Weapon')) {
		if (other.isA('SLNullWeapon') || !conf.noYJ && other.isA('SLSidearm')) {
			return true;
		}
		if (!conf.noYJ && other.isA('Enforcer')) {
			replaceWith(other, string(class'SLUtil'.default.package) $ "." $ "SLSidearm");
			return false;
		}
		if (weaponString != "" && !other.isA(weaponName)) {
			level.game.bCoopWeaponMode = false;
			replaceWith(other, string(class'SLUtil'.default.package) $ "." $ weaponString);
			return false;
		}
	}

	if (other.isA('Ammo')) {
		if (!conf.noYJ && (other.isA('Miniammo') || other.isA('BulletBox'))) {
			replaceWith(other, string(class'SLUtil'.default.package) $ "." $ "SLSidearmAmmo");
			return false;
		} else if (!other.isA(ammoName)) {
			replaceWith(other, string(class'SLUtil'.default.package) $ "." $ ammoString);
			return False;
		}
	}

	bSuperRelevant = 0;
	return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks whether the give actor should NOT be replaced by checkReplacement(). The 
 *                actors that shouldn't be replaced are specified by the excludedActors variable in
 *                SLConfig.
 *  $PARAM        a  The actor for which has to be checked if it is not to be replaced.
 *  $REQUIRE      a != none
 *  $RETURN       True if the actor shouln't be replaced, false if it should.
 *  $SINCE        SLV203
 *
 **************************************************************************************************/
function bool dontReplaceActor(Actor a) {
	local bool bNoReplace;
	local int index;
	local string className;
	
	className = string(a.class);
	
	while (!bNoReplace && index < arrayCount(conf.excludedActors) && conf.excludedActors[index] != "") {
		
		if (right(conf.excludedActors[index], 1) == "*" &&
		    len(className) >= len(conf.excludedActors[index]) - 1 &&
		    left(className, len(conf.excludedActors[index]) - 1) ~=
		      left(conf.excludedActors[index], len(conf.excludedActors[index]) - 1)) {
			bNoReplace = true;
		} else if (conf.excludedActors[index] ~= className) {
			bNoReplace = true;
		}
		
		index++;
	}
	
	return bNoReplace;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called whenever a player restarts. Gives the player an initial amount of fuel
 *                cores, as specified by SLConfig.initialFuelCores
 *  $PARAM        other  The pawn that has restarted.
 *  $REQUIRE      other != none
 *  $SINCE        SLV203
 *
 **************************************************************************************************/
function modifyPlayer(Pawn other) {
	local SLFuelCore fc;
	
	if (nextMutator != none) {
		nextMutator.modifyPlayer(other);
	}
	
	if (conf.initialFuelCores > 0) {
		// $ADD(SLV203) Option to give players a certain amount of fuel cores each time they respawn.
		fc = spawn(class'SLFuelCore', other);
		fc.giveTo(other);
		fc.ammoAmount = conf.initialFuelCores;
	}
}

// $END_CLASSIFIED

defaultproperties {
    weaponName=SLKonglauncher
    ammoName=SLFuelcore
    weaponString="SLKonglauncher"
    ammoString="SLFuelcore"
    defaultWeapon=Class'SLKonglauncher'
}
