/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLUtil
 *  $VERSION  1.31
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *  $CHANGES
 *    2006.11.15  Def    -  Added support for the SLV Kill detection checks in the SLKillMsgFix
 *                          class.
 *
 *    2007.04.07  Def    -  Removed BETA expiration code.
 *
 *    2007.04.08  Def    -  Added bPrivate variable to indicate a private build.
 *
 *    2007.04.20  Def    -  Optimized the SLV Kill detection support by only locating the mutator
 *                          in the first instance of this class. If the mutator is located it will
 *                          be stored in the default value of killMsgFixMutator.
 *
 **************************************************************************************************/
class SLUtil extends SLInfo;

var() name package;
var() string version;
var() string vstr;
var() string when;
var() int build;
var() bool bLeftMeshesLoaded;
var() string leftWeapons[2];
var() bool bPrivate;
var bool searchedForKMFMutator;
var SLKillMsgFix killMsgFixMutator;

function postBeginPlay() {
	if (!searchedForKMFMutator) {
		default.searchedForKMFMutator = true;
		searchedForKMFMutator = true;
		
		foreach allActors(class'SLKillMsgFix', killMsgFixMutator) {
			break;
		}
		if (killMsgFixMutator != none) {
			default.killMsgFixMutator = self.killMsgFixMutator;
		}
	}
}

function setDamageString(string s, Actor k, Actor o) {
	level.game.specialDamageString = parseKillString(s, k, o);
	
	// This might be a potential SLV kill.
	if (class'SLKillMsgFix'.default.bEnableSLVCheck && killMsgFixMutator != none && Pawn(k) != none) {
		killMsgFixMutator.registerSLVKill(Pawn(k));
	}		
}

function string parseKillString(string s, Actor killer, Actor other, optional bool bNames) {
	
	if (killer != none && killer.isA('Pawn') && Pawn(killer).bIsFemale) {
		replace(s, "%kpa", "her");
		replace(s, "%kop", "her");
	} else {
		replace(s, "%kpa", "his");
		replace(s, "%kop", "him");
	}
	
	if (other != none && other.isA('Pawn') && Pawn(other).bIsFemale) {
		replace(s, "%opa", "her");
		replace(s, "%oop", "her");
	} else {
		replace(s, "%opa", "his");
		replace(s, "%oop", "him");
	}
	
	if (bNames) {
		replace(s, "%k", sname(killer));
		replace(s, "%ks", sname(killer) $ poss(sname(killer)));
	} else {
		replace(s, "%ks", "%k" $ poss(sname(killer)));
	}
	
	replace(s, "%os", sname(other) $ poss(sname(other)));
	return s;
}

function string poss(string name) {
	if (right(name, 1) == "s") {
		return "'";
	} else {
		return "'s";
	}
}

function replace(out string str, string in, string out) {
	local int i;

	i = instr(str, in);
	if (i != -1) {
		str = left(str, i) $ out $ right(str, len(str) - (i + len(in)));
	}
}

function centerRotComp(out float comp) {
	if (comp < -32768) {
		comp += 65536;
	} else if (comp > 32768) {
		comp -= 65536;
	}
}

function bool hasViewPort(Actor a) {
	return (a != none && a.isA('PlayerPawn') && PlayerPawn(a).player != none && Viewport(PlayerPawn(a).player) != none);
}

function bool legacy() {
	return (!(level.game.isA('Strangewar') || level.game.isA('Strangerace')));
}

static function string getVersionString() {
	return default.vstr $ "." $ default.build $ " built on " $ default.when;
}

function loadLeftMeshes() {
	local int i;
	local class<SLWeapon> c;
	local SkeletalMesh m;

	if (!default.bLeftMeshesLoaded) {
		for (i = 0; i < arrayCount(leftWeapons); i++) {
			if (leftWeapons[i] != "") {
				c = class<SLWeapon>(dynamicLoadObject(string(default.Package) $ "." $ leftWeapons[i], class'Class'));
				if (c != none) {
					m = SkeletalMesh(dynamicLoadObject(c.default.leftMesh, class'SkeletalMesh'));
				} else {
					log(string(self) @ "loadLeftMeshes(): couldn't load class: " $ string(default.package) $ "." $ leftWeapons[i]);
				}
			}
			
		}
		default.bLeftMeshesLoaded = True;
	}
}

function Mutator getHUDMutator(PlayerPawn p, name c, optional bool delete) {
	local Mutator m;
	local Mutator last;

	if (p != none && p.myHUD != none) {
		m = p.myHUD.HUDMutator;
		while (m != none) {
			if (m.isA(c)) {
				if (delete) {
					if (m == p.myHUD.HUDMutator) {
						p.myHUD.HUDMutator = m.nextHUDMutator;
					} else {
						last.nextHUDMutator = m.nextHUDMutator;
					}
					m.nextHUDMutator = none;
				}
				return m;
			}
			last = m;
			m = m.nextHUDMutator;
		}
	}
	return none;
}

function string dumpHUDChain(PlayerPawn p) {
	local Mutator m;
	local string buf;

	if (p != none && p.myHUD != none) {
		buf = "HUD: " $ sname(p.myHUD);
		m = p.myHUD.HUDMutator;
		while (m != none) {
			buf = buf $ " > " $ sname(m);
			m = m.nextHUDMutator;
		}
	} else {
		buf = "No PlayerPawn or myHUD.";
	}
	return buf;
}

function bool UTPureOn(PlayerPawn p) {
	if (hasViewPort(p)) {
		if (inStr(string(p.player.console.class), "UTPure") >= 0) {
			return true;
		}
	}
	return false;
}
// $END_CLASSIFIED

defaultproperties {
	package=SLV205
	version="205"
	vstr="RELEASE-2.05"
	when="Tue Aug 31 22:12:38 2010"
	bPrivate=false
	build=79
	leftWeapons(0)="SLSidearm"
	leftWeapons(1)="SLKonglauncher"
}
