/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLWreckage
 *  $VERSION  1.30
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED
class SLWreckage extends SLEjectedBrass;

var Mesh chunks[8];
var bool smokin;
var() Class<Effects> smokeClass;

simulated function postBeginPlay() {
	mesh = chunks[rand(8)];
	drawScale = 1.0 + frand() * 2.0;
	smokin = true;
	SetTimer(0.01, false);
}

simulated function hitWall(vector hn, Actor wall) {
	local DirectionalBlast db;

	super.hitWall(hn, wall);
	
	if (!level.bDropDetail && level.netMode != NM_DedicatedServer) {
		db = spawn(class'SLDScar', self);
		if (db != none) {
			db.drawScale = drawScale * 0.25 + 0.25 * frand();
			db.directionalAttach(velocity, hn);
		}
	}
	
}

simulated function landed(vector hn) {
	smokin = false;
	setTimer(0.0, false);
	super.landed(hn);
}

simulated function timer() {
	local Effects puff;

	if (smokin) {
		if (!level.bDropDetail) {
			puff = spawn(smokeClass, , , location + vrand() * 15);
			puff.drawScale += lifeSpan / default.lifeSpan - 1.25 * frand();
		}
		
		if (vsize(velocity) != 0) {
			setTimer(fmax(0.9 - vsize(velocity) / 600.0, 0.01), false);
		}
		
	} else {
		super.timer();
	}
}
// $END_CLASSIFIED

defaultproperties {
	chunks(0)=SkeletalMesh'SLV2Models.Chunk1'
	chunks(1)=SkeletalMesh'SLV2Models.Chunk2'
	chunks(2)=SkeletalMesh'SLV2Models.Chunk3'
	chunks(3)=SkeletalMesh'SLV2Models.Chunk4'
	chunks(4)=SkeletalMesh'SLV2Models.chunk5'
	chunks(5)=SkeletalMesh'SLV2Models.chunk6'
	chunks(6)=SkeletalMesh'SLV2Models.chunk7'
	chunks(7)=SkeletalMesh'SLV2Models.chunk8'
	smokeClass=Class'SLDamagePuff'
	bLevel=False
	sinkfudge=40.00
	HitSounds(0)=Sound'SLV2Sounds.Rocket.Bounce'
	HitSounds(1)=Sound'SLV2Sounds.Rocket.bounce2'
	HitSounds(2)=Sound'SLV2Sounds.Rocket.bounce3'
	HitSounds(3)=Sound'SLV2Sounds.Rocket.bounce4'
	HitSounds(4)=Sound'SLV2Sounds.Rocket.bounce5'
	Num=5
	RemoteRole=ROLE_None
	LifeSpan=20.00
	Mesh=SkeletalMesh'SLV2Models.Chunk1'
	CollisionRadius=6.00
	CollisionHeight=6.00
}
